@@ -48,7 +48,7 @@ import numpy as N
 
 from ..compat import crange, cnext, CStringIO
 from .. import utils
-from .. import document
+from .. import datasets
 from .. import qtall as qt4
 from .import base
 
@@ -330,14 +330,14 @@ class DescriptorPart(object):
 
                 # create the dataset
                 if self.datatype == 'float':
-                    ds = document.Dataset( data = vals, serr = sym,
+                    ds = datasets.Dataset( data = vals, serr = sym,
                                            nerr = neg, perr = pos,
                                            linked = linkedfile )
                 elif self.datatype == 'date':
-                    ds = document.DatasetDateTime( data=vals,
+                    ds = datasets.DatasetDateTime( data=vals,
                                                    linked=linkedfile )
                 elif self.datatype == 'string':
-                    ds = document.DatasetText( data=vals,
+                    ds = datasets.DatasetText( data=vals,
                                                linked = linkedfile )
                 else:
                     raise RuntimeError("Invalid data type")
@@ -676,8 +676,8 @@ class SimpleRead2D(object):
             g = [float(v) for v in cols[1:]]
         except ValueError:
             raise Read2DError("%s is not a list of numerical values" % attr)
-        if not utils.checkAscending(g):
-            raise Read2DError("%s values are not ascending" % attr)
+        if utils.checkOrder(g) == 0:
+            raise Read2DError("%s are not ascending or descending" % attr)
         setattr(self, attr, g)
 
     def _paramInvertRows(self, cols):
@@ -766,8 +766,11 @@ class SimpleRead2D(object):
                 raise Read2DError(
                     "x|y grid|cent are incompatible with gridatedge")
 
-            self.xcent = N.array(rows[-1])
-            self.ycent = N.array([r[0] for r in rows[-2::-1]])
+            if len(rows) > 0:
+                self.xcent = N.array(rows[-1])
+            else:
+                self.xcent = N.array([])
+            self.ycent = N.array([r[0] for r in rows[:-1]])
 
             # chop out grid
             rows = [ r[1:] for r in rows[:-1] ]
@@ -787,28 +790,31 @@ class SimpleRead2D(object):
 
         if self.params.invertcols:
             self.data = self.data[:,::-1]
-            if self.xedge is not None:
-                self.xedge = self.xedge[::-1]
-            if self.xcent is not None:
-                self.xcent = self.xcent[::-1]
         if self.params.invertrows:
             self.data = self.data[::-1,:]
-            if self.yedge is not None:
-                self.yedge = self.yedge[::-1]
-            if self.ycent is not None:
-                self.ycent = self.ycent[::-1]
 
         # transpose matrix if requested
         if self.params.transpose:
             self.data = N.transpose(self.data).copy()
             self.xedge, self.yedge = self.xedge, self.yedge
 
-        # sanity checks
-        for a in 'xedge', 'yedge', 'xcent', 'ycent':
-            v = getattr(self, a)
-            if v is not None and not utils.checkAscending(v):
-                raise Read2DError("%s must be ascending" % a)
+        # check orders of coords - flip if wrong
+        for attr in 'xedge', 'xcent', 'yedge', 'ycent':
+            v = getattr(self, attr)
+            if v is not None:
+                order = utils.checkOrder(v)
+                if order == 0:
+                    raise Read2DError(
+                        '%s must be ascending or descending' % attr)
+                elif order == -1:
+                    # flip direction of coord and data
+                    setattr(self, attr, v[::-1])
+                    if attr[0] == 'x':
+                        self.data = self.data[:,::-1]
+                    else:
+                        self.data = self.data[::-1,:]
 
+        # more sanity checks
         if ( (self.xedge is not None and
               len(self.xedge) != self.data.shape[1]+1) or
              (self.yedge is not None and
@@ -825,7 +831,7 @@ class SimpleRead2D(object):
         """Set the data in the output dict out
         """
 
-        ds = document.Dataset2D(self.data,
+        ds = datasets.Dataset2D(self.data,
                                 xrange=self.params.xrange,
                                 yrange=self.params.yrange,
                                 xedge=self.xedge, yedge=self.yedge,
@@ -835,3 +841,145 @@ class SimpleRead2D(object):
 
         fullname = self.params.prefix + self.name + self.params.suffix
         out[fullname] = ds
+
+#####################################################################
+# n-dimensional data reading
+
+class ReadNDError(base.ImportingError):
+    pass
+
+class SimpleReadND(object):
+    def __init__(self, name, params):
+        """Read dataset with name given.
+        params is a ImportParamsND object
+        """
+        self.name = name
+        self.params = params.copy()
+
+    ####################################################################
+
+    def _paramTranspose(self, cols):
+        self.params.transpose = True
+
+    def _paramShape(self, cols):
+        sizes = [int(x) for x in cols[1:]]
+        self.params.shape = tuple(sizes)
+
+    ####################################################################
+
+    def readData(self, stream):
+        """Read data from stream given
+
+        stream consists of:
+        optional:
+         transpose    - swap rows and columns
+         shape size1 size2 ... - give dimensions of dataset
+        then:
+         Matrix of columns and rows, separated by line endings.
+         A single line is a single dimension
+         Higher orders are given by using increasing numbers of
+         separating newlines
+        """
+
+        settings = {
+            'transpose': self._paramTranspose,
+            'shape': self._paramShape,
+            }
+
+        vals = []
+
+        # keep track of where we are in terms of index
+        dimstack = []
+        dimidx = 0
+
+        # loop over lines
+        while stream.newLine():
+            cols = stream.allColumns()
+
+            # check to see whether parameter is set
+            if len(cols) > 0:
+                c = cols[0].lower()
+                if c in settings:
+                    settings[c](cols)
+                    stream.flushLine()
+                    continue
+
+            # read columns
+            line = []
+            while True:
+                v = stream.nextColumn()
+                if v is None:
+                    break
+                try:
+                    line.append(float(v))
+                except ValueError:
+                    raise ReadNDError("Could not interpret number '%s'" % v)
+
+            if len(line) > 0:
+                # previous blank lines
+                if dimidx != len(dimstack):
+                    # insert new dimensions, if required
+                    while dimidx < 0:
+                        dimstack.insert(0, 0)
+                        dimidx += 1
+                        vals = [vals]
+
+                    # move to next value at current level
+                    dimstack[dimidx] += 1
+                    for i in crange(dimidx+1, len(dimstack)):
+                        dimstack[i] = 0
+                    dimidx = len(dimstack)
+
+                # lookup correct place in hierarchy and append
+                v = vals
+                for s in dimstack:
+                    while s >= len(v):
+                        v.append([])
+                    v = v[s]
+                v.append(N.array(line, dtype=N.float64))
+            else:
+                if len(vals) > 0:
+                    dimidx -= 1
+
+        if self.params.shape is not None:
+            # flatten so we can reshape properly later (this is to
+            # allow free form input with the shape option)
+            fdata = []
+            def flatten(d):
+                for x in d:
+                    if isinstance(x, list):
+                        flatten(x)
+                    else:
+                        fdata.append(x)
+            flatten(vals)
+            vals = N.hstack((fdata))
+
+        try:
+            self.data = N.array(vals, dtype=N.float64)
+        except ValueError:
+            raise ReadNDError("Could not convert data to n-D matrix")
+
+        # obvious check
+        if self.data.ndim < 1:
+            raise ReadNDError("Needs at least a 1D dataset")
+
+        # for 1d data, an extra dimension is sometimes added
+        if self.data.ndim == 2 and self.data.shape[0] == 1:
+            self.data = self.data.reshape(self.data.shape[-1])
+
+        # reshape if requested
+        if self.params.shape is not None:
+            self.data = self.data.reshape(self.params.shape)
+
+        # transpose matrix if requested
+        if self.params.transpose:
+            self.data = N.transpose(self.data).copy()
+
+    def setOutput(self, out, linkedfile=None):
+        """Set the data in the output dict out
+        """
+
+        ds = datasets.DatasetND(self.data)
+        ds.linked = linkedfile
+        fullname = self.params.prefix + self.name + self.params.suffix
+        out[fullname] = ds
