@@ -19,12 +19,12 @@
 """Dataset creation dialog for 2d data."""
 
 from __future__ import division
-from ..compat import crange, citems
+from ..compat import crange, citems, cstr
 from .. import qtall as qt4
 from .. import utils
 from .. import document
+from .. import datasets
 from .veuszdialog import VeuszDialog
-from . import dataeditdialog
 
 def _(text, disambiguation=None, context="DataCreate2D"):
     """Translate text."""
@@ -96,7 +96,7 @@ class DataCreate2DDialog(VeuszDialog):
             if ds.datatype == 'numeric':
                 datasets[ds.dimensions-1].append(name)
         datasets[0].sort()
-        datasets[1].sort()        
+        datasets[1].sort()
 
         # make sure names are escaped if they have funny characters
         self.escapeDatasets(datasets[0])
@@ -132,19 +132,19 @@ class DataCreate2DDialog(VeuszDialog):
         self.namecombo.setEditText(dsname)
         self.linkcheckbox.setChecked(True)
 
-        if isinstance(ds, document.Dataset2DXYZExpression):
+        if isinstance(ds, datasets.Dataset2DXYZExpression):
             self.fromxyzexpr.click()
             self.xexprcombo.setEditText(ds.exprx)
             self.yexprcombo.setEditText(ds.expry)
             self.zexprcombo.setEditText(ds.exprz)
 
-        elif isinstance(ds, document.Dataset2DExpression):
+        elif isinstance(ds, datasets.Dataset2DExpression):
             self.from2dexpr.click()
             self.xexprcombo.clearEditText()
             self.yexprcombo.clearEditText()
             self.zexprcombo.setEditText(ds.expr)
 
-        elif isinstance(ds, document.Dataset2DXYFunc):
+        elif isinstance(ds, datasets.Dataset2DXYFunc):
             self.fromxyfunc.click()
             self.xexprcombo.setEditText('%g:%g:%g' % tuple(ds.xstep))
             self.yexprcombo.setEditText('%g:%g:%g' % tuple(ds.ystep))
@@ -164,7 +164,7 @@ class DataCreate2DDialog(VeuszDialog):
         disable = False
         # need name and zexpr
         disable = disable or not text['name'] or not text['zexpr']
-        
+
         if self.mode == 'xyzexpr':
             # need x and yexpr
             disable = disable or not text['xexpr'] or not text['yexpr']
@@ -172,7 +172,7 @@ class DataCreate2DDialog(VeuszDialog):
         elif self.mode == '2dexpr':
             # nothing else
             pass
-                
+
         elif self.mode == 'xyfunc':
             # need x and yexpr in special step format min:max:step
             disable = disable or ( checkGetStep(text['xexpr']) is None or
@@ -180,7 +180,7 @@ class DataCreate2DDialog(VeuszDialog):
 
         # finally check button
         self.createbutton.setDisabled(disable)
-        
+
     def createButtonClickedSlot(self):
         """Create button pressed."""
 
@@ -189,29 +189,30 @@ class DataCreate2DDialog(VeuszDialog):
             text[name] = getattr(self, name+'combo').currentText().strip()
 
         link = self.linkcheckbox.checkState() == qt4.Qt.Checked
-        if self.mode == 'xyzexpr':
-            # build operation
-            op = document.OperationDataset2DCreateExpressionXYZ(
-                text['name'],
-                text['xexpr'], text['yexpr'], text['zexpr'],
-                link)
 
-        elif self.mode == '2dexpr': 
-            op = document.OperationDataset2DCreateExpression(
-                text['name'], text['zexpr'], link)
-
-        elif self.mode == 'xyfunc':
-            xstep = checkGetStep(text['xexpr'])
-            ystep = checkGetStep(text['yexpr'])
-
-            # build operation
-            op = document.OperationDataset2DXYFunc(
-                text['name'],
-                xstep, ystep,
-                text['zexpr'], link)
-
-        # apply operation, catching evaluation errors
+        # create and apply operation, catching evaluation errors
         try:
+            if self.mode == 'xyzexpr':
+                # build operation
+                op = document.OperationDataset2DCreateExpressionXYZ(
+                    text['name'],
+                    text['xexpr'], text['yexpr'], text['zexpr'],
+                    link)
+
+            elif self.mode == '2dexpr':
+                op = document.OperationDataset2DCreateExpression(
+                    text['name'], text['zexpr'], link)
+
+            elif self.mode == 'xyfunc':
+                xstep = checkGetStep(text['xexpr'])
+                ystep = checkGetStep(text['yexpr'])
+
+                # build operation
+                op = document.OperationDataset2DXYFunc(
+                    text['name'],
+                    xstep, ystep,
+                    text['zexpr'], link)
+
             # check expression is okay
             op.validateExpression(self.document)
 
@@ -219,21 +220,22 @@ class DataCreate2DDialog(VeuszDialog):
             self.document.applyOperation(op)
             # forces an evaluation
             self.document.data[text['name']].data
+
         except (document.CreateDatasetException,
-                document.DatasetException):
+                datasets.DatasetException) as e:
+
             msg = _("Failed to create dataset '%s'") % text['name']
+            s = cstr(e)
+            if s:
+                msg += ' (%s)' % s
         else:
             msg = _("Created dataset '%s'") % text['name']
 
         self.notifylabel.setText(msg)
         qt4.QTimer.singleShot(4000, self.notifylabel.clear)
-        
+
 def recreateDataset(mainwindow, document, dataset, datasetname):
     """Open dialog to recreate a DatasetExpression / DatasetRange."""
     dialog = DataCreate2DDialog(mainwindow, document)
     mainwindow.showDialog(dialog)
     dialog.reEditDataset(dataset, datasetname)
-
-for c in (document.Dataset2DXYZExpression, document.Dataset2DExpression,
-          document.Dataset2DXYFunc):
-    dataeditdialog.recreate_register[c] = recreateDataset
