@@ -20,6 +20,7 @@ from __future__ import division
 from .. import qtall as qt4
 from .. import setting
 from .. import utils
+from .. import document
 from .veuszdialog import VeuszDialog
 
 def _(text, disambiguation=None, context="PrefsDialog"):
@@ -61,6 +62,9 @@ class PreferencesDialog(VeuszDialog):
             setdb['plot_updatepolicy'])
         self.intervalCombo.setCurrentIndex(index)
         self.threadSpinBox.setValue( setdb['plot_numthreads'] )
+        self.translationEdit.setText( setdb['translation_file'] )
+        self.translationBrowseButton.clicked.connect(
+            self.translationBrowseClicked)
 
         # disable thread option if not supported
         if not qt4.QFontDatabase.supportsThreadedFontRendering():
@@ -69,39 +73,24 @@ class PreferencesDialog(VeuszDialog):
                                             "threaded drawing support"))
 
         # use cwd for file dialogs
-        self.cwdCheck.setChecked( setdb['dirname_usecwd'] )
+        (self.dirDocCWDRadio if setdb['dirname_usecwd'] else self.dirDocPrevRadio).click()
+
+        # exporting documents
+        {
+            'doc': self.dirExportDocRadio,
+            'cwd': self.dirExportCWDRadio,
+            'prev': self.dirExportPrevRadio,
+        }[setdb.get('dirname_export_location')].click()
+
+        # templates when exporting
+        self.exportTemplSingleEdit.setText(setdb['export_template_single'])
+        self.exportTemplMultiEdit.setText(setdb['export_template_multi'])
 
         # set icon size
         self.iconSizeCombo.setCurrentIndex(
             self.iconSizeCombo.findText(
                 str(setdb['toolbar_size'])))
 
-        # set export dpi
-        dpis = ('75', '90', '100', '150', '200', '300')
-        self.exportDPI.addItems(dpis)
-        self.exportDPIPDF.addItems(dpis)
-
-        self.exportDPI.setValidator( qt4.QIntValidator(10, 10000, self) )
-        self.exportDPI.setEditText( str(setdb['export_DPI']) )
-        self.exportDPIPDF.setValidator( qt4.QIntValidator(10, 10000, self) )
-        self.exportDPIPDF.setEditText( str(setdb['export_DPI_PDF']) )
-        self.exportSVGTextAsText.setChecked( setdb['export_SVG_text_as_text'] )
-
-        # set export antialias
-        self.exportAntialias.setChecked( setdb['export_antialias'])
-
-        # quality of jpeg export
-        self.exportQuality.setValue( setdb['export_quality'] )
-
-        # changing background color of bitmaps
-        self.exportBackgroundButton.clicked.connect(
-            self.slotExportBackgroundChanged )
-        self.updateExportBackground(setdb['export_background'])
-
-        # set color setting
-        self.exportColor.setCurrentIndex(
-            {True:0, False:1}[setdb['export_color']])
-
         # default stylesheet
         self.styleLineEdit.setText(setdb['stylesheet_default'])
         self.styleBrowseButton.clicked.connect(self.styleBrowseClicked)
@@ -124,16 +113,36 @@ class PreferencesDialog(VeuszDialog):
         self.pickerToConsoleCheck.setChecked( setdb['picker_to_console'] )
         self.pickerToClipboardCheck.setChecked( setdb['picker_to_clipboard'] )
 
+        # python path
+        self.externalPythonPath.setText(setdb['external_pythonpath'])
+        self.externalGhostscript.setText(setdb['external_ghostscript'])
+        self.externalGhostscriptBrowse.clicked.connect(
+            self.externalGhostscriptBrowseClicked)
+        self.externalNewVerCheck.setChecked(setdb['vercheck_disabled'])
+        if utils.disableVersionChecks:
+            self.externalNewVerCheck.setEnabled(False)
+        self.externalNewVerCheck.setChecked(setdb['feedback_disabled'])
+        if utils.disableFeedback:
+            self.externalFeedbackCheck.setEnabled(False)
+
     def setupColorTab(self):
         """Initialise color tab
         this makes a grid of controls for each color
         consisting of label, isdefault check and change color button."""
 
+        setdb = setting.settingdb
+
+        # theme
+        themes = sorted(list(document.colors.colorthemes))
+        self.colorThemeDefCombo.addItems(themes)
+        self.colorThemeDefCombo.setCurrentIndex(
+            themes.index(setdb['colortheme_default']))
+
+        # UI colors
         self.chosencolors = {}
         self.colorbutton = {}
         self.colordefaultcheck = {}
         layout = qt4.QGridLayout()
-        setdb = setting.settingdb
         for row, colname in enumerate(setdb.colors):
             isdefault, colval = setting.settingdb['color_%s' % colname]
             self.chosencolors[colname] = qt4.QColor(colval)
@@ -178,27 +187,6 @@ class PreferencesDialog(VeuszDialog):
             pixmap.fill(val)
             self.colorbutton[name].setIcon( qt4.QIcon(pixmap) )
 
-    def updateExportBackground(self, colorname):
-        """Update color on export background."""
-        pixmap = qt4.QPixmap(16, 16)
-        col = utils.extendedColorToQColor(colorname)
-        pixmap.fill(col)
-
-        # update button (storing color in button itself - what fun!)
-        self.exportBackgroundButton.setIcon(qt4.QIcon(pixmap))
-        self.exportBackgroundButton.iconcolor = colorname
-
-    def slotExportBackgroundChanged(self):
-        """Button clicked to change background."""
-
-        color = qt4.QColorDialog.getColor(
-            utils.extendedColorToQColor(self.exportBackgroundButton.iconcolor),
-            self,
-            "Choose color",
-            qt4.QColorDialog.ShowAlphaChannel )
-        if color.isValid():
-            self.updateExportBackground( utils.extendedColorFromQColor(color) )
-
     def accept(self):
         """Keep settings if okay pressed."""
 
@@ -211,9 +199,22 @@ class PreferencesDialog(VeuszDialog):
         setdb['plot_antialias'] = self.antialiasCheck.isChecked()
         setdb['ui_english'] = self.englishCheck.isChecked()
         setdb['plot_numthreads'] = self.threadSpinBox.value()
+        setdb['translation_file'] = self.translationEdit.text()
 
         # use cwd
-        setdb['dirname_usecwd'] = self.cwdCheck.isChecked()
+        setdb['dirname_usecwd'] = self.dirDocCWDRadio.isChecked()
+
+        for radio, val in (
+                (self.dirExportDocRadio, 'doc'),
+                (self.dirExportCWDRadio, 'cwd'),
+                (self.dirExportPrevRadio, 'prev'),
+                ):
+            if radio.isChecked():
+                setdb['dirname_export_location'] = val
+
+        # templates for exporting
+        setdb['export_template_single'] = self.exportTemplSingleEdit.text().strip()
+        setdb['export_template_multi'] = self.exportTemplMultiEdit.text().strip()
 
         # update icon size if necessary
         iconsize = int( self.iconSizeCombo.currentText() )
@@ -223,50 +224,48 @@ class PreferencesDialog(VeuszDialog):
                 if isinstance(widget, qt4.QToolBar):
                     widget.setIconSize( qt4.QSize(iconsize, iconsize) )
 
-        # update dpi if possible
-        # FIXME: requires some sort of visual notification of validator
-        for cntrl, setn in ((self.exportDPI, 'export_DPI'),
-                            (self.exportDPIPDF, 'export_DPI_PDF')):
-            try:
-                text = cntrl.currentText()
-                valid = cntrl.validator().validate(text, 0)[0]
-                if valid == qt4.QValidator.Acceptable:
-                    setdb[setn] = int(text)
-            except ValueError:
-                pass
-
-        # export settings
-        setdb['export_antialias'] = self.exportAntialias.isChecked()
-        setdb['export_quality'] = self.exportQuality.value()
-
-        setdb['export_color'] = {0: True, 1: False}[
-            self.exportColor.currentIndex()]
-        setdb['export_background'] = self.exportBackgroundButton.iconcolor
-        setdb['export_SVG_text_as_text'] = self.exportSVGTextAsText.isChecked()
-
         # new document settings
         setdb['stylesheet_default'] = self.styleLineEdit.text()
         setdb['custom_default'] = self.customLineEdit.text()
 
-        # colors
+        # color theme
+        setdb['colortheme_default'] = self.colorThemeDefCombo.currentText()
+
+        # UI colors
         for name, color in self.chosencolors.items():
             isdefault = self.colordefaultcheck[name].isChecked()
             colorname = color.name()
             setdb['color_' + name] = (isdefault, colorname)
 
+        # plugins
+        plugins = self.pluginmodel.stringList()
+        setdb['plugins'] = plugins
+
         # picker
         setdb['picker_to_clipboard'] = self.pickerToClipboardCheck.isChecked()
         setdb['picker_to_console'] = self.pickerToConsoleCheck.isChecked()
 
-        # plugins
-        plugins = self.pluginmodel.stringList()
-        setdb['plugins'] = plugins
+        # python path
+        setdb['external_pythonpath'] = self.externalPythonPath.text()
+        # where to find ghostscript
+        setdb['external_ghostscript'] = self.externalGhostscript.text()
+        # version updates
+        setdb['vercheck_disabled'] = self.externalNewVerCheck.isChecked()
+        # feedback
+        setdb['feedback_disabled'] = self.externalFeedbackCheck.isChecked()
 
         self.plotwindow.updatePlotSettings()
 
         # write settings out now, rather than wait until the end
         setdb.writeSettings()
 
+    def translationBrowseClicked(self):
+        """Browse for a translation."""
+        filename = self.parent().fileOpenDialog(
+            [_('Translation file (*.qm)')], _('Choose translation file'))
+        if filename:
+            self.translationEdit.setText(filename)
+
     def styleBrowseClicked(self):
         """Browse for a stylesheet."""
         filename = self.parent().fileOpenDialog(
@@ -295,3 +294,11 @@ class PreferencesDialog(VeuszDialog):
         sel = self.pluginList.selectionModel().currentIndex()
         if sel.isValid():
             self.pluginmodel.removeRow( sel.row() )
+
+    def externalGhostscriptBrowseClicked(self):
+        """Choose a ghostscript executable."""
+
+        filename = self.parent().fileOpenDialog(
+            [_('All files (*)')], _('Choose ghostscript executable'))
+        if filename:
+            self.externalGhostscript.setText(filename)
