@@ -20,12 +20,12 @@
 
 from .widgetfactory import *
 from .doc import *
-from .datasets import *
+from .evaluate import *
 from .commandinterface import *
 from .commandinterpreter import *
 from .operations import *
 from .mime import *
-from .dataset_histo import *
 from .painthelper import *
 from .export import Export, printDialog
 from .dbusinterface import *
+from .loader import loadDocument, executeScript, LoadError
