@@ -23,11 +23,12 @@ Requires: PyQt-x11-gpl-4.6-snapshot-20090906.tar.gz
           pyemf
 """
 
-from __future__ import division
+from __future__ import division, absolute_import
 import struct
 
 import pyemf
 from .. import qtall as qt4
+from ..compat import cbytes
 
 inch_mm = 25.4
 scale = 100
@@ -39,7 +40,7 @@ def isStockObject(obj):
 class _EXTCREATEPEN(pyemf._EMR._EXTCREATEPEN):
     """Extended pen creation record with custom line style."""
 
-    emr_typedef = [
+    typedef = [
         ('i','handle',0),
         ('i','offBmi',0),
         ('i','cbBmi',0),
@@ -51,7 +52,7 @@ class _EXTCREATEPEN(pyemf._EMR._EXTCREATEPEN):
         ('i','color'),
         ('i','brushhatch',0),
         ('i','numstyleentries')]
-    
+
     def __init__(self, style=pyemf.PS_SOLID, width=1, color=0,
                  styleentries=[]):
         """Create pen.
@@ -63,18 +64,13 @@ class _EXTCREATEPEN(pyemf._EMR._EXTCREATEPEN):
         self.color = pyemf._normalizeColor(color)
         self.brushstyle = 0x0  # solid
 
-        if style & pyemf.PS_USERSTYLE == 0:
-            self.styleentries = []
-        else:
-            self.styleentries = styleentries
+        if style & pyemf.PS_STYLE_MASK != pyemf.PS_USERSTYLE:
+            styleentries = []
 
-        self.numstyleentries = len(self.styleentries)
-
-    def sizeExtra(self):
-        return struct.calcsize("i")*len(self.styleentries)
-
-    def serializeExtra(self, fh):
-        self.serializeList(fh, "i", self.styleentries)
+        self.numstyleentries = len(styleentries)
+        if styleentries:
+            self.unhandleddata = struct.pack(
+                "i"*self.numstyleentries, *styleentries)
 
     def hasHandle(self):
         return True
@@ -150,13 +146,13 @@ class EMFPaintEngine(qt4.QPaintEngine):
         """Draw pixmap to display."""
 
         # convert pixmap to BMP format
-        bytes = qt4.QByteArray()
-        buffer = qt4.QBuffer(bytes)
-        buffer.open(qt4.QIODevice.WriteOnly)
-        pixmap.save(buffer, "BMP")
+        bytearr = qt4.QByteArray()
+        buf = qt4.QBuffer(bytearr)
+        buf.open(qt4.QIODevice.WriteOnly)
+        pixmap.save(buf, "BMP")
 
         # chop off bmp header to get DIB
-        bmp = str(buffer.data())
+        bmp = cbytes(buf.data())
         dib = bmp[0xe:]
         hdrsize, = struct.unpack('<i', bmp[0xe:0x12])
         dataindex, = struct.unpack('<i', bmp[0xa:0xe])
@@ -171,15 +167,16 @@ class EMFPaintEngine(qt4.QPaintEngine):
         epix.yDest = int(r.top()*scale)
         epix.cxDest = int(r.width()*scale)
         epix.cyDest = int(r.height()*scale)
-        epix.xSrc = sr.left()
-        epix.ySrc = sr.top()
-        epix.cxSrc = sr.width()
-        epix.cySrc = sr.height()
+        epix.xSrc = int(sr.left())
+        epix.ySrc = int(sr.top())
+        epix.cxSrc = int(sr.width())
+        epix.cySrc = int(sr.height())
 
         epix.dwRop = 0xcc0020 # SRCCOPY
-        epix.offBmiSrc = epix.serializeOffset()
+        offset = epix.format.minstructsize + 8
+        epix.offBmiSrc = offset
         epix.cbBmiSrc = hdrsize
-        epix.offBitsSrc = epix.serializeOffset() + dataindex - 0xe
+        epix.offBitsSrc = offset + dataindex - 0xe
         epix.cbBitsSrc = datasize
         epix.iUsageSrc = 0x0 # DIB_RGB_COLORS
 
@@ -282,7 +279,7 @@ class EMFPaintEngine(qt4.QPaintEngine):
         color = (qc.red(), qc.green(), qc.blue())
         self.pencolor = color
 
-        if pen.style() & qt4.Qt.CustomDashLine:
+        if pen.style() == qt4.Qt.CustomDashLine:
             # make an extended pen if we need a custom dash pattern
             dash = [int(pen.widthF()*scale*f) for f in pen.dashPattern()]
             newpen = self.emf._appendHandle(
@@ -337,7 +334,7 @@ class EMFPaintEngine(qt4.QPaintEngine):
             clipmode = {
                  qt4.Qt.ReplaceClip: pyemf.RGN_COPY,
                  qt4.Qt.IntersectClip: pyemf.RGN_AND,
-                 qt4.Qt.UniteClip: pyemf.RGN_OR}[operation]
+            }[operation]
         else:
             # is this the only wave to get rid of clipping?
             self.emf.BeginPath()
@@ -366,14 +363,14 @@ class EMFPaintEngine(qt4.QPaintEngine):
             self._updatePen(state.pen())
         if ss & qt4.QPaintEngine.DirtyBrush:
             self._updateBrush(state.brush())
+        if ss & qt4.QPaintEngine.DirtyTransform:
+            self._updateTransform(state.transform())
         if ss & qt4.QPaintEngine.DirtyClipPath:
             self._updateClipPath(state.clipPath(), state.clipOperation())
         if ss & qt4.QPaintEngine.DirtyClipRegion:
             path = qt4.QPainterPath()
             path.addRegion(state.clipRegion())
             self._updateClipPath(path, state.clipOperation())
-        if ss & qt4.QPaintEngine.DirtyTransform:
-            self._updateTransform(state.matrix())
 
     def type(self):
         return qt4.QPaintEngine.PostScript
@@ -410,3 +407,13 @@ class EMFPaintDevice(qt4.QPaintDevice):
             return int(self.engine.dpi)
         elif m == qt4.QPaintDevice.PdmPhysicalDpiY:
             return int(self.engine.dpi)
+        elif m == qt4.QPaintDevice.PdmDevicePixelRatio:
+            return 1
+
+        # Qt >= 5.6
+        elif m == getattr(qt4.QPaintDevice, 'PdmDevicePixelRatioScaled', -1):
+            return 1
+
+        else:
+            # fall back
+            return qt4.QPaintDevice.metric(self, m)
