@@ -184,7 +184,7 @@ class OperationWidgetPaste(operations.OperationMultiple):
 
         # fire up interpreter to read file
         interpreter = commandinterpreter.CommandInterpreter(document)
-        parentwidget = document.resolveFullWidgetPath(self.parentpath)
+        parentwidget = document.resolveWidgetPath(None, self.parentpath)
 
         lines = self.mimedata.split('\n')
         numwidgets = int(lines[0])
@@ -245,16 +245,16 @@ class OperationWidgetClone(OperationWidgetPaste):
 
     def __init__(self, widget, newparent, newname):
         mime = generateWidgetsMime([widget])
-        mime = str(mime.data(widgetmime))
-        OperationWidgetPaste.__init__(self, newparent, mime,
-                                      newnames=[newname])
+        mimedec = mime.data(widgetmime).data().decode('utf-8')
+        OperationWidgetPaste.__init__(
+            self, newparent, mimedec, newnames=[newname])
 
     def do(self, document):
         """Do the import."""
         widgets = OperationWidgetPaste.do(self, document)
         return widgets[0]
 
-class OperationDataPaste(object):
+class OperationDataPaste(operations.Operation):
     """Paste dataset from mime data."""
 
     descr = 'paste data'
