@@ -33,9 +33,9 @@ import io
 import numpy as N
 
 from ..compat import czip, crange, citems, cbasestr
-from . import datasets
 from . import widgetfactory
 
+from .. import datasets
 from .. import plugins
 from .. import qtall as qt4
 
@@ -46,52 +46,63 @@ def _(text, disambiguation=None, context="Operations"):
 ###############################################################################
 # Setting operations
 
-class OperationSettingSet(object):
+class Operation(object):
+    """Root class for operations."""
+
+    descr = 'REPLACE THIS'
+
+    def do(self, document):
+        """Apply operation to document."""
+
+    def undo(self, document):
+        """Undo operation."""
+
+class OperationSettingSet(Operation):
     """Set a variable to a value."""
 
     descr = _('change setting')
-    
+
     def __init__(self, setting, value):
         """Set the setting to value.
         Setting may be a widget path
         """
-        
+
         if isinstance(setting, cbasestr):
             self.settingpath = setting
         else:
             self.settingpath = setting.path
         self.value = value
-        
+
     def do(self, document):
         """Apply setting variable."""
-        setting = document.resolveFullSettingPath(self.settingpath)
+        setting = document.resolveSettingPath(None, self.settingpath)
         if setting.isReference():
             self.oldvalue = setting.getReference()
         else:
             self.oldvalue = setting.get()
         setting.set(self.value)
-        
+
     def undo(self, document):
         """Return old value back..."""
-        setting = document.resolveFullSettingPath(self.settingpath)
+        setting = document.resolveSettingPath(None, self.settingpath)
         setting.set(self.oldvalue)
 
-class OperationSettingPropagate(object):
+class OperationSettingPropagate(Operation):
     """Propagate setting to other widgets."""
-    
+
     descr = _('propagate setting')
-    
+
     def __init__(self, setting, widgetname = None, root = None,
                  maxlevels = -1):
 
         """Take the setting given, and propagate it to other widgets,
         according to the parameters here.
-        
+
         If widgetname is given then only propagate it to widgets with
         the name given.
 
         widgets are located from the widget given (root if not set)
-        
+
         Up to maxlevels levels of widgets are changed (<0 means infinite)
         """
 
@@ -106,20 +117,20 @@ class OperationSettingPropagate(object):
         # work out path of setting relative to widget
         path = []
         s = setting
-        while not s.isWidget():
+        while not s.iswidget:
             path.insert(0, s.name)
             s = s.parent
         self.setpath = path[1:]
         self.widgettype = s.typename
-        
+
     def do(self, document):
         """Apply the setting propagation."""
         # default is root widget
         if not self.rootpath:
             root = document.basewidget
         else:
-            root = document.resolveFullWidgetPath(self.rootpath)
-            
+            root = document.resolveWidgetPath(None, self.rootpath)
+
         # get a list of matching widgets
         widgetlist = []
         self._recursiveGet(root, self.widgetname, self.widgettype, widgetlist,
@@ -135,102 +146,101 @@ class OperationSettingPropagate(object):
 
             self.restorevals[s.path] = s.val
             s.set(self.val)
-          
+
     def undo(self, document):
         """Undo all those changes."""
-        
+
         for setpath, setval in citems(self.restorevals):
-            setting = document.resolveFullSettingPath(setpath)
+            setting = document.resolveSettingPath(None, setpath)
             setting.set(setval)
 
+    @staticmethod
     def _recursiveGet(root, name, typename, outlist, maxlevels):
         """Add those widgets in root with name and type to outlist.
-    
+
         If name or typename are None, then ignore the criterion.
         maxlevels is the maximum number of levels to check
         """
-    
+
         if maxlevels != 0:
-    
+
             # if levels is not zero, add the children of this root
             newmaxlevels = maxlevels - 1
             for w in root.children:
                 if ( (w.name == name or name is None) and
                      (w.typename == typename or typename is None) ):
                     outlist.append(w)
-    
-                OperationSettingPropagate._recursiveGet(w, name, typename,
-                                                        outlist, newmaxlevels)
 
-    _recursiveGet = staticmethod(_recursiveGet)
+                OperationSettingPropagate._recursiveGet(
+                    w, name, typename, outlist, newmaxlevels)
 
 ###############################################################################
 # Widget operations
-        
-class OperationWidgetRename(object):
+
+class OperationWidgetRename(Operation):
     """Rename widget."""
-    
+
     descr = _('rename')
-    
+
     def __init__(self, widget, newname):
         """Rename the widget to newname."""
-        
+
         self.widgetpath = widget.path
         self.newname = newname
-        
+
     def do(self, document):
         """Rename widget."""
-        
-        widget = document.resolveFullWidgetPath(self.widgetpath)
+
+        widget = document.resolveWidgetPath(None, self.widgetpath)
         self.oldname = widget.name
         widget.rename(self.newname)
         self.newpath = widget.path
-        
+
     def undo(self, document):
         """Undo rename."""
-        
-        widget = document.resolveFullWidgetPath(self.newpath)
+
+        widget = document.resolveWidgetPath(None, self.newpath)
         widget.rename(self.oldname)
-        
-class OperationWidgetDelete(object):
+
+class OperationWidgetDelete(Operation):
     """Delete widget."""
-    
+
     descr = _('delete')
-    
+
     def __init__(self, widget):
         """Delete the widget."""
-        
+
         self.widgetpath = widget.path
-        
+
     def do(self, document):
         """Delete widget."""
-        
-        self.oldwidget = document.resolveFullWidgetPath(self.widgetpath)
+
+        self.oldwidget = document.resolveWidgetPath(None, self.widgetpath)
         oldparent = self.oldwidget.parent
         self.oldwidget.parent = None
         self.oldparentpath = oldparent.path
         self.oldindex = oldparent.children.index(self.oldwidget)
         oldparent.removeChild(self.oldwidget.name)
-        
+
     def undo(self, document):
         """Restore deleted widget."""
-        
-        oldparent = document.resolveFullWidgetPath(self.oldparentpath)
+
+        oldparent = document.resolveWidgetPath(None, self.oldparentpath)
         self.oldwidget.parent = oldparent
         oldparent.addChild(self.oldwidget, index=self.oldindex)
 
-class OperationWidgetsDelete(object):
+class OperationWidgetsDelete(Operation):
     """Delete mutliple widget."""
-    
+
     descr = _('delete')
-    
+
     def __init__(self, widgets):
         """Delete the widget."""
         self.widgetpaths = [w.path for w in widgets]
-        
+
     def do(self, document):
         """Delete widget."""
-        
+
         # ignore widgets which share ancestry
         # as deleting the parent deletes the child
         widgetpaths = list(self.widgetpaths)
@@ -251,7 +261,7 @@ class OperationWidgetsDelete(object):
 
         # delete each widget keeping track of details
         for path in widgetpaths:
-            self.oldwidgets.append( document.resolveFullWidgetPath(path) )
+            self.oldwidgets.append( document.resolveWidgetPath(None, path) )
             oldparent = self.oldwidgets[-1].parent
             self.oldparentpaths.append( oldparent.path )
             self.oldindexes.append( oldparent.children.index(self.oldwidgets[-1]) )
@@ -259,42 +269,42 @@ class OperationWidgetsDelete(object):
 
     def undo(self, document):
         """Restore deleted widget."""
-        
+
         # put back widgets in reverse order so that indexes are corrent
         for i in crange(len(self.oldwidgets)-1,-1,-1):
-            oldparent = document.resolveFullWidgetPath(self.oldparentpaths[i])
+            oldparent = document.resolveWidgetPath(None, self.oldparentpaths[i])
             oldparent.addChild(self.oldwidgets[i], index=self.oldindexes[i])
-        
-class OperationWidgetMoveUpDown(object):
+
+class OperationWidgetMoveUpDown(Operation):
     """Move a widget up or down in the hierarchy."""
 
     descr = _('move')
-    
+
     def __init__(self, widget, direction):
         """Move the widget specified up or down in the hierarchy.
-        
+
         direction is -1 for 'up' or +1 for 'down'
         """
-        
+
         self.widgetpath = widget.path
         self.direction = direction
-    
+
     def do(self, document):
         """Move the widget."""
-        
-        widget = document.resolveFullWidgetPath(self.widgetpath)
+
+        widget = document.resolveWidgetPath(None, self.widgetpath)
         parent = widget.parent
         self.suceeded = parent.moveChild(widget, self.direction)
         self.newpath = widget.path
-    
+
     def undo(self, document):
         """Move it back."""
         if self.suceeded:
-            widget = document.resolveFullWidgetPath(self.newpath)
+            widget = document.resolveWidgetPath(None, self.newpath)
             parent = widget.parent
             parent.moveChild(widget, -self.direction)
-            
-class OperationWidgetMove(object):
+
+class OperationWidgetMove(Operation):
     """Move a widget arbitrarily in the hierarchy."""
 
     descr = _('move')
@@ -309,9 +319,9 @@ class OperationWidgetMove(object):
     def do(self, document):
         """Move widget."""
 
-        child = document.resolveFullWidgetPath(self.oldchildpath)
+        child = document.resolveWidgetPath(None, self.oldchildpath)
         oldparent = child.parent
-        newparent = document.resolveFullWidgetPath(self.newparentpath)
+        newparent = document.resolveWidgetPath(None, self.newparentpath)
         self.oldchildindex = oldparent.children.index(child)
         self.oldparentpath = oldparent.path
         self.oldname = None
@@ -351,9 +361,9 @@ class OperationWidgetMove(object):
     def undo(self, document):
         """Undo move."""
 
-        newparent = document.resolveFullWidgetPath(self.newparentpath)
-        child = document.resolveFullWidgetPath(self.newchildpath)
-        oldparent = document.resolveFullWidgetPath(self.oldparentpath)
+        newparent = document.resolveWidgetPath(None, self.newparentpath)
+        child = document.resolveWidgetPath(None, self.newchildpath)
+        oldparent = document.resolveWidgetPath(None, self.oldparentpath)
 
         # remove from new parent
         del newparent.children[self.newindex]
@@ -365,15 +375,15 @@ class OperationWidgetMove(object):
         if self.oldname is not None:
             child.name = self.oldname
 
-class OperationWidgetAdd(object):
+class OperationWidgetAdd(Operation):
     """Add a widget of specified type to parent."""
 
     descr = _('add')
-    
+
     def __init__(self, parent, type, autoadd=True, name=None,
                  index=-1, **defaultvals):
         """Add a widget of type given
-        
+
         parent is the parent widget
         type is the type to add (string)
         autoadd adds children automatically for some widgets
@@ -381,82 +391,80 @@ class OperationWidgetAdd(object):
         index is position in parent to add the widget
         settings can be passed to the created widgets as optional arguments
         """
-        
+
         self.parentpath = parent.path
-        self.type = type
+        self.wtype = type
         self.autoadd = autoadd
         self.name = name
         self.index = index
         self.defaultvals = defaultvals
-        
+
     def do(self, document):
         """Create the new widget.
-        
+
         Returns the new widget
         """
-        
-        parent = document.resolveFullWidgetPath(self.parentpath)
-        w = widgetfactory.thefactory.makeWidget(self.type, parent,
-                                                autoadd=self.autoadd,
-                                                name=self.name,
-                                                index=self.index,
-                                                **self.defaultvals)
+
+        parent = document.resolveWidgetPath(None, self.parentpath)
+        w = widgetfactory.thefactory.makeWidget(
+            self.wtype, parent, document,
+            autoadd=self.autoadd,
+            name=self.name,
+            index=self.index,
+            **self.defaultvals)
         self.createdname = w.name
         return w
-        
+
     def undo(self, document):
         """Remove the added widget."""
-        
-        parent = document.resolveFullWidgetPath(self.parentpath)
+
+        parent = document.resolveWidgetPath(None, self.parentpath)
         parent.removeChild(self.createdname)
 
 ###############################################################################
 # Dataset operations
-    
-class OperationDatasetSet(object):
+
+class OperationDatasetSet(Operation):
     """Set a dataset to that specified."""
-    
+
     descr = _('set dataset')
-    
+
     def __init__(self, datasetname, dataset):
         self.datasetname = datasetname
         self.dataset = dataset
-        
+
     def do(self, document):
         """Set dataset, backing up existing one."""
-    
-        if self.datasetname in document.data:
-            self.olddata = document.data[self.datasetname]
-        else:
-            self.olddata = None
-            
+
+        self.olddata = document.data.get(self.datasetname)
         document.setData(self.datasetname, self.dataset)
 
     def undo(self, document):
         """Undo the data setting."""
-        
-        document.deleteData(self.datasetname)
-        if self.olddata is not None:
+
+        if self.olddata is None:
+            document.deleteData(self.datasetname)
+        else:
             document.setData(self.datasetname, self.olddata)
-    
-class OperationDatasetDelete(object):
+
+class OperationDatasetDelete(Operation):
     """Delete a dateset."""
-    
+
     descr = _('delete dataset')
-    
+
     def __init__(self, datasetname):
         self.datasetname = datasetname
-    
+
     def do(self, document):
         """Remove dataset from document, but preserve for undo."""
         self.olddata = document.data[self.datasetname]
         document.deleteData(self.datasetname)
-        
+
     def undo(self, document):
         """Put dataset back"""
         document.setData(self.datasetname, self.olddata)
 
-class OperationDatasetRename(object):
+class OperationDatasetRename(Operation):
     """Rename the dataset.
 
     Assumes newname doesn't already exist
@@ -504,96 +512,96 @@ class OperationDatasetRename(object):
 
         document.renameDataset(self.newname, self.oldname)
 
-class OperationDatasetDuplicate(object):
+class OperationDatasetDuplicate(Operation):
     """Duplicate a dataset.
-    
+
     Assumes duplicate name doesn't already exist
     """
-    
+
     descr = _('duplicate dataset')
-    
+
     def __init__(self, origname, duplname):
         self.origname = origname
         self.duplname = duplname
-        
+
     def do(self, document):
         """Make the duplicate"""
-        self.olddata = document.data.get(self.duplname, None)
+        self.olddata = document.data.get(self.duplname)
 
         dataset = document.data[self.origname]
         duplicate = dataset.returnCopy()
         document.setData(self.duplname, duplicate)
-        
+
     def undo(self, document):
         """Delete the duplicate"""
-        
+
         if self.olddata is None:
             document.deleteData(self.duplname)
         else:
             document.setData(self.duplname, self.olddata)
-        
-class OperationDatasetUnlinkFile(object):
+
+class OperationDatasetUnlinkFile(Operation):
     """Remove association between dataset and file."""
     descr = _('unlink dataset')
-    
+
     def __init__(self, datasetname):
         self.datasetname = datasetname
-        
+
     def do(self, document):
         dataset = document.data[self.datasetname]
         self.oldfilelink = dataset.linked
         dataset.linked = None
-        
+
     def undo(self, document):
         dataset = document.data[self.datasetname]
         dataset.linked = self.oldfilelink
 
-class OperationDatasetUnlinkRelation(object):
+class OperationDatasetUnlinkRelation(Operation):
     """Remove association between dataset and another dataset.
     """
-    
+
     descr = _('unlink dataset')
-    
+
     def __init__(self, datasetname):
         self.datasetname = datasetname
-        
+
     def do(self, document):
         dataset = document.data[self.datasetname]
         self.olddataset = dataset
         ds = dataset.returnCopy()
         document.setData(self.datasetname, ds)
-        
+
     def undo(self, document):
         document.setData(self.datasetname, self.olddataset)
-        
-class OperationDatasetCreate(object):
+
+class OperationDatasetCreate(Operation):
     """Create dataset base class."""
-    
+
     def __init__(self, datasetname):
         self.datasetname = datasetname
         self.parts = {}
-        
+
     def setPart(self, part, val):
         self.parts[part] = val
-        
+
     def do(self, document):
         """Record old dataset if it exists."""
-        self.olddataset = document.data.get(self.datasetname, None)
-        
+        self.olddataset = document.data.get(self.datasetname)
+
     def undo(self, document):
         """Delete the created dataset."""
         document.deleteData(self.datasetname)
         if self.olddataset is not None:
             document.setData(self.datasetname, self.olddataset)
-        
+
 class OperationDatasetCreateRange(OperationDatasetCreate):
     """Create a dataset in a specfied range."""
-    
+
     descr = _('create dataset from range')
-    
+
     def __init__(self, datasetname, numsteps, parts, linked=False):
         """Create a dataset with numsteps values.
-        
+
         parts is a dict containing keys 'data', 'serr', 'perr' and/or 'nerr'. The values
         are tuples with (start, stop) values for each range.
         """
@@ -601,16 +609,16 @@ class OperationDatasetCreateRange(OperationDatasetCreate):
         self.numsteps = numsteps
         self.parts = parts
         self.linked = linked
-        
+
     def do(self, document):
         """Create dataset using range."""
-        
+
         OperationDatasetCreate.do(self, document)
         data = self.parts['data']
         serr = self.parts.get('serr', None)
         perr = self.parts.get('perr', None)
         nerr = self.parts.get('nerr', None)
-        
+
         ds = datasets.DatasetRange(self.numsteps, data, serr=serr,
                                    perr=perr, nerr=nerr)
         if not self.linked:
@@ -620,23 +628,23 @@ class OperationDatasetCreateRange(OperationDatasetCreate):
 
         document.setData(self.datasetname, ds)
         return ds
-        
+
 class CreateDatasetException(Exception):
     """Thrown by dataset creation routines."""
     pass
-        
+
 class OperationDatasetCreateParameteric(OperationDatasetCreate):
     """Create a dataset using expressions dependent on t."""
-    
+
     descr = _('create parametric dataset')
-    
+
     def __init__(self, datasetname, t0, t1, numsteps, parts, linked=False):
         """Create a parametric dataset.
-        
+
         Variable t goes from t0 to t1 in numsteps.
         parts is a dict with keys 'data', 'serr', 'perr' and/or 'nerr'
         The values are expressions for evaluating."""
-        
+
         OperationDatasetCreate.__init__(self, datasetname)
         self.numsteps = numsteps
         self.t0 = t0
@@ -657,24 +665,24 @@ class OperationDatasetCreateParameteric(OperationDatasetCreate):
             # copy these values if we don't want to link
             ds = datasets.Dataset(data=ds.data, serr=ds.serr,
                                   perr=ds.perr, nerr=ds.nerr)
-        
+
         document.setData(self.datasetname, ds)
         return ds
-        
+
 class OperationDatasetCreateExpression(OperationDatasetCreate):
     descr = _('create dataset from expression')
 
     def __init__(self, datasetname, parts, link, parametric=None):
         """Create a dataset from existing dataset using expressions.
-        
+
         parts is a dict with keys 'data', 'serr', 'perr' and/or 'nerr'
         The values are expressions for evaluating.
-        
+
         If link is True, then the dataset is linked to the expressions
         Parametric is a tuple (min, max, numitems) if creating parametric
         datasets.
         """
-        
+
         OperationDatasetCreate.__init__(self, datasetname)
         self.parts = parts
         self.link = link
@@ -692,7 +700,7 @@ class OperationDatasetCreateExpression(OperationDatasetCreate):
         ds.document = document
 
         return ds.updateEvaluation()
-        
+
     def do(self, document):
         """Create the dataset."""
         OperationDatasetCreate.do(self, document)
@@ -706,18 +714,80 @@ class OperationDatasetCreateExpression(OperationDatasetCreate):
             # copy these values if we don't want to link
             ds = datasets.Dataset(data=ds.data, serr=ds.serr,
                                   perr=ds.perr, nerr=ds.nerr)
-        
+
         document.setData(self.datasetname, ds)
         return ds
 
-class OperationDataset2DBase(object):
+class OperationDatasetsFilter(Operation):
+    """Operation to filter datasets."""
+
+    descr = _("filter datasets")
+
+    def __init__(self, inexpr, indatasets,
+                 prefix="", suffix="",
+                 invert=False, replaceblanks=False):
+        """Initialise operation:
+        inexpr: input expression
+        indatasets: list of dataset names
+        prefix, suffix: output prefix/suffix
+        invert: invert filter expression
+        replaceblanks: replace output with blank/nan values.
+        """
+
+        if not prefix and not suffix:
+            raise ValueError("Prefix and/or suffix must be given")
+        self.inexpr = inexpr
+        self.indatasets = indatasets
+        self.prefix = prefix
+        self.suffix = suffix
+        self.invert = invert
+        self.replaceblanks = replaceblanks
+
+    def makeGen(self):
+        """Return generator object."""
+        return datasets.DatasetFilterGenerator(
+            self.inexpr, self.indatasets,
+            prefix=self.prefix, suffix=self.suffix,
+            invert=self.invert, replaceblanks=self.replaceblanks)
+
+    def check(self, doc):
+        """Check the filter is ok.
+
+        Return (ok, [list of errors])
+        """
+
+        log = self.makeGen().evaluateFilter(doc)
+        if log:
+            return (False, log)
+        return (True, [])
+
+    def do(self, doc):
+        """Do the operation."""
+
+        gen = self.makeGen()
+        self.olddatasets = {}
+        for name in self.indatasets:
+            outname = self.prefix + name + self.suffix
+            self.olddatasets[outname] = doc.data.get(outname)
+            doc.setData(outname, datasets.DatasetFiltered(gen, name, doc))
+
+    def undo(self, doc):
+        """Undo operation."""
+
+        for name, val in citems(self.olddatasets):
+            if val is None:
+                doc.deleteData(name)
+            else:
+                doc.setData(name, val)
+
+class OperationDataset2DBase(Operation):
     """Operation as base for 2D dataset creation operations."""
 
     def __init__(self, name, link):
         """Setup operation."""
         self.datasetname = name
         self.link = link
-    
+
     def validateExpression(self, document):
         """Validate expression is okay."""
         ds = self.makeDSClass()
@@ -791,7 +861,7 @@ class OperationDataset2DXYFunc(OperationDataset2DBase):
     def makeDSClass(self):
         return datasets.Dataset2DXYFunc(self.xstep, self.ystep, self.expr)
 
-class OperationDatasetUnlinkByFile(object):
+class OperationDatasetUnlinkByFile(Operation):
     """Unlink all datasets associated with file."""
 
     descr = _('unlink datasets')
@@ -816,7 +886,7 @@ class OperationDatasetUnlinkByFile(object):
             except KeyError:
                 pass
 
-class OperationDatasetDeleteByFile(object):
+class OperationDatasetDeleteByFile(Operation):
     """Delete all datasets associated with file."""
 
     descr = _('delete datasets')
@@ -841,7 +911,7 @@ class OperationDatasetDeleteByFile(object):
 ###############################################################################
 # Import datasets
 
-class OperationDataTag(object):
+class OperationDataTag(Operation):
     """Add a tag to a list of datasets."""
 
     descr = _('add dataset tags')
@@ -865,7 +935,7 @@ class OperationDataTag(object):
         for name in self.removetags:
             document.data[name].tags.remove(self.tag)
 
-class OperationDataUntag(object):
+class OperationDataUntag(Operation):
     """Add a tag to a list of datasets."""
 
     descr = _('remove dataset tags')
@@ -888,14 +958,14 @@ class OperationDataUntag(object):
 ###############################################################################
 # Alter dataset
 
-class OperationDatasetAddColumn(object):
+class OperationDatasetAddColumn(Operation):
     """Add a column to a dataset, blanked to zero."""
 
     descr = _('add dataset column')
 
     def __init__(self, datasetname, columnname):
         """Initialise column columnname in datasetname.
-        
+
         columnname can be one of 'data', 'serr', 'perr' or 'nerr'
         """
         self.datasetname = datasetname
@@ -918,18 +988,18 @@ class OperationDatasetAddColumn(object):
         setattr(ds, self.columnname, None)
         document.setData(self.datasetname, ds)
 
-class OperationDatasetSetVal(object):
+class OperationDatasetSetVal(Operation):
     """Set a value in the dataset."""
 
     descr = _('change dataset value')
-    
+
     def __init__(self, datasetname, columnname, row, val):
         """Set row in column columnname to val."""
         self.datasetname = datasetname
         self.columnname = columnname
         self.row = row
         self.val = val
-        
+
     def do(self, document):
         """Set the value."""
         ds = document.data[self.datasetname]
@@ -944,8 +1014,8 @@ class OperationDatasetSetVal(object):
         datacol = getattr(ds, self.columnname)
         datacol[self.row] = self.oldval
         ds.changeValues(self.columnname, datacol)
-    
-class OperationDatasetSetVal2D(object):
+
+class OperationDatasetSetVal2D(Operation):
     """Set a value in a 2D dataset."""
 
     descr = _('change 2D dataset value')
@@ -970,17 +1040,17 @@ class OperationDatasetSetVal2D(object):
         ds.data[self.row, self.col] = self.oldval
         document.modifiedData(ds)
 
-class OperationDatasetDeleteRow(object):
+class OperationDatasetDeleteRow(Operation):
     """Delete a row or several in the dataset."""
 
     descr = _('delete dataset row')
-    
+
     def __init__(self, datasetname, row, numrows=1):
         """Delete a row in a dataset."""
         self.datasetname = datasetname
         self.row = row
         self.numrows = numrows
-        
+
     def do(self, document):
         """Set the value."""
         ds = document.data[self.datasetname]
@@ -991,17 +1061,17 @@ class OperationDatasetDeleteRow(object):
         ds = document.data[self.datasetname]
         ds.insertRows(self.row, self.numrows, self.saveddata)
 
-class OperationDatasetInsertRow(object):
+class OperationDatasetInsertRow(Operation):
     """Insert a row or several in the dataset."""
 
     descr = _('insert dataset row')
-    
+
     def __init__(self, datasetname, row, numrows=1):
         """Delete a row in a dataset."""
         self.datasetname = datasetname
         self.row = row
         self.numrows = numrows
-        
+
     def do(self, document):
         """Set the value."""
         ds = document.data[self.datasetname]
@@ -1015,70 +1085,86 @@ class OperationDatasetInsertRow(object):
 ###############################################################################
 # Custom setting operations
 
-class OperationSetCustom(object):
+class OperationSetCustom(Operation):
     """Set custom objects, such as constants."""
 
     descr = _('set a custom definition')
 
-    def __init__(self, vals):
-        """customtype is the type of custom object to set:
-        eg functions, constants
-        customval is a dict of the values."""
+    # translate ctype below into attribute of evaluate
+    type_to_attr = {
+        'definition': 'def_definitions',
+        'function':   'def_definitions',
+        'constant':   'def_definitions',
+        'import':     'def_imports',
+        'color':      'def_colors',
+        'colormap':   'def_colormaps',
+    }
 
+    def __init__(self, ctype, vals):
+        """Set custom values to be the list given.
+
+        ctype is one of 'definition', 'function', 'constant',
+        'import', 'color' or 'colormap'
+        """
+        self.ctype = ctype
         self.customvals = list(vals)
 
+    def _getlist(self, document):
+        return getattr(document.evaluate, self.type_to_attr[self.ctype])
+
     def do(self, document):
         """Set the custom object."""
-        self.oldval = list(document.customs)
-        document.customs = self.customvals
-        document.updateEvalContext()
-        
+        lst = self._getlist(document)
+        self.oldval = list(lst)
+        lst[:] = self.customvals
+        document.evaluate.update()
+
     def undo(self, document):
         """Restore custom object."""
-        document.customs = self.oldval
-        document.updateEvalContext()
+        self._getlist(document)[:] = self.oldval
+        document.evaluate.update()
 
 ###############################################################################
 # Misc operations
-        
-class OperationMultiple(object):
+
+class OperationMultiple(Operation):
     """Multiple operations batched into one."""
-    
+
     def __init__(self, operations, descr='change'):
         """A batch operation made up of the operations in list.
-        
+
         Optional argument descr gives a description of the combined operation
         """
         self.operations = operations
         if descr:
             self.descr = descr
-        
+
     def addOperation(self, op):
         """Add an operation to the list of operations."""
         self.operations.append(op)
-        
+
     def do(self, document):
         """Do the multiple operations."""
         for op in self.operations:
             op.do(document)
-            
+
     def undo(self, document):
         """Undo the multiple operations."""
-        
+
         # operations need to undone in reverse order
         for op in self.operations[::-1]:
             op.undo(document)
 
 class OperationLoadStyleSheet(OperationMultiple):
     """An operation to load a stylesheet."""
-    
+
     descr = _('load stylesheet')
 
     def __init__(self, filename):
         """Load stylesheet with filename."""
         OperationMultiple.__init__(self, [], descr=None)
         self.filename = os.path.abspath(filename)
-        
+
     def do(self, document):
         """Do the import."""
 
@@ -1095,20 +1181,21 @@ class OperationLoadStyleSheet(OperationMultiple):
         except:
             document.batchHistory(None)
             raise
-        
+        document.batchHistory(None)
+
 class OperationLoadCustom(OperationLoadStyleSheet):
     descr = _('load custom definitions')
 
 class OperationToolsPlugin(OperationMultiple):
     """An operation to represent what a tools plugin does."""
-    
+
     def __init__(self, plugin, fields):
         """Use tools plugin, passing fields."""
         OperationMultiple.__init__(self, [], descr=None)
         self.plugin = plugin
         self.fields = fields
         self.descr = plugin.name
-        
+
     def do(self, document):
         """Use the plugin."""
 
@@ -1126,16 +1213,16 @@ class OperationToolsPlugin(OperationMultiple):
             raise
         document.batchHistory(None)
 
-class OperationDatasetPlugin(object):
+class OperationDatasetPlugin(Operation):
     """An operation to activate a dataset plugin."""
-    
+
     def __init__(self, plugin, fields, datasetnames={}):
         """Use dataset plugin, passing fields."""
         self.plugin = plugin
         self.fields = fields
         self.descr = plugin.name
         self.names = datasetnames
-        
+
     def do(self, document):
         """Use the plugin.
         """
@@ -1180,3 +1267,67 @@ class OperationDatasetPlugin(object):
         # put back old datasets
         for name, ds in citems(self.olddata):
             document.setData(name, ds)
+
+class OperationDatasetHistogram(Operation):
+    """Operation to make histogram from data."""
+
+    descr = _("make histogram")
+
+    def __init__(self, expr, outposns, outvalues,
+                 binparams=None, binmanual=None, method='counts',
+                 cumulative = 'none',
+                 errors=False):
+        """
+        inexpr = input dataset expression
+        outposns = name of dataset for bin positions
+        outvalues = name of dataset for bin values
+        binparams = None / (num, minval, maxval, islog)
+        binmanual = None / [1,2,3,4,5]
+        method = ('counts', 'density', or 'fractions')
+        cumulative = ('none', 'smalltolarge', 'largetosmall')
+        errors = True/False
+        """
+
+        self.expr = expr
+        self.outposns = outposns
+        self.outvalues = outvalues
+        self.binparams = binparams
+        self.binmanual = binmanual
+        self.method = method
+        self.cumulative = cumulative
+        self.errors = errors
+
+    def do(self, document):
+        """Create histogram datasets."""
+
+        gen = datasets.DatasetHistoGenerator(
+            document, self.expr, binparams=self.binparams,
+            binmanual=self.binmanual,
+            method=self.method,
+            cumulative=self.cumulative,
+            errors=self.errors)
+
+        self.oldposnsds = self.oldvaluesds = None
+
+        if self.outvalues != '':
+            self.oldvaluesds = document.data.get(self.outvalues, None)
+            document.setData(self.outvalues, gen.generateValueDataset())
+
+        if self.outposns != '':
+            self.oldposnsds = document.data.get(self.outposns, None)
+            document.setData(self.outposns, gen.generateBinDataset())
+
+    def undo(self, document):
+        """Undo creation of datasets."""
+
+        if self.oldposnsds is not None:
+            if self.outposns != '':
+                document.setData(self.outposns, self.oldposnsds)
+        else:
+            document.deleteData(self.outposns)
+
+        if self.oldvaluesds is not None:
+            if self.outvalues != '':
+                document.setData(self.outvalues, self.oldvaluesds)
+        else:
+            document.deleteData(self.outvalues)
