@@ -20,12 +20,14 @@
 
 """Plugins for creating datasets."""
 
-from __future__ import division
+from __future__ import division, print_function
+import re
 import numpy as N
 from . import field
 
 from ..compat import czip, citems, cstr
 from .. import utils
+from .. import datasets
 try:
     from ..helpers import qtloops
 except ImportError:
@@ -51,8 +53,20 @@ def numpyCopyOrNone(data):
         return None
     return N.array(data, dtype=N.float64)
 
+class _DatasetBase(object):
+    """Base class for dataset objects to be returned from plugins."""
+
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset
+        (or None if not a real dataset."""
+        return None
+
+    def _customs(self):
+        """Return list of custom definitions to add to document."""
+        return []
+
 # these classes are returned from dataset plugins
-class Dataset1D(object):
+class Dataset1D(_DatasetBase):
     """1D dataset for ImportPlugin or DatasetPlugin."""
     def __init__(self, name, data=[], serr=None, perr=None, nerr=None):
         """1D dataset
@@ -83,11 +97,14 @@ class Dataset1D(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        # need to do the import here as otherwise we get a loop
-        from .. import document
-        return document.Dataset1DPlugin(manager, self)
+        return datasets.Dataset1DPlugin(manager, self)
 
-class Dataset2D(object):
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset."""
+        return datasets.Dataset(
+            data=self.data, serr=self.serr, perr=self.perr, nerr=self.nerr)
+
+class Dataset2D(_DatasetBase):
     """2D dataset for ImportPlugin or DatasetPlugin."""
     def __init__(self, name, data=[[]], rangex=None, rangey=None,
                  xedge=None, yedge=None,
@@ -126,16 +143,48 @@ class Dataset2D(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        from .. import document
-        return document.Dataset2DPlugin(manager, self)
+        return datasets.Dataset2DPlugin(manager, self)
 
-class DatasetDateTime(object):
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset."""
+        return datasets.Dataset2D(
+            data=self.data,
+            xrange=self.rangex, yrange=self.rangey,
+            xedge=self.xedge, yedge=self.yedge,
+            xcent=self.xcent, ycent=self.ycent)
+
+class DatasetND(_DatasetBase):
+    """ND dataset for ImportPlugin or DatasetPlugin."""
+    def __init__(self, name, data=[]):
+        """N-dimensional dataset.
+        name: name of dataset
+        data: ND numpy array of values (or lists to convert)
+        """
+        self.name = name
+        self.update(data=data)
+
+    def update(self, data=[]):
+        self.data = N.array(data, dtype=N.float64)
+
+    def _null(self):
+        """Empty data contents."""
+        self.data = N.array([])
+
+    def _makeVeuszDataset(self, manager):
+        """Make a Veusz dataset from the plugin dataset."""
+        return datasets.DatasetNDPlugin(manager, self)
+
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset."""
+        return datasets.DatasetND(data=self.data)
+
+class DatasetDateTime(_DatasetBase):
     """Date-time dataset for ImportPlugin or DatasetPlugin."""
 
     def __init__(self, name, data=[]):
         """A date dataset
         name: name of dataset
-        data: list of datetime objects
+        data: list of datetime datasets
         """
         self.name = name
         self.update(data=data)
@@ -164,10 +213,13 @@ class DatasetDateTime(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        from .. import document
-        return document.DatasetDateTimePlugin(manager, self)
+        return datasets.DatasetDateTimePlugin(manager, self)
 
-class DatasetText(object):
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset."""
+        return datasets.DatasetDateTime(data=self.data)
+
+class DatasetText(_DatasetBase):
     """Text dataset for ImportPlugin or DatasetPlugin."""
     def __init__(self, name, data=[]):
         """A text dataset
@@ -186,10 +238,13 @@ class DatasetText(object):
 
     def _makeVeuszDataset(self, manager):
         """Make a Veusz dataset from the plugin dataset."""
-        from .. import document
-        return document.DatasetTextPlugin(manager, self)
+        return datasets.DatasetTextPlugin(manager, self)
 
-class Constant(object):
+    def _unlinkedVeuszDataset(self):
+        """Convert this to an equivalent (unlinked) Veusz dataset."""
+        return datasets.DatasetText(data=self.data)
+
+class Constant(_DatasetBase):
     """Dataset to return to set a Veusz constant after import.
     This is only useful in an ImportPlugin, not a DatasetPlugin
     """
@@ -199,7 +254,10 @@ class Constant(object):
         self.name = name
         self.val = val
 
-class Function(object):
+    def _customs(self):
+        return [['constant', self.name, self.val]]
+
+class Function(_DatasetBase):
     """Dataset to return to set a Veusz function after import."""
     def __init__(self, name, val):
         """Map string value val to name.
@@ -209,10 +267,13 @@ class Function(object):
         self.name = name
         self.val = val
 
+    def _customs(self):
+        return [['function', self.name, self.val]]
+
 # class to pass to plugin to give parameters
 class DatasetPluginHelper(object):
     """Helpers to get existing datasets for plugins."""
-    
+
     def __init__(self, doc):
         """Construct helper object to pass to DatasetPlugins."""
         self._doc = doc
@@ -238,9 +299,8 @@ class DatasetPluginHelper(object):
     @property
     def datasetsdatetime(self):
         """Return list of existing date-time datesets"""
-        from .. import document
         return [name for name, ds in citems(self._doc.data) if
-                isinstance(ds, document.DatasetDateTime)]
+                isinstance(ds, datasets.DatasetDateTime)]
 
     @property
     def locale(self):
@@ -254,11 +314,8 @@ class DatasetPluginHelper(object):
 
         Returns None if expression could not be evaluated.
         """
-        ds = self._doc.evalDatasetExpression(expr, part=part)
-        if ds is not None:
-            return ds.data
-        else:
-            return None
+        ds = datasets.evalDatasetExpression(self._doc, expr, part=part)
+        return None if ds is None else ds.data
 
     def getDataset(self, name, dimensions=1):
         """Return numerical dataset object for name given.
@@ -270,32 +327,31 @@ class DatasetPluginHelper(object):
         name not found: raise a DatasetPluginException
         dimensions not right: raise a DatasetPluginException
         """
-        from .. import document
         try:
             ds = self._doc.data[name]
         except KeyError:
             raise DatasetPluginException(_("Unknown dataset '%s'") % name)
 
-        if ds.dimensions != dimensions:
+        if ds.dimensions != dimensions and dimensions != 'all':
             raise DatasetPluginException(
-                _("Dataset '%s' does not have %i dimensions") % (
+                _("Dataset '%s' does not have %s dimensions") % (
                     name, dimensions))
         if ds.datatype != 'numeric':
             raise DatasetPluginException(
                 _("Dataset '%s' is not a numerical dataset") % name)
 
-        if isinstance(ds, document.DatasetDateTime):
+        if isinstance(ds, datasets.DatasetDateTime):
             return DatasetDateTime(name, data=ds.data)
         elif ds.dimensions == 1:
-            return Dataset1D(name, data=ds.data, serr=ds.serr, 
+            return Dataset1D(name, data=ds.data, serr=ds.serr,
                              perr=ds.perr, nerr=ds.nerr)
         elif ds.dimensions == 2:
             return Dataset2D(name, ds.data,
-                             xrange=ds.xrange, yrange=ds.yrange,
+                             rangex=ds.xrange, rangey=ds.yrange,
                              xedge=ds.xedge, yedge=ds.yedge,
                              xcent=ds.xcent, ycent=ds.ycent)
         else:
-            raise RuntimeError("Invalid number of dimensions in dataset")
+            return DatasetND(name, ds.data)
 
     def getDatasets(self, names, dimensions=1):
         """Get a list of numerical datasets (of the dimension given)."""
@@ -327,7 +383,7 @@ class DatasetPluginManager(object):
         doc - document instance
         fields - fields to pass to plugin
         """
-        
+
         self.plugin = plugin
         self.document = doc
         self.helper = DatasetPluginHelper(doc)
@@ -364,7 +420,7 @@ class DatasetPluginManager(object):
     def saveToFile(self, fileobj):
         """Save command to load in plugin and parameters."""
 
-        args = [ repr(self.plugin.name), repr(self.fields) ]
+        args = [ utils.rrepr(self.plugin.name), utils.rrepr(self.fields) ]
 
         # look for renamed or deleted datasets
         names = {}
@@ -379,7 +435,7 @@ class DatasetPluginManager(object):
                 names[dsname] = currentname
 
         if names:
-            args.append( "datasetnames="+repr(names) )
+            args.append( "datasetnames="+utils.rrepr(names) )
 
         fileobj.write( 'DatasetPlugin(%s)\n' % (', '.join(args)) )
 
@@ -427,8 +483,8 @@ class DatasetPlugin(object):
 
     def getDatasets(self, fields):
         """Override this to return a list of (empty) Dataset1D,
-        Dataset2D and DatasetText objects to provide the initial names
-        and type of datasets.
+        Dataset2D, DatasetText, DatasetDateTime and DatasetND
+        objects to provide the initial names and type of datasets.
 
         These should be saved for updating in updateDatasets.
 
@@ -485,18 +541,19 @@ def combineAddedErrors(inds, length):
 
     for d in inds:
         f = N.isfinite(d.data)
+        l = len(f)
 
         if errortype == 'symmetric' and d.serr is not None:
-            serr[f] += d.serr[f]**2 
+            serr[:l][f] += d.serr[f]**2
         elif errortype == 'asymmetric':
             if d.serr is not None:
                 v = (d.serr[f])**2
-                perr[f] += v
-                nerr[f] += v
+                perr[:l][f] += v
+                nerr[:l][f] += v
             if d.perr is not None:
-                perr[f] += (d.perr[f])**2
+                perr[:l][f] += (d.perr[f])**2
             if d.nerr is not None:
-                nerr[f] += (d.nerr[f])**2
+                nerr[:l][f] += (d.nerr[f])**2
 
     if serr is not None: serr = N.sqrt(serr)
     if perr is not None: perr = N.sqrt(perr)
@@ -521,7 +578,7 @@ def combineMultipliedErrors(inds, length, data):
             f = f[:length]
 
         if errortype == 'symmetric' and d.serr is not None:
-            serr[f] += (d.serr[f]/d.data[f])**2 
+            serr[f] += (d.serr[f]/d.data[f])**2
         elif errortype == 'asymmetric':
             if d.serr is not None:
                 v = (d.serr[f]/d.data[f])**2
@@ -548,7 +605,7 @@ class MultiplyDatasetPlugin(_OneOutputDatasetPlugin):
     description_short = _('Multiply dataset by a constant')
     description_full = _('Multiply a dataset by a factor. '
                          'Error bars are also scaled.')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -579,7 +636,7 @@ class AddDatasetPlugin(_OneOutputDatasetPlugin):
     description_short = _('Add a constant to a dataset')
     description_full = _('Add a dataset by adding a value. '
                          'Error bars remain the same.')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -867,8 +924,8 @@ class MeanDatasetPlugin(_OneOutputDatasetPlugin):
         num = N.zeros(maxlength, dtype=N.int)
         for d in inds:
             f = N.isfinite(d.data)
-            tot[f] += d.data[f]
-            num[f] += 1
+            tot[:len(d.data)][f] += d.data[f]
+            num[:len(d.data)][f] += 1
         data = tot / num
 
         def averageError(errtype, fallback=None):
@@ -883,8 +940,8 @@ class MeanDatasetPlugin(_OneOutputDatasetPlugin):
                 # add values if not missing
                 if vals is not None:
                     f = N.isfinite(vals)
-                    tot[f] += (vals[f]) ** 2
-                    num[f] += 1
+                    tot[:len(f)][f] += (vals[f]) ** 2
+                    num[:len(f)][f] += 1
                 else:
                     # treat as zero errors if missing errors
                     num[:len(d.data)] += 1
@@ -918,7 +975,7 @@ class AddDatasetsPlugin(_OneOutputDatasetPlugin):
             ]
 
     def updateDatasets(self, fields, helper):
-        """Compute means of dataset."""
+        """Sum datasets."""
 
         inds = helper.getDatasets(fields['ds_in'])
         if len(inds) == 0:
@@ -930,8 +987,8 @@ class AddDatasetsPlugin(_OneOutputDatasetPlugin):
         anyfinite = N.zeros(maxlength, dtype=N.bool)
         for d in inds:
             f = N.isfinite(d.data)
-            data[f] += d.data[f]
-            anyfinite[f] = True
+            data[:len(d.data)][f] += d.data[f]
+            anyfinite[:len(d.data)][f] = True
         data[N.logical_not(anyfinite)] = N.nan
 
         # handle error bars
@@ -948,7 +1005,7 @@ class SubtractDatasetPlugin(_OneOutputDatasetPlugin):
     description_short = _('Subtract two datasets')
     description_full = _('Subtract two datasets. '
                          'Combined error bars are also calculated.')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -1016,11 +1073,13 @@ class SubtractMeanDatasetPlugin(_OneOutputDatasetPlugin):
         dsin = helper.getDataset(fields['ds_in'])
 
         vals = dsin.data
-        mean = vals[N.isfinite(vals)].mean()
-        vals = vals - mean
+        if len(vals) > 0:
+            mean = vals[N.isfinite(vals)].mean()
+            vals = vals - mean
 
         if fields['divstddev']:
-            vals /= vals[N.isfinite(vals)].std()
+            if len(vals) > 0:
+                vals /= vals[N.isfinite(vals)].std()
 
         self.dsout.update(
             data=vals, serr=dsin.serr, perr=dsin.perr, nerr=dsin.nerr)
@@ -1041,16 +1100,17 @@ class SubtractMinimumDatasetPlugin(_OneOutputDatasetPlugin):
             ]
 
     def updateDatasets(self, fields, helper):
-        """Do scaling of dataset."""
+        """Do subtraction of dataset."""
 
         dsin = helper.getDataset(fields['ds_in'])
 
-        vals = dsin.data
-        minval = vals[N.isfinite(vals)].min()
-        vals = vals - minval
+        data = dsin.data
+        filtered = data[N.isfinite(data)]
+        if len(filtered) != 0:
+            data = data - filtered.min()
 
         self.dsout.update(
-            data=vals, serr=dsin.serr, perr=dsin.perr, nerr=dsin.nerr)
+            data=data, serr=dsin.serr, perr=dsin.perr, nerr=dsin.nerr)
 
 class MultiplyDatasetsPlugin(_OneOutputDatasetPlugin):
     """Dataset plugin to multiply two or more datasets."""
@@ -1060,7 +1120,7 @@ class MultiplyDatasetsPlugin(_OneOutputDatasetPlugin):
     description_short = _('Multiply two or more datasets')
     description_full = _('Multiply two or more datasets. '
                          'Combined error bars are also calculated.')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -1100,7 +1160,7 @@ class DivideDatasetsPlugin(_OneOutputDatasetPlugin):
                           ' between two datasets')
     description_full = _('Divide or compute fractional difference'
                          ' between two datasets')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -1147,15 +1207,21 @@ class DivideMaxPlugin(_OneOutputDatasetPlugin):
     def updateDatasets(self, fields, helper):
 
         inds = helper.getDataset( fields['ds_in'] )
-        maxval = N.nanmax( inds.data )
+
+        data = inds.data
+        filtered = data[N.isfinite(data)]
+        if len(filtered) == 0:
+            maxval = N.nan
+        else:
+            maxval = filtered.max()
 
         # divide data
-        data = inds.data / maxval
+        data = data / maxval
         # divide error bars
         serr = perr = nerr = None
-        if inds.serr: serr = inds.serr / maxval
-        if inds.perr: perr = inds.perr / maxval
-        if inds.nerr: nerr = inds.nerr / maxval
+        if inds.serr is not None: serr = inds.serr / maxval
+        if inds.perr is not None: perr = inds.perr / maxval
+        if inds.nerr is not None: nerr = inds.nerr / maxval
 
         self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
 
@@ -1177,15 +1243,21 @@ class DivideNormalizePlugin(_OneOutputDatasetPlugin):
     def updateDatasets(self, fields, helper):
 
         inds = helper.getDataset( fields['ds_in'] )
-        tot = N.nansum( inds.data )
+
+        data = inds.data
+        filtered = data[N.isfinite(data)]
+        if len(filtered) == 0:
+            tot = 0
+        else:
+            tot = N.sum(filtered)
 
         # divide data
-        data = inds.data / tot
+        data = data / tot
         # divide error bars
         serr = perr = nerr = None
-        if inds.serr: serr = inds.serr / tot
-        if inds.perr: perr = inds.perr / tot
-        if inds.nerr: nerr = inds.nerr / tot
+        if inds.serr is not None: serr = inds.serr / tot
+        if inds.perr is not None: perr = inds.perr / tot
+        if inds.nerr is not None: nerr = inds.nerr / tot
 
         self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
 
@@ -1412,7 +1484,7 @@ class FilterDatasetPlugin(_OneOutputDatasetPlugin):
     description_short = _('Filter a dataset using an expression')
     description_full = _('Filter a dataset using an expression, '
                          'e.g. "x>10" or "(x>1) & (y<2)"')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -1425,11 +1497,14 @@ class FilterDatasetPlugin(_OneOutputDatasetPlugin):
             ]
 
     def updateDatasets(self, fields, helper):
-        """Do shifting of dataset."""
+        """Do filtering of dataset."""
         ds_in = helper.getDataset(fields['ds_in'])
         filt = helper.evaluateExpression(fields['filter'])
 
-        data, serr, perr, nerr = ds_in.data, ds_in.serr, ds_in.perr, ds_in.nerr
+        data = ds_in.data
+        serr = getattr(ds_in, 'serr', None)
+        perr = getattr(ds_in, 'perr', None)
+        nerr = getattr(ds_in, 'nerr', None)
 
         if filt is None:
             # select nothing
@@ -1464,7 +1539,7 @@ class MovingAveragePlugin(_OneOutputDatasetPlugin):
     description_full = _('Compute moving average for regularly spaced data.'
                          'Average is computed either\nside of each data point '
                          'by number of points given.')
-    
+
     def __init__(self):
         """Define fields."""
         self.fields = [
@@ -1475,9 +1550,9 @@ class MovingAveragePlugin(_OneOutputDatasetPlugin):
                             default=True),
             field.FieldDataset('ds_out', _('Output dataset')),
             ]
-    
+
     def updateDatasets(self, fields, helper):
-        """Do shifting of dataset."""
+        """Compute moving average of dataset."""
         ds_in = helper.getDataset(fields['ds_in'])
         weights = None
         if fields['weighterrors']:
@@ -1511,7 +1586,7 @@ class LinearInterpolatePlugin(_OneOutputDatasetPlugin):
             ]
 
     def updateDatasets(self, fields, helper):
-        """Do shifting of dataset."""
+        """Compute linear interpolation of dataset."""
         ds_x = helper.getDataset(fields['ds_x']).data
         ds_y = helper.getDataset(fields['ds_y']).data
         ds_xprime = helper.getDataset(fields['ds_xprime']).data
@@ -1656,14 +1731,15 @@ class SortPlugin(_OneOutputDatasetPlugin):
         if fields['ds_sort'].strip():
             ds_sort = helper.getDataset(fields['ds_sort'])
 
-        idxs = N.argsort(ds_sort.data)
+        minlen = min(len(ds_sort.data), len(ds.data))
+        idxs = N.argsort(ds_sort.data[:minlen])
         if fields['reverse']:
             idxs = idxs[::-1]
 
-        out = { 'data': ds.data[idxs] }
-        if ds.serr is not None: out['serr'] = ds.serr[idxs]
-        if ds.perr is not None: out['perr'] = ds.perr[idxs]
-        if ds.nerr is not None: out['nerr'] = ds.nerr[idxs]
+        out = { 'data': ds.data[:minlen][idxs] }
+        if ds.serr is not None: out['serr'] = ds.serr[:minlen][idxs]
+        if ds.perr is not None: out['perr'] = ds.perr[:minlen][idxs]
+        if ds.nerr is not None: out['nerr'] = ds.nerr[:minlen][idxs]
 
         self.dsout.update(**out)
 
@@ -1696,7 +1772,10 @@ class SortTextPlugin(_OneOutputDatasetPlugin):
         ds = helper.getTextDataset(fields['ds_in']).data
         if fields['ds_sort'].strip():
             ds_sort = helper.getDataset(fields['ds_sort'])
-            idxs = N.argsort(ds_sort.data)
+            length = min(len(ds), len(ds_sort.data))
+            ds = ds[:length]
+
+            idxs = N.argsort(ds_sort.data[:length])
             dout = []
             for i in idxs:
                 dout.append(ds[i])
@@ -1851,6 +1930,200 @@ class ConvertNumbersToText(DatasetPlugin):
 
         self.dsout.update(data=data)
 
+class ClipPlugin(_OneOutputDatasetPlugin):
+    """Compute moving average for dataset."""
+
+    menu = (_('Compute'), _('Clipped dataset'),)
+    name = 'Clip'
+    description_short = _('Clip data between a minimum and maximum')
+    description_full = _('Clip data points to minimum and/or maximum values')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', _('Input dataset')),
+            field.FieldFloat('minimum', _('Minimum'), default=0.),
+            field.FieldBool('disablemin', _('Disable minimum')),
+            field.FieldFloat('maximum', _('Maximum'), default=1.),
+            field.FieldBool('disablemax', _('Disable maximum')),
+            field.FieldBool('cliperrs', _('Clip error bars'), default=True),
+            field.FieldDataset('ds_out', _('Output dataset')),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Do clipping of dataset."""
+        ds_in = helper.getDataset(fields['ds_in'])
+        data = N.array(ds_in.data)
+        perr = getattr(ds_in, 'perr')
+        nerr = getattr(ds_in, 'nerr')
+        serr = getattr(ds_in, 'serr')
+
+        cliperrs = fields['cliperrs']
+        # force asymmetric errors if clipping error bars
+        if cliperrs and serr is not None and (nerr is None or perr is None):
+            perr = serr
+            nerr = -serr
+            serr = None
+
+        # we have to clip the ranges, so calculate these first
+        upper = (data+perr) if (cliperrs and perr is not None) else None
+        lower = (data+nerr) if (cliperrs and nerr is not None) else None
+
+        # note: this preserves nan values
+        if not fields['disablemin']:
+            minv = fields['minimum']
+            data[data<minv] = minv
+            if upper is not None:
+                upper[upper<minv] = minv
+            if lower is not None:
+                lower[lower<minv] = minv
+        if not fields['disablemax']:
+            maxv = fields['maximum']
+            data[data>maxv] = maxv
+            if upper is not None:
+                upper[upper>maxv] = maxv
+            if lower is not None:
+                lower[lower>maxv] = maxv
+
+        if upper is not None:
+            perr = upper-data
+        if lower is not None:
+            nerr = lower-data
+
+        self.dsout.update(data=data, serr=serr, perr=perr, nerr=nerr)
+
+class LogPlugin(_OneOutputDatasetPlugin):
+    """Compute logarithm of data."""
+
+    menu = (_('Compute'), _('Log'),)
+    name = 'Logarithm'
+    description_short = _('Compute log of data')
+    description_full = _('Compute logarithm of data with arbitrary base')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', _('Input dataset')),
+            field.FieldFloat('base', _('Base'), default=10., minval=1e-10),
+            field.FieldDataset('ds_out', _('Output dataset')),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Compute log of dataset."""
+        ds_in = helper.getDataset(fields['ds_in'])
+        data = N.array(ds_in.data)
+        perr = getattr(ds_in, 'perr')
+        nerr = getattr(ds_in, 'nerr')
+        serr = getattr(ds_in, 'serr')
+
+        uppers = lowers = None
+        if perr is not None:
+            uppers = data + perr
+        elif serr is not None:
+            uppers = data + serr
+        if nerr is not None:
+            lowers = data + nerr
+        elif serr is not None:
+            lowers = data - serr
+
+        # convert base e to base given
+        invlogbase = 1. / N.log(fields['base'])
+
+        logdata = N.log(data) * invlogbase
+        logperr = None if uppers is None else N.log(uppers)*invlogbase - logdata
+        lognerr = None if lowers is None else N.log(lowers)*invlogbase - logdata
+
+        self.dsout.update(data=logdata, perr=logperr, nerr=lognerr)
+
+class ExpPlugin(_OneOutputDatasetPlugin):
+    """Compute exponential of data."""
+
+    menu = (_('Compute'), _('Exponential'),)
+    name = 'Exponential'
+    description_short = _('Compute exponential of data')
+    description_full = _('Compute exponential of data')
+
+    def __init__(self):
+        """Define fields."""
+        self.fields = [
+            field.FieldDataset('ds_in', _('Input dataset')),
+            field.FieldFloat('base', _('Base'), default=10., minval=1e-10),
+            field.FieldDataset('ds_out', _('Output dataset')),
+            ]
+
+    def updateDatasets(self, fields, helper):
+        """Compute exponential of dataset."""
+        ds_in = helper.getDataset(fields['ds_in'])
+        data = N.array(ds_in.data)
+        perr = getattr(ds_in, 'perr')
+        nerr = getattr(ds_in, 'nerr')
+        serr = getattr(ds_in, 'serr')
+
+        uppers = lowers = None
+        if perr is not None:
+            uppers = data + perr
+        elif serr is not None:
+            uppers = data + serr
+        if nerr is not None:
+            lowers = data + nerr
+        elif serr is not None:
+            lowers = data - serr
+
+        base = fields['base']
+        expdata = base**data
+        expperr = None if uppers is None else base**uppers - expdata
+        expnerr = None if lowers is None else base**lowers - expdata
+
+        self.dsout.update(data=expdata, perr=expperr, nerr=expnerr)
+
+class ReshapeDatasetPlugin(DatasetPlugin):
+    """Reshape a dataset to an n-dimensional dataset."""
+
+    menu = (_('Convert'), _('Reshape'))
+    name = 'Reshape'
+    description_short = _('Reshape to an aribtrary n-dimensional dataset')
+    description_full = _(
+        'Take a dataset with aribtrary shape and reshape into an '
+        'n-dimensional dataset with a shape given. '
+        'shape should be a space or comma-separated list of dimensions, '
+        'where -1 automatically calculates the length.')
+
+    def __init__(self):
+        self.fields = [
+            field.FieldDataset('ds_in', _('Input dataset'), dims='all'),
+            field.FieldText('shape', _('Shape')),
+            field.FieldBool('transpose', _('Transpose')),
+            field.FieldDataset('ds_out', _('Output dataset'), dims='all'),
+            ]
+
+    def getDatasets(self, fields):
+        if fields['ds_out'] == '':
+            raise DatasetPluginException(_('Invalid output dataset name'))
+        self.dsout = DatasetND(fields['ds_out'])
+        return [self.dsout]
+
+    def updateDatasets(self, fields, helper):
+        ds_in = helper.getDataset(fields['ds_in'], dimensions='all')
+
+        shapetxt = fields['shape']
+        shapesplit = re.split("[,;x* ]+", shapetxt)
+
+        try:
+            shape = tuple([int(x) for x in shapesplit])
+        except ValueError:
+            raise DatasetPluginException(
+                'Could not interpret shape text')
+        try:
+            data = ds_in.data.reshape(shape)
+        except ValueError:
+            raise DatasetPluginException(
+                'Cannot reshape: wrong number of dimensions')
+
+        if fields['transpose']:
+            data = N.transpose(data)
+
+        self.dsout.update(data)
+
 datasetpluginregistry += [
     AddDatasetPlugin,
     AddDatasetsPlugin,
@@ -1865,6 +2138,9 @@ datasetpluginregistry += [
     MeanDatasetPlugin,
     ExtremesDatasetPlugin,
     CumulativePlugin,
+    ClipPlugin,
+    LogPlugin,
+    ExpPlugin,
 
     ConcatenateDatasetPlugin,
     InterleaveDatasetPlugin,
@@ -1876,6 +2152,7 @@ datasetpluginregistry += [
 
     PolarToCartesianPlugin,
     ConvertNumbersToText,
+    ReshapeDatasetPlugin,
 
     FilterDatasetPlugin,
 
