@@ -67,7 +67,7 @@ class Edit(qt4.QLineEdit):
         qt4.QLineEdit.__init__(self, parent)
         self.setting = setting
 
-        self.setText( setting.toText() )
+        self.setText( setting.toUIText() )
         self.editingFinished.connect(self.validateAndSet)
         self.setting.setOnModified(self.onModified)
 
@@ -79,7 +79,7 @@ class Edit(qt4.QLineEdit):
 
         text = self.text()
         try:
-            val = self.setting.fromText(text)
+            val = self.setting.fromUIText(text)
             styleClear(self)
             self.sigSettingChanged.emit(self, self.setting, val)
 
@@ -89,7 +89,7 @@ class Edit(qt4.QLineEdit):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        self.setText( self.setting.toText() )
+        self.setText( self.setting.toUIText() )
 
 class _EditBox(qt4.QTextEdit):
     """A popup edit box to support editing long text sections.
@@ -169,7 +169,7 @@ class String(qt4.QWidget):
 
         layout = qt4.QHBoxLayout()
         layout.setSpacing(0)
-        layout.setMargin(0)
+        layout.setContentsMargins(0,0,0,0)
         self.setLayout(layout)
 
         self.edit = qt4.QLineEdit()
@@ -179,7 +179,7 @@ class String(qt4.QWidget):
         layout.addWidget(b)
 
         # set the text of the widget to the 
-        self.edit.setText( setting.toText() )
+        self.edit.setText( setting.toUIText() )
 
         self.edit.editingFinished.connect(self.validateAndSet)
         b.toggled.connect(self.buttonToggled)
@@ -218,7 +218,7 @@ class String(qt4.QWidget):
 
         text = self.edit.text()
         try:
-            val = self.setting.fromText(text)
+            val = self.setting.fromUIText(text)
             styleClear(self.edit)
             self.sigSettingChanged.emit(self, self.setting, val)
 
@@ -228,7 +228,7 @@ class String(qt4.QWidget):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        self.edit.setText( self.setting.toText() )
+        self.edit.setText( self.setting.toUIText() )
 
 class Int(qt4.QSpinBox):
     """A control for changing an integer."""
@@ -263,6 +263,60 @@ class Int(qt4.QSpinBox):
         self.setValue( self.setting.val )
         self.ignorechange = False
 
+class FloatSlider(qt4.QWidget):
+    """A slider control for a numerical value.
+    Note: QSlider is integer only, so dragging slider makes values integers
+    """
+
+    sigSettingChanged = qt4.pyqtSignal(qt4.QObject, object, object)
+
+    def __init__(self, setting, parent):
+        qt4.QWidget.__init__(self, parent)
+        self.setting = setting
+        self.setting.setOnModified(self.onModified)
+
+        layout = qt4.QHBoxLayout()
+        layout.setSpacing(0)
+        layout.setContentsMargins(0, 0, 0, 0)
+        self.setLayout(layout)
+
+        s = self.slider = qt4.QSlider(qt4.Qt.Horizontal)
+        s.setMinimum(int(setting.minval/setting.scale))
+        s.setMaximum(int(setting.maxval/setting.scale))
+        s.setPageStep(int(setting.step/setting.scale))
+        s.setTickInterval(int(setting.tick/setting.scale))
+        s.setTickPosition(qt4.QSlider.TicksAbove)
+        layout.addWidget(self.slider)
+
+        self.edit = qt4.QLineEdit()
+        layout.addWidget(self.edit)
+
+        self.edit.editingFinished.connect(self.validateAndSet)
+        self.slider.valueChanged.connect(self.movedPosition)
+
+        self.onModified()
+
+    def validateAndSet(self):
+        """Validate text is numeric."""
+        try:
+            val = self.setting.fromUIText(self.edit.text())
+            styleClear(self.edit)
+            self.sigSettingChanged.emit(self, self.setting, val)
+
+        except utils.InvalidType:
+            styleError(self.edit)
+
+    def movedPosition(self, val):
+        """Someone dragged the slider."""
+        self.sigSettingChanged.emit(
+            self, self.setting, float(val)*self.setting.scale)
+
+    @qt4.pyqtSlot()
+    def onModified(self):
+        self.edit.setText(self.setting.toUIText())
+        self.slider.setValue(int(round(
+            self.setting.get()/self.setting.scale)))
+
 class Bool(qt4.QCheckBox):
     """A check box for changing a bool setting."""
 
@@ -327,23 +381,28 @@ class Choice(qt4.QComboBox):
     sigSettingChanged = qt4.pyqtSignal(qt4.QObject, object, object)
 
     def __init__(self, setting, iseditable, vallist, parent, icons=None,
+                 uilist=None,
                  descriptions=None):
         qt4.QComboBox.__init__(self, parent)
 
         self.setting = setting
-
+        self.vallist = vallist
+        self.uilist = uilist
         self.setEditable(iseditable)
 
         # stops combobox readjusting in size to fit contents
         self.setSizeAdjustPolicy(
             qt4.QComboBox.AdjustToMinimumContentsLengthWithIcon)
 
+        # whether to show ui text to replace some items
+        toadd = uilist if uilist is not None else vallist
+
         if icons is None:
             # add items to list (text only)
-            self.addItems( list(vallist) )
+            self.addItems( list(toadd) )
         else:
             # add pixmaps and text to list
-            for icon, text in czip(icons, vallist):
+            for icon, text in czip(icons, toadd):
                 self.addItem(icon, text)
 
         # use tooltip descriptions if requested
@@ -353,13 +412,13 @@ class Choice(qt4.QComboBox):
 
         # choose the correct setting
         try:
-            index = list(vallist).index(setting.toText())
+            index = list(vallist).index(setting.toUIText())
             self.setCurrentIndex(index)
         except ValueError:
             # for cases when this is editable
             # set the text of the widget to the setting
             assert iseditable
-            self.setEditText( setting.toText() )
+            self.setEditText( setting.toUIText() )
 
         # if a different item is selected
         self.activated[str].connect(self.slotActivated)
@@ -382,8 +441,15 @@ class Choice(qt4.QComboBox):
         """If a different item is chosen."""
 
         text = self.currentText()
+
+        # convert to ui text if set
+        if self.uilist is not None:
+            idx = self.uilist.index(text)
+            if idx >= 0:
+                text = self.vallist[idx]
+
         try:
-            val = self.setting.fromText(text)
+            val = self.setting.fromUIText(text)
             styleClear(self)
             self.sigSettingChanged.emit(self, self.setting, val)
 
@@ -393,7 +459,14 @@ class Choice(qt4.QComboBox):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        text = self.setting.toText()
+        text = self.setting.toUIText()
+
+        # convert to ui text
+        if self.uilist is not None:
+            idx = self.vallist.index(text)
+            if idx >= 0:
+                text = self.uilist[idx]
+
         index = self.findText(text)
         if index >= 0:
             self.setCurrentIndex(index)
@@ -415,12 +488,7 @@ class ChoiceSwitch(Choice):
 
     def updateState(self):
         """Set hidden state of settings."""
-        s1, s2 = self.setting.strue, self.setting.sfalse
-        if self.setting.showfn(self.setting.val):
-            show, hide = s1, s2
-        else:
-            show, hide = s2, s1
-
+        show, hide = self.setting.showfn(self.setting.val)
         if hasattr(self.parent(), 'showHideSettings'):
             self.parent().showHideSettings(show, hide)
 
@@ -441,6 +509,7 @@ class FillStyleExtended(ChoiceSwitch):
     def _generateIcons(cls):
         """Generate a list of pixmaps for drop down menu."""
 
+        from .. import document
         from . import collections
         brush = collections.BrushExtended("")
         brush.color = 'black'
@@ -453,17 +522,19 @@ class FillStyleExtended(ChoiceSwitch):
         path = qt4.QPainterPath()
         path.addRect(0, 0, size, size)
 
+        doc = document.Document()
+        phelper = document.PaintHelper(doc, (1,1))
+
         for f in utils.extfillstyles:
             pix = qt4.QPixmap(size, size)
             pix.fill()
-            painter = qt4.QPainter(pix)
-            painter.pixperpt = 1.
-            painter.scaling = 1.
+            painter = document.DirectPainter(pix)
             painter.setRenderHint(qt4.QPainter.Antialiasing)
+            painter.updateMetaData(phelper)
             brush.style = f
             utils.brushExtFillPath(painter, brush, path)
             painter.end()
-            icons.append( qt4.QIcon(pix) )
+            icons.append(qt4.QIcon(pix))
 
 class MultiLine(qt4.QTextEdit):
     """For editting multi-line settings."""
@@ -480,7 +551,7 @@ class MultiLine(qt4.QTextEdit):
         self.setTabChangesFocus(True)
         
         # set the text of the widget to the 
-        self.setPlainText( setting.toText() )
+        self.setPlainText( setting.toUIText() )
 
         self.setting.setOnModified(self.onModified)
 
@@ -511,7 +582,7 @@ class MultiLine(qt4.QTextEdit):
 
         text = self.toPlainText()
         try:
-            val = self.setting.fromText(text)
+            val = self.setting.fromUIText(text)
             styleClear(self)
             self.sigSettingChanged.emit(self, self.setting, val)
 
@@ -521,7 +592,7 @@ class MultiLine(qt4.QTextEdit):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        self.setPlainText( self.setting.toText() )
+        self.setPlainText( self.setting.toUIText() )
 
 class Notes(MultiLine):
     """For editing notes."""
@@ -638,7 +709,7 @@ class Dataset(qt4.QWidget):
 
         layout = qt4.QHBoxLayout()
         layout.setSpacing(0)
-        layout.setMargin(0)
+        layout.setContentsMargins(0,0,0,0)
         layout.addWidget(self.choice)
         layout.addWidget(b)
         self.setLayout(layout)
@@ -649,10 +720,12 @@ class Dataset(qt4.QWidget):
         # get datasets of the correct dimension
         datasets = []
         for name, ds in citems(self.document.data):
-            if ds.dimensions == self.dimensions and (
-                ds.datatype == self.datatype or
-                self.datatype == 'all' or
-                ds.datatype in self.datatype ):
+            if ( (ds.dimensions == self.dimensions or
+                  self.dimensions == 'all') and (
+                      ds.datatype == self.datatype or
+                      self.datatype == 'all' or
+                      ds.datatype in self.datatype)
+            ):
                 datasets.append(name)
         datasets.sort()
 
@@ -853,20 +926,23 @@ class LineStyle(Choice):
         setn.get('color').set('black')
         setn.get('width').set('1pt')
         
+        doc = document.Document()
         for lstyle in cls._lines:
             pix = qt4.QPixmap(*size)
             pix.fill()
 
-            ph = document.PaintHelper( (1, 1) )
-
-            painter = qt4.QPainter(pix)
+            painter = document.DirectPainter(pix)
             painter.setRenderHint(qt4.QPainter.Antialiasing)
 
+            phelper = document.PaintHelper(doc, (1, 1))
+            painter.updateMetaData(phelper)
+
             setn.get('style').set(lstyle)
             
-            painter.setPen( setn.makeQPen(ph) )
-            painter.drawLine( int(size[0]*0.1), size[1]/2,
-                              int(size[0]*0.9), size[1]/2 )
+            painter.setPen( setn.makeQPen(painter) )
+            painter.drawLine(
+                int(size[0]*0.1), size[1]//2,
+                int(size[0]*0.9), size[1]//2)
             painter.end()
             icons.append( qt4.QIcon(pix) )
 
@@ -875,6 +951,7 @@ class LineStyle(Choice):
 class _ColNotifier(qt4.QObject):
     sigNewColor = qt4.pyqtSignal(cstr)
 
+
 class Color(qt4.QWidget):
     """A control which lets the user choose a color.
 
@@ -883,28 +960,18 @@ class Color(qt4.QWidget):
 
     sigSettingChanged = qt4.pyqtSignal(qt4.QObject, object, object)
 
-    _icons = None
-    _colors = None
-    _colnotifier = None
-
-    def __init__(self, setting,  parent):
+    def __init__(self, setting, parent):
         qt4.QWidget.__init__(self, parent)
 
-        if self._icons is None:
-            self._generateIcons()
-
         self.setting = setting
- 
+        self.document = setting.getDocument()
+        self.colors = self.document.evaluate.colors
+
         # combo box
         c = self.combo = qt4.QComboBox()
         c.setEditable(True)
-        for color in self._colors:
-            c.addItem(self._icons[color], color)
         c.activated[str].connect(self.slotActivated)
 
-        # add color if a color is added by a different combo box
-        Color._colnotifier.sigNewColor.connect(self.addcolorSlot)
-
         # button for selecting colors
         b = self.button = qt4.QPushButton()
         b.setFlat(True)
@@ -913,82 +980,56 @@ class Color(qt4.QWidget):
         b.setMaximumWidth(24)
         b.clicked.connect(self.slotButtonClicked)
 
+        c.setModel(self.colors.model)
+        self.setColor(self.setting.val)
+
         if setting.readonly:
             c.setEnabled(False)
             b.setEnabled(False)
-                     
+
         layout = qt4.QHBoxLayout()
         layout.setSpacing(0)
-        layout.setMargin(0)
+        layout.setContentsMargins(0,0,0,0)
         layout.addWidget(c)
         layout.addWidget(b)
 
-        self.setColor( setting.toText() )
+        self.setColor(setting.toUIText())
         self.setLayout(layout)
         self.setting.setOnModified(self.onModified)
 
-    def addcolorSlot(self, color):
-        """When another Color combo adds a color, add one to this one"""
-        self.combo.addItem(self._icons[color], color)
-
-    @classmethod
-    def _generateIcons(cls):
-        """Generate a list of icons for drop down menu.
-        Does not generate existing icons
-        """
-
-        size = 12
-        if cls._icons is None:
-            cls._icons = {}
-        
-        icons = cls._icons
-        for c in cls._colors:
-            if c not in icons:
-                pix = qt4.QPixmap(size, size)
-                pix.fill( qt4.QColor(c) )
-                icons[c] = qt4.QIcon(pix)
-                if cls._colnotifier is not None:
-                    # tell other combo boxes a color has been added
-                    cls._colnotifier.sigNewColor.emit(c)
-
-        if cls._colnotifier is None:
-            cls._colnotifier = _ColNotifier()
-    
     def slotButtonClicked(self):
         """Open dialog to edit color."""
 
-        col = qt4.QColorDialog.getColor(self.setting.color(), self)
+        col = qt4.QColorDialog.getColor(
+            self.colors.get(self.setting.val), self)
         if col.isValid():
             # change setting
             val = col.name()
             self.sigSettingChanged.emit(self, self.setting, val)
 
-    def slotActivated(self, val):
+    def slotActivated(self, text):
         """A different value is selected."""
-        
-        text = self.combo.currentText()
-        val = self.setting.fromText(text)
+        val = self.setting.fromUIText(text)
         self.sigSettingChanged.emit(self, self.setting, val)
 
     def setColor(self, color):
         """Update control with color given."""
 
-        # construct color icon if not there
-        if color not in Color._icons:
-            Color._colors.append(color)
-            Color._generateIcons()
-        
-        # add text to combo if not there
         index = self.combo.findText(color)
+        if index < 0:
+            # add text to combo if not there
+            self.combo.addItem(color)
+            index = self.combo.findText(color)
 
-        # set correct index in combobox
         self.combo.setCurrentIndex(index)
-        self.button.setIcon( self.combo.itemIcon(index) )
+
+        icon = self.combo.itemData(index, qt4.Qt.DecorationRole)
+        self.button.setIcon(icon)
 
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        self.setColor( self.setting.toText() )
+        self.setColor( self.setting.toUIText() )
 
 class WidgetSelector(Choice):
     """For choosing from a list of widgets."""
@@ -1043,7 +1084,7 @@ class Axis(WidgetSelector):
 
         # get parent widget
         widget = self.setting.parent
-        while not widget.isWidget() and widget is not None:
+        while not widget.iswidget and widget is not None:
             widget = widget.parent
 
         # get list of axis widgets up the tree
@@ -1081,32 +1122,40 @@ class ListSet(qt4.QFrame):
 
         parent is the parent widget.
         """
-        
+
         qt4.QFrame.__init__(self, parent)
         self.setFrameStyle(qt4.QFrame.Box)
         self.defaultval = defaultval
         self.setting = setting
         self.controls = []
         self.layout = qt4.QGridLayout(self)
-        self.layout.setMargin( self.layout.margin()//2 )
+        s = self.layout.contentsMargins().left()//2
+        self.layout.setContentsMargins(s,s,s,s)
         self.layout.setSpacing( self.layout.spacing()//4 )
 
-        # ignore changes if this set
-        self.ignorechange = False
-
-        self.populate()
+        self.doneinit = False
+        self.onModified()
         self.setting.setOnModified(self.onModified)
-    
-    def populateRow(self, row, val):
+
+    def populateRow(self, row):
         """Populate the row in the control.
 
         Returns a list of the widgets created.
         """
-        return None
-    
+        return []
+
+    def updateRow(self, cntrls, val):
+        """Set controls on row, given values for row, val."""
+        pass
+
     def populate(self):
         """Construct the list of controls."""
 
+        if len(self.setting.val) == len(self.controls) and self.doneinit:
+            # no change in number of controls
+            return
+        self.doneinit = True   # we need to add the add/remove buttons below
+
         # delete all children in case of refresh
         self.controls = []
         for c in self.children():
@@ -1118,7 +1167,8 @@ class ListSet(qt4.QFrame):
         # iterate over each row
         row = -1
         for row, val in enumerate(self.setting.val):
-            cntrls = self.populateRow(row, val)
+            cntrls = self.populateRow(row)
+            self.updateRow(cntrls, val)
             for col in crange(len(cntrls)):
                 self.layout.addWidget(cntrls[col], row, col)
             for c in cntrls:
@@ -1128,10 +1178,10 @@ class ListSet(qt4.QFrame):
         # buttons at end
         bbox = qt4.QWidget()
         h = qt4.QHBoxLayout(bbox)
-        h.setMargin(0)
+        h.setContentsMargins(0,0,0,0)
         bbox.setLayout(h)
         self.layout.addWidget(bbox, row+1, 0, 1, -1)
-        
+
         # a button to add a new entry
         b = qt4.QPushButton('Add')
         h.addWidget(b)
@@ -1142,7 +1192,7 @@ class ListSet(qt4.QFrame):
         b = qt4.QPushButton('Delete')
         h.addWidget(b)
         b.clicked.connect(self.onDeleteClicked)
-        b.setEnabled( len(self.setting.val) > 0 )
+        b.setEnabled(len(self.setting.val) > 0)
         b.show()
 
     def onAddClicked(self):
@@ -1164,11 +1214,9 @@ class ListSet(qt4.QFrame):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-
-        if not self.ignorechange:
-            self.populate()
-        else:
-            self.ignorechange = False
+        self.populate()
+        for cntrls, row in czip(self.controls, self.setting.val):
+            self.updateRow(cntrls, row)
 
     def identifyPosn(self, widget):
         """Identify the position this widget is in.
@@ -1182,40 +1230,42 @@ class ListSet(qt4.QFrame):
                     return (row, col)
         return (None, None)
 
-    def addColorButton(self, row, col, tooltip):
+    def addColorButton(self, tooltip):
         """Add a color button to the list at the position specified."""
-
-        color = self.setting.val[row][col]
         wcolor = qt4.QPushButton()
         wcolor.setFlat(True)
         wcolor.setSizePolicy(qt4.QSizePolicy.Maximum, qt4.QSizePolicy.Maximum)
         wcolor.setMaximumHeight(24)
         wcolor.setMaximumWidth(24)
-
-        pix = qt4.QPixmap(self.pixsize, self.pixsize)
-        pix.fill( utils.extendedColorToQColor(color) )
-        wcolor.setIcon( qt4.QIcon(pix) )
         wcolor.setToolTip(tooltip)
-
         wcolor.clicked.connect(self.onColorClicked)
         return wcolor
 
-    def addToggleButton(self, row, col, tooltip):
-        """Make a toggle button."""
+    def updateColorButton(self, cntrl, color):
+        """Given color control, update color."""
 
-        toggle = self.setting.val[row][col]
+        pix = qt4.QPixmap(self.pixsize, self.pixsize)
+        qcolor = self.setting.getDocument().evaluate.colors.get(color)
+        pix.fill(qcolor)
+        cntrl.setIcon(qt4.QIcon(pix))
+
+    def addToggleButton(self, tooltip):
+        """Make a toggle button."""
         wtoggle = qt4.QCheckBox()
-        wtoggle.setChecked(toggle)
         wtoggle.setToolTip(tooltip)
         wtoggle.toggled.connect(self.onToggled)
         return wtoggle
 
-    def addCombo(self, row, col, tooltip, values, icons, texts):
+    def updateToggleButton(self, cntrl, val):
+        """Update toggle with value."""
+        cntrl.setChecked(val)
+
+    def addCombo(self, tooltip, values, icons, texts):
         """Make an enumeration combo - choose from a set of icons."""
-        
-        val = self.setting.val[row][col]
 
         wcombo = qt4.QComboBox()
+        wcombo.setToolTip(tooltip)
+        wcombo.activated[int].connect(self.onComboChanged)
 
         if texts is None:
             for icon in icons:
@@ -1224,31 +1274,33 @@ class ListSet(qt4.QFrame):
             for text, icon in czip(texts, icons):
                 wcombo.addItem(icon, text)
 
-        wcombo.setCurrentIndex(values.index(val))
-        wcombo.setToolTip(tooltip)
-        wcombo.activated[int].connect(self.onComboChanged)
         wcombo._vz_values = values
         return wcombo
 
+    def updateCombo(self, cntrl, val):
+        """Update selected item in combo."""
+        cntrl.setCurrentIndex(cntrl._vz_values.index(val))
+
     def _updateRowCol(self, row, col, val):
         """Update value on row and column."""
         rows = list(self.setting.val)
         items = list(rows[row])
         items[col] = val
         rows[row] = tuple(items)
-        self.ignorechange = True
         self.sigSettingChanged.emit(self, self.setting, rows)
 
     def onToggled(self, on):
         """Checkbox toggled."""
         row, col = self.identifyPosn(self.sender())
-        self._updateRowCol(row, col, on)
+        if row is not None:
+            self._updateRowCol(row, col, on)
 
     def onComboChanged(self, val):
         """Update the setting if the combo changes."""
         sender = self.sender()
         row, col = self.identifyPosn(sender)
-        self._updateRowCol(row, col, sender._vz_values[val])
+        if row is not None:
+            self._updateRowCol(row, col, sender._vz_values[val])
 
     def onColorClicked(self):
         """Color button clicked for line."""
@@ -1256,8 +1308,10 @@ class ListSet(qt4.QFrame):
         row, col = self.identifyPosn(sender)
 
         rows = self.setting.val
+        qcolor = self.setting.getDocument().evaluate.colors.get(
+            rows[row][col])
         color = qt4.QColorDialog.getColor(
-            utils.extendedColorToQColor(rows[row][col]),
+            qcolor,
             self,
             "Choose color",
             qt4.QColorDialog.ShowAlphaChannel )
@@ -1270,18 +1324,19 @@ class ListSet(qt4.QFrame):
 
             # change the color
             pix = qt4.QPixmap(self.pixsize, self.pixsize)
-            pix.fill( utils.extendedColorToQColor(color) )
-            sender.setIcon( qt4.QIcon(pix) )
-            
+            qcolor = self.setting.getDocument().evaluate.colors.get(color)
+            pix.fill(qcolor)
+            sender.setIcon(qt4.QIcon(pix))
+
 class LineSet(ListSet):
     """A list of line styles.
     """
 
     def __init__(self, setting, parent):
-        ListSet.__init__(self, ('solid', '1pt', 'black', False),
-                         setting, parent)
+        ListSet.__init__(
+            self, ('solid', '1pt', 'black', False), setting, parent)
 
-    def populateRow(self, row, val):
+    def populateRow(self, row):
         """Add the widgets for the row given."""
 
         # create line icons if not already created
@@ -1289,25 +1344,30 @@ class LineSet(ListSet):
             LineStyle._generateIcons()
 
         # make line style selector
-        wlinestyle = self.addCombo(row, 0, _('Line style'),
-                                   LineStyle._lines,
-                                   LineStyle._icons, None)
-        
+        wlinestyle = self.addCombo(
+            _('Line style'), LineStyle._lines, LineStyle._icons, None)
+
         # make line width edit box
         wwidth = qt4.QLineEdit()
-        wwidth.setText(self.setting.val[row][1])
-        wwidth.setToolTip('Line width')
+        wwidth.setToolTip(_('Line width'))
         wwidth.editingFinished.connect(self.onWidthChanged)
 
         # make color selector button
-        wcolor = self.addColorButton(row, 2, _('Line color'))
+        wcolor = self.addColorButton(_('Line color'))
 
         # make hide checkbox
-        whide = self.addToggleButton(row, 3, _('Hide line'))
+        whide = self.addToggleButton( _('Hide line'))
 
         # return created controls
         return [wlinestyle, wwidth, wcolor, whide]
 
+    def updateRow(self, cntrls, val):
+        """Update controls with row settings."""
+        self.updateCombo(cntrls[0], val[0])
+        cntrls[1].setText(val[1])
+        self.updateColorButton(cntrls[2], val[2])
+        self.updateToggleButton(cntrls[3], val[3])
+
     def onWidthChanged(self):
         """Width has changed - validate."""
 
@@ -1346,8 +1406,10 @@ class _FillBox(qt4.QScrollArea):
 
         self.extbrush = thesetting.returnBrushExtended(row)
 
-        from ..windows.treeeditwindow import SettingsProxySingle, \
-            PropertyList
+        # need to add a real parent, so that the colors can be resolved
+        self.extbrush.parent = thesetting.parent
+
+        from ..windows.treeeditwindow import SettingsProxySingle, PropertyList
 
         fbox = self
         class DirectSetProxy(SettingsProxySingle):
@@ -1372,7 +1434,7 @@ class _FillBox(qt4.QScrollArea):
         # get value of brush and get data for row
         e = self.extbrush
         rowdata = [e.style, e.color, e.hide]
-        if e.style != 'solid':
+        if e.style != 'solid' or e.transparency > 0:
             rowdata += [
                 e.transparency, e.linewidth, e.linestyle,
                 e.patternspacing, e.backcolor,
@@ -1414,28 +1476,27 @@ class FillSet(ListSet):
     """A list of fill settings."""
 
     def __init__(self, setting, parent):
-        ListSet.__init__(self, ('solid', 'black', False),
-                         setting, parent)
+        ListSet.__init__(
+            self, ('solid', 'black', False), setting, parent)
 
-    def populateRow(self, row, val):
+    def populateRow(self, row):
         """Add the widgets for the row given."""
 
         # construct fill icons if not already done
-        if FillStyle._icons is None:
-            FillStyle._generateIcons()
+        if FillStyleExtended._icons is None:
+            FillStyleExtended._generateIcons()
     
         # make fill style selector
-        wfillstyle = self.addCombo(row, 0, _("Fill style"),
-                                   FillStyle._fills,
-                                   FillStyle._icons,
-                                   FillStyle._fills)
+        wfillstyle = self.addCombo(
+            _("Fill style"),
+            utils.extfillstyles, FillStyleExtended._icons, utils.extfillstyles)
         wfillstyle.setMinimumWidth(self.pixsize)
 
         # make color selector button
-        wcolor = self.addColorButton(row, 1, _("Fill color"))
+        wcolor = self.addColorButton(_("Fill color"))
 
         # make hide checkbox
-        whide = self.addToggleButton(row, 2, _("Hide fill"))
+        whide = self.addToggleButton(_("Hide fill"))
 
         # extended options
         wmore = DotDotButton(tooltip=_("More options"))
@@ -1444,6 +1505,11 @@ class FillSet(ListSet):
         # return widgets
         return [wfillstyle, wcolor, whide, wmore]
 
+    def updateRow(self, cntrls, val):
+        self.updateCombo(cntrls[0], val[0])
+        self.updateColorButton(cntrls[1], val[1])
+        self.updateToggleButton(cntrls[2], val[2])
+
     def buttonAtRow(self, row):
         """Get .. button on row."""
         return self.layout.itemAtPosition(row, 3).widget()
@@ -1477,7 +1543,7 @@ class MultiSettingWidget(qt4.QWidget):
 
         self.grid = layout = qt4.QGridLayout()
         layout.setHorizontalSpacing(0)
-        layout.setContentsMargins(0, 0, 0, 0)
+        layout.setContentsMargins(0,0,0,0)
         self.setLayout(layout)
 
         self.last = ()
@@ -1687,12 +1753,12 @@ class Filename(qt4.QWidget):
 
         layout = qt4.QHBoxLayout()
         layout.setSpacing(0)
-        layout.setMargin(0)
+        layout.setContentsMargins(0,0,0,0)
         self.setLayout(layout)
 
         # the actual edit control
         self.edit = qt4.QLineEdit()
-        self.edit.setText( setting.toText() )
+        self.edit.setText( setting.toUIText() )
         layout.addWidget(self.edit)
         
         b = self.button = DotDotButton(checkable=False,
@@ -1725,10 +1791,11 @@ class Filename(qt4.QWidget):
             filefilter = ("Images (*.png *.jpg *.jpeg *.bmp *.svg *.tiff *.tif "
                           "*.gif *.xbm *.xpm);;" + filefilter)
 
-        filename = qt4.QFileDialog.getOpenFileName(
+        retn = qt4.QFileDialog.getOpenFileName(
             self, title, self.edit.text(), filefilter)
 
-        if filename:
+        if retn:
+            filename = retn[0]
             self.sigSettingChanged.emit(self, self.setting, filename)
 
     def validateAndSet(self):
@@ -1736,7 +1803,7 @@ class Filename(qt4.QWidget):
 
         text = self.edit.text()
         try:
-            val = self.setting.fromText(text)
+            val = self.setting.fromUIText(text)
             styleClear(self.edit)
             self.sigSettingChanged.emit(self, self.setting, val)
 
@@ -1746,7 +1813,7 @@ class Filename(qt4.QWidget):
     @qt4.pyqtSlot()
     def onModified(self):
         """called when the setting is changed remotely"""
-        self.edit.setText( self.setting.toText() )
+        self.edit.setText( self.setting.toUIText() )
 
 class FontFamily(qt4.QFontComboBox):
     """List the font families, showing each font."""
@@ -1785,7 +1852,7 @@ class FontFamily(qt4.QFontComboBox):
     @qt4.pyqtSlot()
     def onModified(self):
         """Make control reflect chosen setting."""
-        self.setCurrentFont( qt4.QFont(self.setting.toText()) )
+        self.setCurrentFont( qt4.QFont(self.setting.toUIText()) )
 
 class ErrorStyle(Choice):
     """Choose different error bar styles."""
@@ -1819,7 +1886,7 @@ class Colormap(Choice):
     size = (32, 12)
 
     def __init__(self, setn, document, parent):
-        names = sorted(document.colormaps)
+        names = sorted(document.evaluate.colormaps)
 
         icons = Colormap._generateIcons(document, names)
         Choice.__init__(self, setn, True,
@@ -1840,9 +1907,9 @@ class Colormap(Choice):
 
         # iterate over colour maps
         for name in names:
-            val = document.colormaps.get(name, None)
-            if val in kls._icons:
-                icon = kls._icons[val]
+            val = document.evaluate.colormaps.get(name, None)
+            if name in kls._icons:
+                icon = kls._icons[name]
             else:
                 if val is None:
                     # empty icon
@@ -1855,7 +1922,7 @@ class Colormap(Choice):
                                                 0., size[0]-1., 0)
                     pixmap = qt4.QPixmap.fromImage(image)
                 icon = qt4.QIcon(pixmap)
-                kls._icons[val] = icon
+                kls._icons[name] = icon
             retn.append(icon)
         return retn
 
@@ -1877,4 +1944,4 @@ class AxisBound(Choice):
         Re-set text as float or date."""
 
         if self.currentText().lower() != 'auto':
-            self.setEditText( self.setting.toText() )
+            self.setEditText( self.setting.toUIText() )
