@@ -23,6 +23,7 @@ from __future__ import division
 import sys
 import string
 import re
+import os
 import os.path
 import threading
 import codecs
@@ -33,7 +34,7 @@ from collections import defaultdict
 
 from ..compat import citems, cstr, CStringIO, cbasestr, cpy3, cbytes, crepr, \
     crange
-from .. import qtall as qt4
+from .. import qtall as qt
 import numpy as N
 
 class IgnoreException(Exception):
@@ -46,11 +47,8 @@ def _getVeuszDirectory():
     """Get resource and examples directories for Veusz."""
 
     if hasattr(sys, 'frozen'):
-        # for pyinstaller/py2app compatability
+        # for pyinstaller compatability
         resdir = os.path.dirname(os.path.abspath(sys.executable))
-        if sys.platform == 'darwin':
-            # special case for py2app
-            resdir = os.path.join(resdir, '..', 'Resources')
     else:
         # standard installation
         resdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
@@ -130,29 +128,19 @@ def relpath(filename, dirname):
     # join parts back together
     return os.path.sep.join(fileparts)
 
-# handle and extended #RRGGBBAA color
-extendedcolor_re = re.compile('^#[0-9A-Fa-f]{8}$')
-def extendedColorToQColor(s):
-    if extendedcolor_re.match(s):
-        col = qt4.QColor(s[:-2])
-        col.setAlpha( int(s[-2:], 16) )
-        return col
-    else:
-        return qt4.QColor(s)
-
 def extendedColorFromQColor(col):
     """Make an extended color #RRGGBBAA or #RRGGBB string."""
     if col.alpha() == 255:
         return str(col.name())
     else:
-        return '#%02x%02x%02x%02x' % (col.red(), col.green(), col.blue(),
-                                      col.alpha())
+        return '#%02x%02x%02x%02x' % (
+            col.red(), col.green(), col.blue(), col.alpha())
 
 def pixmapAsHtml(pix):
     """Get QPixmap as html image text."""
-    ba = qt4.QByteArray()
-    buf = qt4.QBuffer(ba)
-    buf.open(qt4.QIODevice.WriteOnly)
+    ba = qt.QByteArray()
+    buf = qt.QBuffer(ba)
+    buf.open(qt.QIODevice.WriteOnly)
     pix.toImage().save(buf, "PNG")
     b64 = cbytes(buf.data().toBase64()).decode('ascii')
     return '<img src="data:image/png;base64,%s">' % b64
@@ -164,7 +152,7 @@ def pythonise(text):
     to make it easier for users. We also have to take account of quotes
     and backslashes.
     """
-    
+
     out = ''
     insingle = False    # in a single quote section
     indouble = False    # in a double quote section
@@ -334,7 +322,7 @@ def openEncoding(filename, encoding, mode='r'):
     instead.
     """
     if filename == '{clipboard}':
-        text = qt4.QApplication.clipboard().text()
+        text = qt.QApplication.clipboard().text()
         return CStringIO(text)
     else:
         return io.open(filename, mode, encoding=encoding, errors='ignore')
@@ -381,7 +369,7 @@ def get_unicode_csv_reader(filename, dialect=csv.excel,
             f = _UTF8Recoder(open(filename), encoding)
     else:
         # take the unicode clipboard and just put into utf-8 format
-        s = qt4.QApplication.clipboard().text()
+        s = qt.QApplication.clipboard().text()
         if not cpy3:
             s = s.encode("utf-8")
         f = CStringIO(s)
@@ -427,7 +415,7 @@ def populateCombo(combo, items):
 def positionFloatingPopup(popup, widget):
     """Position a popped up window (popup) to side and below widget given."""
     pos = widget.parentWidget().mapToGlobal( widget.pos() )
-    desktop = qt4.QApplication.desktop()
+    desktop = qt.QApplication.desktop()
 
     # recalculates out position so that size is correct below
     popup.adjustSize()
@@ -452,21 +440,6 @@ def positionFloatingPopup(popup, widget):
     popup.move(x, y)
     popup.setFocus()
 
-def unique(inlist):
-    """Get unique entries in list."""
-
-    inlist = list(inlist)
-    inlist.sort()
-
-    class X: pass
-    last = X()
-    out = []
-    for x in inlist:
-        if x != last:
-            out.append(x)
-            last = x
-    return out
-
 # based on http://stackoverflow.com/questions/10607841/algorithm-for-topological-sorting-if-cycles-exist
 def topological_sort(dependency_pairs):
     """Given a list of pairs, perform a topological sort.
@@ -552,6 +525,21 @@ def round2delt(fin1, fin2):
     fout = float(out1)
     return fout if fin1 > 0 else -fout
 
+def checkOrder(inv):
+    """Check order of inv
+    Returns: +1: ascending order
+             -1: descending order
+              0: other, or non finite."""
+    v = N.array(inv)
+    if not N.all(N.isfinite(v)):
+        return 0
+    delta = v[1:] - v[:-1]
+    if N.all(delta > 0):
+        return 1
+    if N.all(delta < 0):
+        return -1
+    return 0
+
 def checkAscending(v):
     """Check list of values is finite and ascending."""
     v = N.array(v)
@@ -591,3 +579,62 @@ def unescapeHDFDataName(name):
     name = name.replace('`SL', '/')
     name = name.replace('`BT', '`')
     return name
+
+def allNotNone(*items):
+    """Are all the items not None."""
+    return not any((x is None for x in items))
+
+def findOnPath(cmd):
+    """Find a command on the system path, or None if does not exist."""
+    path = os.getenv('PATH', os.path.defpath)
+    pathparts = path.split(os.path.pathsep)
+    for dirname in pathparts:
+        dirname = dirname.strip('"')
+        cmdtry = os.path.join(dirname, cmd)
+        if os.path.isfile(cmdtry) and os.access(cmdtry, os.X_OK):
+            return cmdtry
+    return None
+
+class Struct:
+    """Simple structure-like class."""
+
+    def __init__(self, **args):
+        self.__dict__.update(args)
+
+    def __repr__(self):
+        return '<%s>' % str(
+            ', '.join('%s:%s' % (k, repr(getattr(self, k)))
+                       for k in sorted(self.__dict__)))
+
+class SvgWidgetFixedAspect(qt.QWidget):
+    """Draw an SVG file with the aspect ratio fixed to the original."""
+
+    def __init__(self, filename, *args):
+        qt.QWidget.__init__(self, *args)
+        self.renderer = qt.QSvgRenderer(filename, self)
+        self.defwidth = self.renderer.defaultSize().width()
+        self.defheight = self.renderer.defaultSize().height()
+
+    def sizeHint(self):
+        return qt.QSize(self.defwidth, self.defheight)
+
+    def paintEvent(self, evt):
+        painter = qt.QPainter(self)
+
+        w = self.width()
+        h = self.height()
+
+        ratio_x = w / self.defwidth
+        ratio_y = h / self.defheight
+        if ratio_x < ratio_y:
+            outw = w
+            outh = w*self.defheight//self.defwidth
+            left = 0
+            top = (h-outh)//2
+        else:
+            outw = h*self.defwidth//self.defheight
+            outh = h
+            left = (w-outw)//2
+            top = 0
+
+        self.renderer.render(painter, qt.QRectF(left, top, outw, outh))
