@@ -21,6 +21,7 @@ positions."""
 
 from __future__ import division
 import numpy as N
+import re
 
 from ..compat import czip
 from .. import qtall as qt4
@@ -166,6 +167,124 @@ class TickLabel(setting.Text):
                                                 'labels from axis'),
                                       usertext= _('Tick offset') ) )
 
+class AutoRange(setting.ChoiceOrMore):
+    """Choose how to choose range of axis."""
+
+    # +5% or -5%
+    re_dr_plusminus = re.compile(r'^\s*([+-][0-9]+)\s*%\s*$')
+    # 5-10%
+    re_dr_percrange = re.compile(r'^\s*(-?[0-9]+)\s*-\s*([0-9]+)\s*%\s*$')
+    # < 5%
+    re_dr_lower = re.compile(r'^\s*<\s*([0-9]+)\s*%\s*$')
+    # > 95%
+    re_dr_upper = re.compile(r'^\s*>\s*([0-9]+)\s*%\s*$')
+
+    def __init__(self, name, value, **args):
+        setting.ChoiceOrMore.__init__(
+            self,
+            'autoRange',
+            ('exact', 'next-tick',
+             '+2%', '+5%', '+10%', '+15%',
+             '-2%', '-5%', '-10%', '-15%',
+             '20-80%', '<20%', '>80%',
+            ),
+            'next-tick',
+            descr = _('If axis range not specified, use range of '
+                      'data and this setting'),
+            descriptions = (
+                _('Use exact data range'),
+                _('Round up to tick marks from data range'),
+                _('Expand 2% beyond data range'),
+                _('Expand 5% beyond data range'),
+                _('Expand 10% beyond data range'),
+                _('Expand 15% beyond data range'),
+                _('Shrink 2% inside data range'),
+                _('Shrink 5% inside data range'),
+                _('Shrink 10% inside data range'),
+                _('Shrink 15% inside data range'),
+                _('20 to 80% of the data range'),
+                _('Up to 20% of the data range'),
+                _('Above 80% of the data range'),
+            ),
+            formatting = True,
+            usertext = _('Auto range'),
+            )
+
+    def copy(self):
+        """Return copy of this setting."""
+        return self._copyHelper((), (), {})
+
+    def autoRangeToFracs(self, rng, doc):
+        """Convert auto range setting to fractions to expand range by.
+
+        Returns tuple of expansion fractions for left and right
+        """
+
+        # +X% or -Y%
+        m = self.re_dr_plusminus.match(rng)
+        if m:
+            v = float(m.group(1))*0.01
+            return v, v
+
+        # X-Y%
+        m = self.re_dr_percrange.match(rng)
+        if m:
+            v1 = -float(m.group(1))*0.01
+            v2 = -(1-float(m.group(2))*0.01)
+            return v1, v2
+
+        # <X%
+        m = self.re_dr_lower.match(rng)
+        if m:
+            return 0, -(1-float(m.group(1))*0.01)
+
+        # >Y%
+        m = self.re_dr_upper.match(rng)
+        if m:
+            return -(float(m.group(1))*0.01), 0
+
+        # error
+        doc.log(_("Invalid axis range '%s'") % rng)
+        return 0, 0
+
+    def adjustPlottedRange(self, plottedrange, adjustmin, adjustmax, log, doc):
+        """Adjust plotted range list given settings.
+        plottedrange: [minval, maxval]
+        adjustmin/adjustmax: bool whether to adjust
+        log: is log axis
+        doc: Document object
+        """
+        rng = self.val
+        if rng == 'exact':
+            pass
+        elif rng == 'next-tick':
+            pass
+        else:
+            # get fractions to expand range by
+            expandfrac1, expandfrac2 = self.autoRangeToFracs(
+                rng, doc)
+            origrange = list(plottedrange)
+            if log:
+                # logarithmic
+                logrng = abs( N.log(plottedrange[1]) -
+                           N.log(plottedrange[0]) )
+                if adjustmin:
+                    plottedrange[0] /= N.exp(logrng * expandfrac1)
+                if adjustmax:
+                    plottedrange[1] *= N.exp(logrng * expandfrac2)
+            else:
+                # linear
+                rng = plottedrange[1] - plottedrange[0]
+                if adjustmin:
+                    plottedrange[0] -= rng*expandfrac1
+                if adjustmax:
+                    plottedrange[1] += rng*expandfrac2
+
+            # if order is wrong, then give error!
+            if plottedrange[1] <= plottedrange[0]:
+                doc.log(_("Invalid axis range '%s'") % rng)
+                plottedrange[:] = origrange
+
 ###############################################################################
 
 class Axis(widget.Widget):
@@ -182,9 +301,6 @@ class Axis(widget.Widget):
         widget.Widget.__init__(self, parent, name=name)
         s = self.settings
 
-        if type(self) == Axis:
-            self.readDefaults()
-
         if self.name == 'y' and s.direction != 'vertical':
             s.direction = 'vertical'
         elif self.name == 'x' and s.direction != 'horizontal':
@@ -213,21 +329,7 @@ class Axis(widget.Widget):
         s.add( setting.Bool('log', False,
                             descr = _('Whether axis is logarithmic'),
                             usertext=_('Log')) )
-        s.add( setting.Choice(
-                'autoRange',
-                ('exact', 'next-tick', '+2%', '+5%', '+10%', '+15%'),
-                'next-tick',
-                descr = _('If axis range not specified, use range of '
-                          'data and this setting'),
-                descriptions = (_('Use exact data range'),
-                                _('Use data range, rounding up to tick marks'),
-                                _('Use data range, adding 2% of range'),
-                                _('Use data range, adding 5% of range'),
-                                _('Use data range, adding 10% of range'),
-                                _('Use data range, adding 15% of range'),
-                                ),
-                formatting = True,
-                usertext = _('Auto range') ) )
+        s.add( AutoRange('autoRange', 'next-tick') )
         s.add( setting.Choice('mode',
                               ('numeric', 'datetime', 'labels'),
                               'numeric',
@@ -328,8 +430,9 @@ class Axis(widget.Widget):
     def userdescription(self):
         """User friendly description."""
         s = self.settings
-        return "range %s to %s%s" % ( str(s.min), str(s.max),
-                                      ['',' (log)'][s.log])
+        return "range %s to %s%s" % (
+            str(s.min), str(s.max),
+            ['',' (log)'][self.plottedLog()])
 
     def isLinked(self):
         """Whether is an axis linked to another."""
@@ -357,6 +460,7 @@ class Axis(widget.Widget):
         """Return whether any of the bounds are automatically determined."""
         return self.settings.min == 'Auto' or self.settings.max == 'Auto'
 
+
     def computePlottedRange(self, force=False, overriderange=None):
         """Convert the range requested into a plotted range."""
 
@@ -418,29 +522,8 @@ class Axis(widget.Widget):
             if self.plottedrange[0] == self.plottedrange[1]:
                 self.plottedrange[1] = self.plottedrange[0]*2
 
-        r = s.autoRange
-        if r == 'exact':
-            pass
-        elif r == 'next-tick':
-            pass
-        else:
-            val = {'+2%': 0.02, '+5%': 0.05, '+10%': 0.1, '+15%': 0.15}[r]
-
-            if s.log:
-                # logarithmic
-                logrng = abs( N.log(self.plottedrange[1]) -
-                           N.log(self.plottedrange[0]) )
-                if s.min == 'Auto':
-                    self.plottedrange[0] /= N.exp(logrng * val)
-                if s.max == 'Auto':
-                    self.plottedrange[1] *= N.exp(logrng * val)
-            else:
-                # linear
-                rng = self.plottedrange[1] - self.plottedrange[0]
-                if s.min == 'Auto':
-                    self.plottedrange[0] -= rng*val
-                if s.max == 'Auto':
-                    self.plottedrange[1] += rng*val
+        s.get('autoRange').adjustPlottedRange(
+            self.plottedrange, s.min=='Auto', s.max=='Auto', s.log, self.document)
 
         self.computeTicks()
 
@@ -450,6 +533,12 @@ class Axis(widget.Widget):
 
         self.docchangeset = self.document.changeset
 
+    def plottedLog(self):
+        """Plotted in log?
+        This is overridden if the mode is incorrect."""
+        return (self.settings.log and
+                self.settings.mode in ('numeric', 'labels'))
+
     def computeTicks(self, allowauto=True):
         """Update ticks given plotted range.
         if allowauto is False, then do not allow ticks to be
@@ -471,7 +560,7 @@ class Axis(widget.Widget):
         axs = tickclass(self.plottedrange[0], self.plottedrange[1],
                         s.MajorTicks.number, s.MinorTicks.number,
                         extendmin = extendmin, extendmax = extendmax,
-                        logaxis = s.log )
+                        logaxis = self.plottedLog())
 
         axs.getTicks()
         self.plottedrange[0] = axs.minval
@@ -556,7 +645,7 @@ class Axis(widget.Widget):
         """Convert the coordinates assuming the machinery is in place."""
 
         # work out fractional posistions, then convert to pixels
-        if self.settings.log:
+        if self.plottedLog():
             fracposns = self.logConvertToPlotter(vals)
         else:
             fracposns = self.linearConvertToPlotter(vals)
@@ -583,7 +672,7 @@ class Axis(widget.Widget):
                  (self.coordParr2 - self.coordParr1) )
 
         # convert from fractional to graph
-        if self.settings.log:
+        if self.plottedLog():
             return self.logConvertFromPlotter(frac)
         else:
             return self.linearConvertFromPlotter(frac)
@@ -614,7 +703,7 @@ class Axis(widget.Widget):
 
         log1 = N.log(self.plottedrange[0])
         log2 = N.log(self.plottedrange[1])
-        return ( N.log( N.clip(v, 1e-99, 1e99) ) - log1 )/(log2 - log1)
+        return (N.log(N.clip(v, 1e-99, 1e99)) - log1)/(log2 - log1)
 
     def logConvertFromPlotter(self, v):
         """Convert from fraction plotter coords to graph coords with log scale.
@@ -678,7 +767,7 @@ class Axis(widget.Widget):
                        coordticks, coordticks*0.+self.coordPerp1,
                        coordticks, coordticks*0.+self.coordPerp2)
 
-    def _drawAxisLine(self, painter):
+    def _drawAxisLine(self, painter, posn):
         """Draw the line of the axis."""
 
         pen = self.settings.get('Line').makeQPen(painter)
@@ -750,7 +839,7 @@ class Axis(widget.Widget):
         for plotter in plotters:
             # get label and label coordinates from plotter (if any)
             labels, coords = plotter.getAxisLabels(dir)
-            if None not in (labels, coords):
+            if labels is not None and coords is not None:
                 # convert coordinates to plotter coordinates
                 pcoords = self._graphToPlotter(coords)
                 for coord, pcoord, lab in czip(coords, pcoords, labels):
@@ -791,7 +880,7 @@ class Axis(widget.Widget):
         # get information about text scales
         tl = s.get('TickLabels')
         scale = tl.scale
-        pen = tl.makeQPen()
+        pen = tl.makeQPen(painter)
 
         # an extra offset if required
         self._delta_axis += tl.get('offset').convert(painter)
@@ -828,8 +917,11 @@ class Axis(widget.Widget):
             else:
                 x, y = parlposn, perpposn
 
-            r = utils.Renderer(painter, font, x, y, text, alignhorz=ax,
-                               alignvert=ay, angle=angle)
+            r = utils.Renderer(
+                painter, font, x, y, text, alignhorz=ax,
+                alignvert=ay, angle=angle,
+                doc=self.document)
+
             if outerbounds is not None:
                 # make sure ticks are within plot
                 if vertical:
@@ -933,54 +1025,18 @@ class Axis(widget.Widget):
                     x = outerbounds[2]
                     ax = -ax
 
-        r = utils.Renderer(painter, font, x, y, text,
-                           ax, ay, angle,
-                           usefullheight = True)
+        r = utils.Renderer(
+            painter, font, x, y, text,
+            ax, ay, angle,
+            usefullheight=True,
+            doc=self.document)
 
         # make sure text is in plot rectangle
         if outerbounds is not None:
             r.ensureInBox( minx=outerbounds[0], maxx=outerbounds[2],
                            miny=outerbounds[1], maxy=outerbounds[3] )
 
-        texttorender.insert(0, (r, s.get('Label').makeQPen()) )
-
-    def _shouldAutoMirror(self):
-        """Work out whether to do mirroring."""
-
-        # FIXME: This is a nasty hack: must think of a better way to do this
-
-        s = self.settings
-        countaxis = 0
-        for c in self.parent.children:
-            try:
-                # don't allow descendents of axis to look like an axis
-                # to this function (e.g. colorbar)
-                if c.isaxis and s.direction == c.settings.direction:
-                    countaxis += 1
-            except AttributeError:
-                # if it's not an axis we get here
-                pass
-        return countaxis <= 1
-
-    def _autoMirrorDraw(self, posn, painter, coordticks, coordminorticks):
-        """Mirror axis to opposite side of graph if there isn't
-        an axis there already."""
-
-        # swap axis to other side
-        s = self.settings
-        if s.otherPosition < 0.5:
-            otheredge = 1.
-        else:
-            otheredge = 0.
-
-        # temporarily change position of axis to other side for drawing
-        self.updateAxisLocation(posn, otherposition=otheredge)
-        if not s.Line.hide:
-            self._drawAxisLine(painter)
-        if not s.MinorTicks.hide:
-            self._drawMinorTicks(painter, coordminorticks)
-        if not s.MajorTicks.hide:
-            self._drawMajorTicks(painter, coordticks)
+        texttorender.insert(0, (r, s.get('Label').makeQPen(painter)) )
 
     def chooseName(self):
         """Get default name for axis. Make x and y axes, then axisN."""
@@ -1058,22 +1114,61 @@ class Axis(widget.Widget):
 
             painter.restore()
 
+    def _drawAutoMirrorTicks(self, posn, painter):
+        s = self.settings
+        coordticks = self._graphToPlotter(self.majortickscalc)
+        coordminorticks = self._graphToPlotter(self.minortickscalc)
+
+        if s.otherPosition < 0.5:
+            otheredge = 1.
+        else:
+            otheredge = 0.
+
+        # temporarily change position of axis to other side for drawing
+        self.updateAxisLocation(posn, otherposition=otheredge)
+        if not s.Line.hide:
+            self._drawAxisLine(painter, posn)
+        if not s.MinorTicks.hide:
+            self._drawMinorTicks(painter, coordminorticks)
+        if not s.MajorTicks.hide:
+            self._drawMajorTicks(painter, coordticks)
+
+    def drawAutoMirror(self, parentposn, phelper, allaxes):
+        """Draw mirrored ticks."""
+
+        s = self.settings
+
+        # if there's another axis in this direction, we don't mirror
+        count = 0
+        thisdir = s.direction
+        for a in allaxes:
+            if a.settings.direction == thisdir and not a.settings.hide:
+                count += 1
+        if count > 1 or not s.autoMirror or s.hide:
+            return
+
+        painter = phelper.painter(self, parentposn, layer=-1)
+        self.updateAxisLocation(parentposn)
+        with painter:
+            painter.save()
+            self._drawAutoMirrorTicks(parentposn, painter)
+            painter.restore()
+
     def draw(self, parentposn, phelper, outerbounds=None):
         """Plot the axis on the painter.
         """
 
-        posn = self.computeBounds(parentposn, phelper)
-        self.updateAxisLocation(posn)
+        self.updateAxisLocation(parentposn)
 
         # exit if axis is hidden
         if self.settings.hide:
             return
 
         self.computePlottedRange()
-        painter = phelper.painter(self, posn)
+        painter = phelper.painter(self, parentposn)
         with painter:
-            self._axisDraw(posn, parentposn, outerbounds, painter, phelper)
-
+            self._axisDraw(
+                parentposn, parentposn, outerbounds, painter, phelper)
 
     def _drawTextWithoutOverlap(self, painter, texttorender):
         """Aall the text is drawn at the end so that we can check it
@@ -1102,7 +1197,7 @@ class Axis(widget.Widget):
 
         # make control item for axis
         phelper.setControlGraph(self, [ controlgraph.ControlAxisLine(
-                    self, s.direction, self.coordParr1,
+                    self, phelper, s.direction, self.coordParr1,
                     self.coordParr2, self.coordPerp, posn) ])
 
         # get tick vals
@@ -1120,7 +1215,7 @@ class Axis(widget.Widget):
 
         # plot the line along the axis
         if not s.Line.hide:
-            self._drawAxisLine(painter)
+            self._drawAxisLine(painter, posn)
 
         # plot minor ticks
         if not s.MinorTicks.hide:
@@ -1143,10 +1238,6 @@ class Axis(widget.Widget):
         if not s.Label.hide and not suppresstext:
             self._drawAxisLabel(painter, sign, outerbounds, texttorender)
 
-        # mirror axis at other side of plot
-        if s.autoMirror and self._shouldAutoMirror():
-            self._autoMirrorDraw(posn, painter, coordticks, coordminorticks)
-
         self._drawTextWithoutOverlap(painter, texttorender)
 
     def updateControlItem(self, cgi):
