@@ -37,7 +37,7 @@ class BarFill(setting.Settings):
     '''Filling of bars.'''
     def __init__(self, name, **args):
         setting.Settings.__init__(self, name, **args)
-        self.add( setting.FillSet('fills', [('solid', 'grey', False)],
+        self.add( setting.FillSet('fills', [('solid', 'auto', False)],
                                   descr = _('Fill styles for dataset bars'),
                                   usertext=_('Fill styles')) )
 
@@ -67,12 +67,6 @@ class BarPlotter(GenericPlotter):
     allowusercreation=True
     description=_('Plot bar charts')
 
-    def __init__(self, parent, name=None):
-        """Initialise bar chart."""
-        GenericPlotter.__init__(self, parent, name=name)
-        if type(self) == BarPlotter:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         """Construct list of settings."""
@@ -282,9 +276,10 @@ class BarPlotter(GenericPlotter):
         maxcoord = N.clip(maxcoord, -32767, 32767)
 
         # draw error bars
-        painter.setPen( self.settings.ErrorBarLine.makeQPenWHide(painter) )
-        w = barwidth*0.25
-        if ishorz:
+        ebl = self.settings.ErrorBarLine
+        painter.setPen( ebl.makeQPenWHide(painter) )
+        w = barwidth*0.25*ebl.endsize
+        if ishorz and not ebl.hideHorz:
             utils.plotLinesToPainter(painter, mincoord, posns,
                                      maxcoord, posns)
             if s.errorstyle == 'barends':
@@ -292,7 +287,7 @@ class BarPlotter(GenericPlotter):
                                          mincoord, posns+w)
                 utils.plotLinesToPainter(painter, maxcoord, posns-w,
                                          maxcoord, posns+w)
-        else:
+        elif not ishorz and not ebl.hideVert:
             utils.plotLinesToPainter(painter, posns, mincoord,
                                      posns, maxcoord)
             if s.errorstyle == 'barends':
@@ -317,7 +312,8 @@ class BarPlotter(GenericPlotter):
         utils.addNumpyPolygonToPath(
             path, extclip, corners[0], corners[1], corners[2], corners[1],
             corners[2], corners[3], corners[0], corners[3])
-        utils.brushExtFillPath(painter, brush, path, stroke=pen)
+        utils.brushExtFillPath(
+            painter, brush, path, stroke=pen, dataindex=dsnum)
 
     def barDrawGroup(self, painter, posns, maxwidth, dsvals,
                      axes, widgetposn, clip):
@@ -473,7 +469,7 @@ class BarPlotter(GenericPlotter):
 
             # actually draw polygon
             brush = s.BarFill.get('fills').returnBrushExtended(dsnum)
-            utils.brushExtFillPath(painter, brush, path)
+            utils.brushExtFillPath(painter, brush, path, dataindex=dsnum)
 
             # now draw lines
             poly = qt4.QPolygonF()
@@ -500,6 +496,12 @@ class BarPlotter(GenericPlotter):
             return 0
         return min( len([k for k in self.settings.keys if k]), len(lengths) )
 
+    def setupAutoColor(self, painter):
+        """Initialise correct number of colors."""
+        lengths = self.settings.get('lengths').getData(self.document)
+        for i in crange(len(lengths)):
+            self.autoColor(painter, dataindex=i)
+
     def getKeyText(self, number):
         """Get key entry."""
         return [k for k in self.settings.keys if k][number]
@@ -518,7 +520,8 @@ class BarPlotter(GenericPlotter):
 
         # get data
         doc = self.document
-        positions = s.get('posn').getData(doc)
+        positions = s.get('posn')
+        positions = None if positions.isEmpty() else positions.getData(doc)
         lengths = s.get('lengths').getData(doc)
         if not lengths:
             return
