@@ -112,12 +112,6 @@ class BoxPlot(GenericPlotter):
     allowusercreation=True
     description=_('Plot box plots')
 
-    def __init__(self, parent, name=None):
-        """Initialise box plot."""
-        GenericPlotter.__init__(self, parent, name=name)
-        if type(self) == BoxPlot:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         """Construct list of settings."""
@@ -365,7 +359,7 @@ class BoxPlot(GenericPlotter):
 
         # draw outliers
         painter.setPen( s.MarkersLine.makeQPenWHide(painter) )
-        painter.setBrush( s.MarkersFill.makeQBrushWHide() )
+        painter.setBrush( s.MarkersFill.makeQBrushWHide(painter) )
         markersize = s.get('markerSize').convert(painter)
         if stats.outliers.shape[0] != 0:
             pltvals = axes[not horz].dataToPlotterCoords(posn, stats.outliers)
@@ -405,14 +399,14 @@ class BoxPlot(GenericPlotter):
             datasets = [ s.get(x).getData(doc) for x in
                          ('whiskermin', 'whiskermax', 'boxmin',
                           'boxmax', 'mean', 'median') ]
-            if None in datasets:
+            if any((d is None for d in datasets)):
                 return
 
         # get axes widgets
         axes = self.parent.getAxes( (s.xAxis, s.yAxis) )
 
         # return if there are no proper axes
-        if ( None in axes or
+        if ( axes[0] is None or axes[1] is None or
              axes[0].settings.direction != 'horizontal' or
              axes[1].settings.direction != 'vertical' ):
             return
