@@ -40,54 +40,55 @@ class FunctionPlotter(GenericPlotter):
     typename='function'
     allowusercreation=True
     description=_('Plot a function')
-    
-    def __init__(self, parent, name=None):
-        """Initialise plotter."""
-
-        GenericPlotter.__init__(self, parent, name=name)
-
-        if type(self) == FunctionPlotter:
-            self.readDefaults()
 
     @classmethod
     def addSettings(klass, s):
         """Construct list of settings."""
         GenericPlotter.addSettings(s)
 
-        s.add( setting.Int('steps',
-                           50,
-                           minval = 3,
-                           descr = _('Number of steps to evaluate the function'
-                                     ' over'),
-                           usertext=_('Steps'), formatting=True), 0 )
-        s.add( setting.Choice('variable', ['x', 'y'], 'x',
-                              descr=_('Variable the function is a function of'),
-                              usertext=_('Variable')),
+        s.add( setting.Int(
+            'steps',
+            50,
+            minval = 3,
+            descr = _('Number of steps to evaluate the function'
+                      ' over'),
+            usertext=_('Steps'), formatting=True), 0 )
+        s.add( setting.Choice(
+            'variable', ['x', 'y'], 'x',
+            descr=_('Variable the function is a function of'),
+            usertext=_('Variable')),
                0 )
-        s.add( setting.Str('function', 'x',
-                           descr=_('Function expression'),
-                           usertext=_('Function')), 0 )
+        s.add( setting.Str(
+            'function', 'x',
+            descr=_('Function expression'),
+            usertext=_('Function')), 0 )
 
-        s.add(setting.FloatOrAuto('min', 'Auto',
-                                  descr=_('Minimum value at which to plot function'),
-                                  usertext=_('Min')))
-        
-        s.add(setting.FloatOrAuto('max', 'Auto',
-                                  descr=_('Maximum value at which to plot function'),
-                                  usertext=_('Max')))
+        s.add(setting.FloatOrAuto(
+            'min', 'Auto',
+            descr=_('Minimum value at which to plot function'),
+            usertext=_('Min')))
 
-        s.add( setting.Line('Line',
-                            descr = _('Function line settings'),
-                            usertext = _('Plot line')),
+        s.add(setting.FloatOrAuto(
+            'max', 'Auto',
+            descr=_('Maximum value at which to plot function'),
+            usertext=_('Max')))
+
+        s.add( setting.Line(
+            'Line',
+            descr = _('Function line settings'),
+            usertext = _('Plot line')),
                pixmap = 'settings_plotline' )
+        s.Line.get('color').newDefault('auto')
 
-        s.add( setting.PlotterFill('FillBelow',
-                                   descr = _('Fill below/left function'),
-                                   usertext = _('Fill below')),
+        s.add( setting.PlotterFill(
+            'FillBelow',
+            descr = _('Fill below/left function'),
+            usertext = _('Fill below')),
                pixmap = 'settings_plotfillbelow' )
-        s.add( setting.PlotterFill('FillAbove',
-                                   descr = _('Fill mode above/right function'),
-                                   usertext = _('Fill above')),
+        s.add( setting.PlotterFill(
+            'FillAbove',
+            descr = _('Fill mode above/right function'),
+            usertext = _('Fill above')),
                pixmap = 'settings_plotfillabove' )
 
     @property
@@ -124,17 +125,17 @@ class FunctionPlotter(GenericPlotter):
             return
 
         # ignore if function isn't sensible
-        compiled = self.document.compileCheckedExpression(s.function)
+        compiled = self.document.evaluate.compileCheckedExpression(s.function)
         if compiled is None:
             return
 
         # find axis to find variable range over
-        axis = self.lookupAxis( {'x': s.xAxis, 'y': s.yAxis}[s.variable] )
-        if not axis:
+        varaxis = self.lookupAxis( {'x': s.xAxis, 'y': s.yAxis}[s.variable] )
+        if not varaxis:
             return
 
         # get range of that axis
-        varaxrange = list(axis.getPlottedRange())
+        varaxrange = list(varaxis.getPlottedRange())
         if varaxrange[0] == varaxrange[1]:
             return
 
@@ -146,8 +147,8 @@ class FunctionPlotter(GenericPlotter):
 
         # work out function in steps
         try:
-            if axis.settings.log:
-                # log spaced steps 
+            if varaxis.settings.log:
+                # log spaced steps
                 l1, l2 = N.log(varaxrange[1]), N.log(varaxrange[0])
                 delta = (l2-l1)/20.
                 points = N.exp(N.arange(l1, l2+delta, delta))
@@ -170,6 +171,9 @@ class FunctionPlotter(GenericPlotter):
         # get values which are finite: excluding nan and inf
         finitevals = vals[N.isfinite(vals)]
 
+        if axis.settings.log:
+            finitevals = finitevals[finitevals > 0]
+
         # update the automatic range
         if len(finitevals) > 0:
             axrange[0] = min(N.min(finitevals), axrange[0])
@@ -268,14 +272,14 @@ class FunctionPlotter(GenericPlotter):
 
     def initEnviron(self):
         """Set up function environment."""
-        return self.document.eval_context.copy()
+        return self.document.evaluate.context.copy()
 
     def getIndependentPoints(self, axes, posn):
         """Calculate the real and screen points to plot for the independent axis"""
 
         s = self.settings
 
-        if ( None in axes or
+        if ( axes[0] is None or axes[1] is None or
              axes[0].settings.direction != 'horizontal' or
              axes[1].settings.direction != 'vertical' ):
             return None, None
@@ -309,7 +313,7 @@ class FunctionPlotter(GenericPlotter):
 
         s = self.settings
 
-        if ( None in axes or
+        if ( axes[0] is None or axes[1] is None or
              axes[0].settings.direction != 'horizontal' or
              axes[1].settings.direction != 'vertical' ):
             return None, None
@@ -317,7 +321,7 @@ class FunctionPlotter(GenericPlotter):
         if axispts is None:
             return None, None
 
-        compiled = self.document.compileCheckedExpression(s.function)
+        compiled = self.document.evaluate.compileCheckedExpression(s.function)
         if not compiled:
             return None, None
 
@@ -339,7 +343,7 @@ class FunctionPlotter(GenericPlotter):
     def calcFunctionPoints(self, axes, posn):
         ipts, pipts = self.getIndependentPoints(axes, posn)
         dpts, pdpts = self.calcDependentPoints(ipts, axes, posn)
-        
+
         if self.settings.variable == 'x':
             return (ipts, dpts), (pipts, pdpts)
         else:
@@ -357,7 +361,7 @@ class FunctionPlotter(GenericPlotter):
             axisnames[0] = axisnames[0] + '(' + axisnames[1] + ')'
 
         (xpts, ypts), (pxpts, pypts) = self.calcFunctionPoints(axes, posn)
-        
+
         return pickable.GenericPickable(
                     self, axisnames, (xpts, ypts), (pxpts, pypts) )
 
@@ -379,7 +383,8 @@ class FunctionPlotter(GenericPlotter):
         (xpts, ypts), (pxpts, pypts) = self.calcFunctionPoints(axes, posn)
 
         # draw the function line
-        if pxpts is None or pypts is None:
+        if ( pxpts is None or pypts is None or
+             pxpts.ndim != 1 or pypts.ndim != 1 ):
             # not sure how to deal with errors here
             painter.setPen( setting.settingdb.color('error') )
             f = qt4.QFont()
@@ -404,4 +409,3 @@ class FunctionPlotter(GenericPlotter):
 
 # allow the factory to instantiate an function plotter
 document.thefactory.register( FunctionPlotter )
-
