@@ -40,14 +40,6 @@ class VectorField(plotters.GenericPlotter):
     allowusercreation = True
     description = _('Plot a vector field')
 
-    def __init__(self, parent, name=None):
-        """Initialse vector field plotter."""
-
-        plotters.GenericPlotter.__init__(self, parent, name=name)
-
-        if type(self) == VectorField:
-            self.readDefaults()
-
     @classmethod
     def addSettings(klass, s):
         '''Construct list of settings.'''
@@ -154,11 +146,12 @@ class VectorField(plotters.GenericPlotter):
 
         s = self.settings
         painter.setPen( s.Line.makeQPenWHide(painter) )
-        painter.setBrush( s.get('Fill').makeQBrushWHide() )
-        utils.plotLineArrow(painter, x+width, y+height*0.5,
-                            width, 180, height*0.25,
-                            arrowleft=s.arrowfront,
-                            arrowright=s.arrowback)
+        painter.setBrush( s.get('Fill').makeQBrushWHide(painter) )
+        utils.plotLineArrow(
+            painter, x+width, y+height*0.5,
+            width, 180, height*0.25,
+            arrowleft=s.arrowfront,
+            arrowright=s.arrowback)
 
         painter.restore()
 
@@ -228,7 +221,7 @@ class VectorField(plotters.GenericPlotter):
                                      cliprect)
         else:
             arrowsize = s.get('arrowsize').convert(painter)
-            painter.setBrush( s.get('Fill').makeQBrushWHide() )
+            painter.setBrush( s.get('Fill').makeQBrushWHide(painter) )
 
             # this is backward - have to convert from dx, dy to angle, length
             angles = 180 - N.arctan2(dy, dx) * (180./N.pi)
