@@ -20,17 +20,19 @@
 """Implements the main window of the application."""
 
 from __future__ import division, print_function
+import os
 import os.path
 import sys
 import glob
 import re
+import datetime
 
 try:
     import h5py
 except ImportError:
     h5py = None
 
-from ..compat import cstr, cstrerror
+from ..compat import cstr, cstrerror, cgetcwd, cbytes
 from .. import qtall as qt4
 
 from .. import document
@@ -44,21 +46,6 @@ from . import plotwindow
 from . import treeeditwindow
 from .datanavigator import DataNavigatorWindow
 
-from ..dialogs.aboutdialog import AboutDialog
-from ..dialogs.reloaddata import ReloadData
-from ..dialogs.datacreate import DataCreateDialog
-from ..dialogs.datacreate2d import DataCreate2DDialog
-from ..dialogs.preferences import PreferencesDialog
-from ..dialogs.errorloading import ErrorLoadingDialog
-from ..dialogs.capturedialog import CaptureDialog
-from ..dialogs.stylesheet import StylesheetDialog
-from ..dialogs.custom import CustomDialog
-from ..dialogs.safetyimport import SafetyImportDialog
-from ..dialogs.histodata import HistoDataDialog
-from ..dialogs.plugin import handlePlugin
-from ..dialogs import importdialog
-from ..dialogs import dataeditdialog
-
 def _(text, disambiguation=None, context='MainWindow'):
     """Translate text."""
     return qt4.QCoreApplication.translate(context, text, disambiguation)
@@ -73,7 +60,9 @@ class DBusWinInterface(vzdbus.Object):
 
     def __init__(self, actions, index):
         prefix = '/Windows/%i/Actions' % index
-        vzdbus.Object.__init__(self, vzdbus.sessionbus, prefix)
+        # possible exception in dbus means we have to check sessionbus
+        if vzdbus.sessionbus is not None:
+            vzdbus.Object.__init__(self, vzdbus.sessionbus, prefix)
         self.actions = actions
 
     @vzdbus.method(dbus_interface=interface, out_signature='as')
@@ -96,7 +85,7 @@ class MainWindow(qt4.QMainWindow):
 
     windows = []
     @classmethod
-    def CreateWindow(cls, filename=None):
+    def CreateWindow(cls, filename=None, mode='graph'):
         """Window factory function.
 
         If filename is given then that file is loaded into the window.
@@ -110,18 +99,28 @@ class MainWindow(qt4.QMainWindow):
             # load document
             win.openFileInWindow(filename)
         else:
-            win.setupDefaultDoc()
+            win.setupDefaultDoc(mode)
 
         # try to select first graph of first page
         win.treeedit.doInitialWidgetSelect()
 
         cls.windows.append(win)
 
-        # check if tutorial wanted
-        if not setting.settingdb['ask_tutorial']:
+        # check if tutorial wanted (only for graph mode)
+        if not setting.settingdb['ask_tutorial'] and mode=='graph':
             win.askTutorial()
-        # don't ask again
-        setting.settingdb['ask_tutorial'] = True
+            # don't ask again
+            setting.settingdb['ask_tutorial'] = True
+
+        # check if version check is ok
+        win.askVersionCheck()
+        # periodically do the check
+        win.doVersionCheck()
+
+        # is it ok to do feedback?
+        win.askFeedbackCheck()
+        # periodically send feedback
+        win.doFeedback()
 
         return win
 
@@ -202,9 +201,8 @@ class MainWindow(qt4.QMainWindow):
 
         # working directory - use previous one
         self.dirname = setdb.get('dirname', qt4.QDir.homePath())
-        self.dirname_export = setdb.get('dirname_export', self.dirname)
         if setdb['dirname_usecwd']:
-            self.dirname = self.dirname_export = os.getcwd()
+            self.dirname = cgetcwd()
 
         # connect plot signals to main window
         self.plot.sigUpdatePage.connect(self.slotUpdatePage)
@@ -246,13 +244,13 @@ class MainWindow(qt4.QMainWindow):
 
     def dragEnterEvent(self, event):
         """Check whether event is valid to be dropped."""
-        if (event.provides("text/uri-list") and
+        if (event.mimeData().hasUrls() and
             self._getVeuszDropFiles(event)):
             event.acceptProposedAction()
 
     def dropEvent(self, event):
         """Respond to a drop event on the current window"""
-        if event.provides("text/uri-list"):
+        if event.mimeData().hasUrls():
             files = self._getVeuszDropFiles(event)
             if files:
                 if self.document.isBlank():
@@ -271,16 +269,20 @@ class MainWindow(qt4.QMainWindow):
             return []
         else:
             # get list of vsz files dropped
-            urls = [u.path() for u in mime.urls()]
+            urls = [u.toLocalFile() for u in mime.urls()]
             urls = [u for u in urls if os.path.splitext(u)[1] == '.vsz']
             return urls
 
-    def setupDefaultDoc(self):
+    def setupDefaultDoc(self, mode):
         """Setup default document."""
 
         if not self.documentsetup:
             # add page and default graph
-            self.document.makeDefaultDoc()
+            self.document.makeDefaultDoc(mode)
+
+            # set color theme
+            self.document.basewidget.settings.get(
+                'colorTheme').set(setting.settingdb['colortheme_default'])
 
             # load defaults if set
             self.loadDefaultStylesheet()
@@ -354,15 +356,18 @@ class MainWindow(qt4.QMainWindow):
             self.document.redoOperation()
 
     def slotEditPreferences(self):
+        from ..dialogs.preferences import PreferencesDialog
         dialog = PreferencesDialog(self)
         dialog.exec_()
 
     def slotEditStylesheet(self):
+        from ..dialogs.stylesheet import StylesheetDialog
         dialog = StylesheetDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotEditCustom(self):
+        from ..dialogs.custom import CustomDialog
         dialog = CustomDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
@@ -375,12 +380,14 @@ class MainWindow(qt4.QMainWindow):
         menuname: string giving prefix for new menu entries (inside actions)
         """
 
+        def getLoadDialog(pluginkls):
+            def _loadPlugin():
+                from ..dialogs.plugin import handlePlugin
+                handlePlugin(self, self.document, pluginkls)
+            return _loadPlugin
+
         menu = []
         for pluginkls in pluginlist:
-            def loaddialog(pluginkls=pluginkls):
-                """Load plugin dialog"""
-                handlePlugin(self, self.document, pluginkls)
-
             actname = menuname + '.' + '.'.join(pluginkls.menu)
             text = pluginkls.menu[-1]
             if pluginkls.has_parameters:
@@ -389,7 +396,7 @@ class MainWindow(qt4.QMainWindow):
                 self,
                 pluginkls.description_short,
                 text,
-                loaddialog)
+                getLoadDialog(pluginkls))
 
             # build up menu from tuple of names
             menulook = menu
@@ -416,20 +423,48 @@ class MainWindow(qt4.QMainWindow):
         # these are actions for main menu toolbars and menus
         a = utils.makeAction
         self.vzactions = {
-            'file.new':
-                a(self, _('New document'), _('&New'),
-                  self.slotFileNew,
-                  icon='kde-document-new', key='Ctrl+N'),
+            'file.new.menu':
+                a(self, _('New document'), _('New'),
+                  None,
+                  icon='kde-document-new'),
+            'file.new.graph':
+                a(self,
+                  _('New graph document'),
+                  _('&New graph document'),
+                  self.slotFileNewGraph,
+                  icon='kde-document-new-graph', key='Ctrl+N'),
+            'file.new.polar':
+                a(self,
+                  _('New polar plot document'),
+                  _('New polar document'),
+                  self.slotFileNewPolar,
+                  icon='kde-document-new-polar'),
+            'file.new.ternary':
+                a(self,
+                  _('New ternary plot document'),
+                  _('New ternary document'),
+                  self.slotFileNewTernary,
+                  icon='kde-document-new-ternary'),
+            'file.new.graph3d':
+                a(self,
+                  _('New 3D plot document'),
+                  _('New 3D document'),
+                  self.slotFileNewGraph3D,
+                  icon='kde-document-new-graph3d'),
+
             'file.open':
                 a(self, _('Open a document'), _('&Open...'),
                   self.slotFileOpen,
                   icon='kde-document-open', key='Ctrl+O'),
+            'file.reload':
+                a(self, _('Reload document from saved version'),
+                  _('Reload...'), self.slotFileReload),
             'file.save':
                 a(self, _('Save the document'), _('&Save'),
                   self.slotFileSave,
                   icon='kde-document-save', key='Ctrl+S'),
             'file.saveas':
-                a(self, _('Save the current graph under a new name'),
+                a(self, _('Save the current document under a new name'),
                   _('Save &As...'), self.slotFileSaveAs,
                   icon='kde-document-save-as'),
             'file.print':
@@ -437,7 +472,7 @@ class MainWindow(qt4.QMainWindow):
                   self.slotFilePrint,
                   icon='kde-document-print', key='Ctrl+P'),
             'file.export':
-                a(self, _('Export the current page'), _('&Export...'),
+                a(self, _('Export to graphics formats'), _('&Export...'),
                   self.slotFileExport,
                   icon='kde-document-export'),
             'file.close':
@@ -462,7 +497,8 @@ class MainWindow(qt4.QMainWindow):
                   self.slotEditPreferences,
                   icon='veusz-edit-prefs'),
             'edit.custom':
-                a(self, _('Edit custom functions and constants'),
+                a(self,
+                  _('Edit custom functions, constants, colors and colormaps'),
                   _('Custom definitions...'),
                   self.slotEditCustom,
                   icon='veusz-edit-custom'),
@@ -510,7 +546,7 @@ class MainWindow(qt4.QMainWindow):
                   self.slotDataImport, icon='kde-vzdata-import'),
             'data.edit':
                 a(self, _('Edit and enter new datasets'), _('&Editor...'),
-                  self.slotDataEdit, icon='kde-edit-veuszedit'),
+                  lambda: self.slotDataEdit(), icon='kde-edit-veuszedit'),
             'data.create':
                 a(self, _('Create new datasets using ranges, parametrically or as functions of existing datasets'), _('&Create...'),
                   self.slotDataCreate, icon='kde-dataset-new-veuszedit'),
@@ -520,6 +556,9 @@ class MainWindow(qt4.QMainWindow):
             'data.capture':
                 a(self, _('Capture remote data'), _('Ca&pture...'),
                   self.slotDataCapture, icon='veusz-capture-data'),
+            'data.filter':
+                a(self, _('Filter data'), _('&Filter...'),
+                  self.slotDataFilter, icon='kde-filter'),
             'data.histogram':
                 a(self, _('Histogram data'), _('&Histogram...'),
                   self.slotDataHistogram, icon='button_bar'),
@@ -530,12 +569,13 @@ class MainWindow(qt4.QMainWindow):
             'help.home':
                 a(self, _('Go to the Veusz home page on the internet'),
                   _('Home page'), self.slotHelpHomepage),
-            'help.project':
-                a(self, _('Go to the Veusz project page on the internet'),
-                  _('GNA Project page'), self.slotHelpProjectPage),
             'help.bug':
                 a(self, _('Report a bug on the internet'),
                   _('Suggestions and bugs'), self.slotHelpBug),
+            'help.update':
+                a(self, _('Download latest version'),
+                  _('Download latest version'), self.slotHelpUpdate),
+
             'help.tutorial':
                 a(self, _('An interactive Veusz tutorial'),
                   _('Tutorial'), self.slotHelpTutorial),
@@ -550,24 +590,38 @@ class MainWindow(qt4.QMainWindow):
         tb.setIconSize(qt4.QSize(iconsize, iconsize))
         tb.setObjectName('veuszmaintoolbar')
         self.addToolBar(qt4.Qt.TopToolBarArea, tb)
-        utils.addToolbarActions(tb, self.vzactions,
-                                ('file.new', 'file.open', 'file.save',
-                                 'file.print', 'file.export'))
+
+        utils.makeMenuGroupSaved(
+            'file.new.menu', self, self.vzactions, (
+                    'file.new.graph', 'file.new.graph3d',
+                    'file.new.polar', 'file.new.ternary',
+            )
+        )
+
+        utils.addToolbarActions(
+            tb, self.vzactions,
+            ('file.new.menu', 'file.open', 'file.save',
+             'file.print', 'file.export'))
 
         # data toolbar
         tb = self.datatoolbar = qt4.QToolBar(_("Data toolbar - Veusz"), self)
         tb.setIconSize(qt4.QSize(iconsize, iconsize))
         tb.setObjectName('veuszdatatoolbar')
         self.addToolBar(qt4.Qt.TopToolBarArea, tb)
-        utils.addToolbarActions(tb, self.vzactions,
-                                ('data.import', 'data.edit',
-                                 'data.create', 'data.capture',
-                                 'data.reload'))
+        utils.addToolbarActions(
+            tb, self.vzactions,
+            ('data.import', 'data.edit',
+             'data.create', 'data.capture',
+             'data.filter', 'data.reload'))
 
         # menu structure
         filemenu = [
-            'file.new', 'file.open',
+            ['file.new', _('New'),
+             ['file.new.graph', 'file.new.graph3d', 'file.new.polar',
+              'file.new.ternary']],
+            'file.open',
             ['file.filerecent', _('Open &Recent'), []],
+            'file.reload',
             '',
             'file.save', 'file.saveas',
             '',
@@ -604,11 +658,11 @@ class MainWindow(qt4.QMainWindow):
         datamenu = [
             ['data.ops', _('&Operations'), datapluginsmenu],
             'data.import', 'data.edit', 'data.create',
-            'data.create2d', 'data.capture', 'data.histogram',
+            'data.create2d', 'data.capture', 'data.filter', 'data.histogram',
             'data.reload',
             ]
         helpmenu = [
-            'help.home', 'help.project', 'help.bug',
+            'help.home', 'help.bug', 'help.update',
             '',
             'help.tutorial',
             '',
@@ -634,6 +688,9 @@ class MainWindow(qt4.QMainWindow):
         self.menus = {}
         utils.constructMenus(self.menuBar(), self.menus, menus, self.vzactions)
 
+        # set icon for File->New
+        self.menus['file.new'].setIcon(utils.getIcon('kde-document-new'))
+
         self.populateExamplesMenu()
 
     def _setPickerFont(self, label):
@@ -645,7 +702,12 @@ class MainWindow(qt4.QMainWindow):
     def populateExamplesMenu(self):
         """Add examples to help menu."""
 
-        examples = glob.glob(os.path.join(utils.exampleDirectory, '*.vsz'))
+        # not cstr here forces to unicode for Python 2, getting
+        # filenames in unicode
+        examples = [ os.path.join(utils.exampleDirectory, f)
+                     for f in os.listdir(cstr(utils.exampleDirectory))
+                     if os.path.splitext(f)[1] == ".vsz" ]
+
         menu = self.menus["help.examples"]
         for ex in sorted(examples):
             name = os.path.splitext(os.path.basename(ex))[0]
@@ -711,6 +773,7 @@ class MainWindow(qt4.QMainWindow):
 
     def slotDataImport(self):
         """Display the import data dialog."""
+        from ..dialogs import importdialog
         dialog = importdialog.ImportDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
@@ -720,6 +783,7 @@ class MainWindow(qt4.QMainWindow):
 
         If editdataset is set to a dataset name, edit this dataset
         """
+        from ..dialogs import dataeditdialog
         dialog = dataeditdialog.DataEditDialog(self, self.document)
         self.showDialog(dialog)
         if editdataset is not None:
@@ -728,46 +792,54 @@ class MainWindow(qt4.QMainWindow):
 
     def slotDataCreate(self):
         """Create new datasets."""
+        from ..dialogs.datacreate import DataCreateDialog
         dialog = DataCreateDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataCreate2D(self):
         """Create new datasets."""
+        from ..dialogs.datacreate2d import DataCreate2DDialog
         dialog = DataCreate2DDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataCapture(self):
         """Capture remote data."""
+        from ..dialogs.capturedialog import CaptureDialog
         dialog = CaptureDialog(self.document, self)
         self.showDialog(dialog)
         return dialog
 
+    def slotDataFilter(self):
+        """Filter datasets."""
+        from ..dialogs.filterdialog import FilterDialog
+        dialog = FilterDialog(self, self.document)
+        self.showDialog(dialog)
+        return dialog
+
     def slotDataHistogram(self):
         """Histogram data."""
+        from ..dialogs.histodata import HistoDataDialog
         dialog = HistoDataDialog(self, self.document)
         self.showDialog(dialog)
         return dialog
 
     def slotDataReload(self):
         """Reload linked datasets."""
+        from ..dialogs.reloaddata import ReloadData
         dialog = ReloadData(self.document, self)
         self.showDialog(dialog)
         return dialog
 
     def slotHelpHomepage(self):
         """Go to the veusz homepage."""
-        qt4.QDesktopServices.openUrl(qt4.QUrl('http://home.gna.org/veusz/'))
-
-    def slotHelpProjectPage(self):
-        """Go to the veusz project page."""
-        qt4.QDesktopServices.openUrl(qt4.QUrl('http://gna.org/projects/veusz/'))
+        qt4.QDesktopServices.openUrl(qt4.QUrl('https://veusz.github.io/'))
 
     def slotHelpBug(self):
         """Go to the veusz bug page."""
         qt4.QDesktopServices.openUrl(
-            qt4.QUrl('https://gna.org/bugs/?group=veusz') )
+            qt4.QUrl('https://github.com/veusz/veusz/issues') )
 
     def askTutorial(self):
         """Ask if tutorial wanted."""
@@ -797,8 +869,98 @@ class MainWindow(qt4.QMainWindow):
 
     def slotHelpAbout(self):
         """Show about dialog."""
+        from ..dialogs.aboutdialog import AboutDialog
         AboutDialog(self).exec_()
 
+    def askVersionCheck(self, mininterval=2):
+        """Check with user whether to do version checks.
+
+        This is only done after the user has been using the program
+        for mininterval days
+
+        """
+
+        dayssinceinstall = (
+            datetime.date.today() -
+            datetime.date(*setting.settingdb['install_date'])).days
+        if (dayssinceinstall<mininterval or
+            setting.settingdb['vercheck_asked_user'] or
+            setting.settingdb['vercheck_disabled'] or
+            utils.disableVersionChecks):
+            return
+
+        retn = qt4.QMessageBox.question(
+            self, _("Version check"),
+            _("Veusz will periodically check for new Veusz versions and\n"
+              "let you know if there is a new one available.\n\n"
+              "Is this ok? This choice can be changed in Preferences."),
+            qt4.QMessageBox.Yes | qt4.QMessageBox.No,
+            qt4.QMessageBox.Yes
+            )
+
+        setting.settingdb['vercheck_disabled'] = retn==qt4.QMessageBox.No
+        setting.settingdb['vercheck_asked_user'] = True
+
+    def doVersionCheck(self):
+        """Check whether there is a new version.
+        """
+        self.vzactions['help.update'].setVisible(False)
+
+        # check is done asynchronously
+        thread = utils.VersionCheckThread(self)
+        thread.newversion.connect(self.slotNewVersion)
+        thread.finished.connect(thread.deleteLater)
+        thread.start()
+
+    def askFeedbackCheck(self, mininterval=3):
+        """Check with user whether to do feedback.
+
+        This is only done after the user has been using the program
+        for mininterval days
+
+        """
+
+        dayssinceinstall = (
+            datetime.date.today() -
+            datetime.date(*setting.settingdb['install_date'])).days
+        if (dayssinceinstall<mininterval or
+            setting.settingdb['feedback_asked_user'] or
+            setting.settingdb['feedback_disabled'] or
+            utils.disableFeedback):
+            return
+
+        retn = qt4.QMessageBox.question(
+            self, _("Send automatic anonymous feedback"),
+            _("Veusz can automatically send anonymous feedback "
+              "to the developers, with information about the version "
+              "of software dependencies, the computer language and how "
+              "often features are used.\n\n"
+              "Is this ok? This choice can be changed in Preferences."),
+            qt4.QMessageBox.Yes | qt4.QMessageBox.No,
+            qt4.QMessageBox.Yes
+            )
+
+        setting.settingdb['feedback_disabled'] = retn==qt4.QMessageBox.No
+        setting.settingdb['feedback_asked_user'] = True
+
+    def doFeedback(self):
+        """Give feedback."""
+        thread = utils.FeedbackCheckThread(self)
+        thread.start()
+
+    def slotNewVersion(self, ver):
+        """Called when there is a new version."""
+        msg = _('Veusz %s is available for download - see Help menu') % ver
+        self.statusBar().showMessage(msg, 5000)
+        self.vzactions['help.update'].setText(
+            _('Download new Veusz %s') % ver)
+        self.vzactions['help.update'].setVisible(True)
+
+    def slotHelpUpdate(self):
+        """Open web page to update."""
+        qt4.QDesktopServices.openUrl(qt4.QUrl(
+            'https://veusz.github.io/download/'))
+
     def queryOverwrite(self):
         """Do you want to overwrite the current document.
 
@@ -809,18 +971,12 @@ class MainWindow(qt4.QMainWindow):
         if self.filename:
             filetext = " '%s'" % os.path.basename(self.filename)
 
-        # show message box
-        mb = qt4.QMessageBox(_("Save file?"),
-                             _("Document%s was modified. Save first?") % filetext,
-                             qt4.QMessageBox.Warning,
-                             qt4.QMessageBox.Yes | qt4.QMessageBox.Default,
-                             qt4.QMessageBox.No,
-                             qt4.QMessageBox.Cancel | qt4.QMessageBox.Escape,
-                             self)
-        mb.setButtonText(qt4.QMessageBox.Yes, _("&Save"))
-        mb.setButtonText(qt4.QMessageBox.No, _("&Discard"))
-        mb.setButtonText(qt4.QMessageBox.Cancel, _("&Cancel"))
-        return mb.exec_()
+        return qt4.QMessageBox.warning(
+            self,
+            _("Save file?"),
+            _("Document%s was modified. Save first?") % filetext,
+            qt4.QMessageBox.Save | qt4.QMessageBox.Discard |
+            qt4.QMessageBox.Cancel)
 
     def closeEvent(self, event):
         """Before closing, check whether we need to save first."""
@@ -831,20 +987,19 @@ class MainWindow(qt4.QMainWindow):
             if v == qt4.QMessageBox.Cancel:
                 event.ignore()
                 return
-            elif v == qt4.QMessageBox.Yes:
+            elif v == qt4.QMessageBox.Save:
                 self.slotFileSave()
 
         # store working directory
         setdb['dirname'] = self.dirname
-        setdb['dirname_export'] = self.dirname_export
 
         # store the current geometry in the settings database
         geometry = ( self.x(), self.y(), self.width(), self.height() )
         setdb['geometry_mainwindow'] = geometry
 
         # store docked windows
-        data = str(self.saveState())
-        setdb['geometry_mainwindowstate'] = data
+        data = self.saveState().data()
+        setdb['geometry_mainwindowstate'] = cbytes(data)
 
         # save current setting db
         setdb.writeSettings()
@@ -872,13 +1027,28 @@ class MainWindow(qt4.QMainWindow):
 
         # restore docked window geometry
         if 'geometry_mainwindowstate' in setdb:
-            b = qt4.QByteArray(setdb['geometry_mainwindowstate'])
-            self.restoreState(b)
+            try:
+                self.restoreState(setdb['geometry_mainwindowstate'])
+            except Exception:
+                # type can be wrong if switching between Py2/3 PyQ4/5
+                pass
 
-    def slotFileNew(self):
-        """New file."""
+    def slotFileNewGraph(self):
+        """New file (graph)."""
         self.CreateWindow()
 
+    def slotFileNewPolar(self):
+        """New file (polar)."""
+        self.CreateWindow(mode='polar')
+
+    def slotFileNewTernary(self):
+        """New file (ternary)."""
+        self.CreateWindow(mode='ternary')
+
+    def slotFileNewGraph3D(self):
+        """New file (graph3d)."""
+        self.CreateWindow(mode='graph3d')
+
     def slotFileSave(self):
         """Save file."""
 
@@ -1010,27 +1180,43 @@ class MainWindow(qt4.QMainWindow):
         Return True if loaded ok
         """
 
-        class _unsafeCmdMsgBox(qt4.QMessageBox):
-            """Show document is unsafe."""
-            def __init__(self, window):
-                qt4.QMessageBox.__init__(
-                    self, _("Unsafe code in document"),
-                    _("The document '%s' contains potentially unsafe code "
-                      "which may damage your computer or data. Please check "
-                      "that the file comes from a trusted source.") % filename,
-                    qt4.QMessageBox.Warning,
-                    qt4.QMessageBox.Yes,
-                    qt4.QMessageBox.No | qt4.QMessageBox.Default,
-                    qt4.QMessageBox.NoButton,
-                    window)
-                self.setButtonText(qt4.QMessageBox.Yes, _("C&ontinue anyway"))
-                self.setButtonText(qt4.QMessageBox.No, _("&Stop loading"))
-
         def _callbackunsafe():
+            """Callback when loading document to ask whether ok to continue loading
+            if unsafe commands are found."""
             qt4.QApplication.restoreOverrideCursor()
-            v = _unsafeCmdMsgBox(self).exec_()
-            qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
-            return v == qt4.QMessageBox.Yes
+            msgbox = qt4.QMessageBox(
+                qt4.QMessageBox.Warning,
+                _("Unsafe code in document"),
+                _("The document '%s' contains potentially unsafe code "
+                  "which may damage your computer or data. Please check "
+                  "that the file comes from a trusted source.") % filename,
+                qt4.QMessageBox.NoButton,
+                self)
+            cont = msgbox.addButton(_("C&ontinue anyway"), qt4.QMessageBox.AcceptRole)
+            stop = msgbox.addButton(_("&Stop loading"), qt4.QMessageBox.RejectRole)
+            msgbox.setDefaultButton(stop)
+            msgbox.exec_()
+            qt4.QApplication.setOverrideCursor(qt4.QCursor(qt4.Qt.WaitCursor))
+            return msgbox.clickedButton() is cont
+
+        def _callbackimporterror(filename, error):
+            """Ask user if they want to give a new filename in case of import
+            error.
+            """
+            qt4.QApplication.restoreOverrideCursor()
+            msgbox = qt4.QMessageBox(self)
+            msgbox.setWindowTitle(_("Import error"))
+            msgbox.setText(
+                _("Could not import data from file '%s':\n\n %s") % (
+                    filename, error))
+            msgbox.setInformativeText(_("Do you want to look for another file?"))
+            msgbox.setStandardButtons(qt4.QMessageBox.Yes | qt4.QMessageBox.Cancel)
+            filename = None
+            if msgbox.exec_() == qt4.QMessageBox.Yes:
+                filename = qt4.QFileDialog.getOpenFileName(self, "Choose data file")
+                filename = filename[0] if filename else None
+            qt4.QApplication.setOverrideCursor(qt4.QCursor(qt4.Qt.WaitCursor))
+            return filename
 
         # save stdout and stderr, then redirect to console
         stdout, stderr = sys.stdout, sys.stderr
@@ -1054,9 +1240,11 @@ class MainWindow(qt4.QMainWindow):
             self.document.load(
                 filename,
                 mode=mode,
-                callbackunsafe=_callbackunsafe)
+                callbackunsafe=_callbackunsafe,
+                callbackimporterror=_callbackimporterror)
 
         except document.LoadError as e:
+            from ..dialogs.errorloading import ErrorLoadingDialog
             qt4.QApplication.restoreOverrideCursor()
             if e.backtrace:
                 d = ErrorLoadingDialog(self, filename, cstr(e), e.backtrace)
@@ -1094,7 +1282,6 @@ class MainWindow(qt4.QMainWindow):
         # use current directory of file if not using cwd mode
         if not setdb['dirname_usecwd']:
             self.dirname = os.path.dirname( os.path.abspath(filename) )
-            self.dirname_export = self.dirname
 
         # notify cmpts which need notification that doc has finished opening
         self.documentOpened.emit()
@@ -1128,131 +1315,57 @@ class MainWindow(qt4.QMainWindow):
         """Populate the recently opened files menu with a list of
         recently opened files"""
 
+        def opener(path):
+            def _fileOpener():
+                self.openFile(path)
+            return _fileOpener
+
         menu = self.menus["file.filerecent"]
         menu.clear()
 
-        newMenuItems = []
         if setdb['main_recentfiles']:
             files = [f for f in setdb['main_recentfiles']
                      if os.path.isfile(f)]
-            self._openRecentFunctions = []
 
             # add each recent file to menu
+            newmenuitems = []
             for i, path in enumerate(files):
-
-                def fileOpener(filename=path):
-                    self.openFile(filename)
-
-                self._openRecentFunctions.append(fileOpener)
-                newMenuItems.append(('filerecent%i' % i, _('Open File %s') % path,
-                                     os.path.basename(path),
-                                     'file.filerecent', fileOpener,
-                                     '', False, ''))
+                newmenuitems.append(
+                    ('filerecent%i' % i,_('Open File %s') % path,
+                     os.path.basename(path),
+                     'file.filerecent', opener(path),
+                     '', False, ''))
 
             menu.setEnabled(True)
             self.recentFileActions = utils.populateMenuToolbars(
-                newMenuItems, self.maintoolbar, self.menus)
+                newmenuitems, self.maintoolbar, self.menus)
         else:
             menu.setEnabled(False)
 
+    def slotFileReload(self):
+        """Reload document from saved version."""
+
+        retn = qt4.QMessageBox.warning(
+            self,
+            _("Reload file"),
+            _("Reload document from file, losing any changes?"),
+            qt4.QMessageBox.Yes | qt4.QMessageBox.Cancel,
+            qt4.QMessageBox.Cancel)
+        if retn == qt4.QMessageBox.Yes:
+            if not os.path.exists(self.filename):
+                qt4.QMessageBox.critical(
+                    self,
+                    _("Reload file"),
+                    _("File %s no longer exists") % self.filename)
+            else:
+                self.openFileInWindow(self.filename)
+
     def slotFileExport(self):
         """Export the graph."""
-
-        # check there is a page
-        if self.document.getNumberPages() == 0:
-            qt4.QMessageBox.warning(self, _("Error - Veusz"),
-                                    _("No pages to export"))
-            return
-
-        # File types we can export to in the form ([extensions], Name)
-        fd = qt4.QFileDialog(self, _('Export page'))
-        fd.setDirectory( self.dirname_export )
-
-        fd.setFileMode( qt4.QFileDialog.AnyFile )
-        fd.setAcceptMode( qt4.QFileDialog.AcceptSave )
-
-        # Create a mapping between a format string and extensions
-        filtertoext = {}
-        # convert extensions to filter
-        exttofilter = {}
-        filters = []
-        # a list of extensions which are allowed
-        validextns = []
-        formats = document.Export.formats
-        for extns, name in formats:
-            extensions = " ".join(["*." + item for item in extns])
-            # join eveything together to make a filter string
-            filterstr = '%s (%s)' % (name, extensions)
-            filtertoext[filterstr] = extns
-            for e in extns:
-                exttofilter[e] = filterstr
-            filters.append(filterstr)
-            validextns += extns
-        fd.setNameFilters(filters)
-
-        # restore last format if possible
-        try:
-            filt = setdb['export_lastformat']
-            fd.selectNameFilter(filt)
-            extn = formats[filters.index(filt)][0][0]
-        except (KeyError, IndexError, ValueError):
-            extn = 'pdf'
-            fd.selectNameFilter( exttofilter[extn] )
-
-        if self.filename:
-            # try to convert current filename to export name
-            filename = os.path.basename(self.filename)
-            filename = os.path.splitext(filename)[0] + '.' + extn
-            fd.selectFile(filename)
-
-        if fd.exec_() == qt4.QDialog.Accepted:
-            # save directory for next time
-            self.dirname_export = fd.directory().absolutePath()
-
-            filterused = str(fd.selectedFilter())
-            setdb['export_lastformat'] = filterused
-
-            chosenextns = filtertoext[filterused]
-
-            # show busy cursor
-            qt4.QApplication.setOverrideCursor( qt4.QCursor(qt4.Qt.WaitCursor) )
-
-            filename = fd.selectedFiles()[0]
-
-            # Add a default extension if one isn't supplied
-            # this is the extension without the dot
-            ext = os.path.splitext(filename)[1][1:]
-            if (ext not in validextns) and (ext not in chosenextns):
-                filename += "." + chosenextns[0]
-
-            export = document.Export(
-                self.document,
-                filename,
-                self.plot.getPageNumber(),
-                bitmapdpi=setdb['export_DPI'],
-                pdfdpi=setdb['export_DPI_PDF'],
-                antialias=setdb['export_antialias'],
-                color=setdb['export_color'],
-                quality=setdb['export_quality'],
-                backcolor=setdb['export_background'],
-                svgtextastext=setdb['export_SVG_text_as_text'],
-                )
-
-            try:
-                export.export()
-            except (RuntimeError, EnvironmentError) as e:
-                if isinstance(e, EnvironmentError):
-                    msg = cstrerror(e)
-                else:
-                    msg = cstr(e)
-
-                qt4.QApplication.restoreOverrideCursor()
-                qt4.QMessageBox.critical(
-                    self, _("Error - Veusz"),
-                    _("Error exporting to file '%s'\n\n%s") %
-                    (filename, msg))
-            else:
-                qt4.QApplication.restoreOverrideCursor()
+        from ..dialogs.export import ExportDialog
+        dialog = ExportDialog(self, self.document, self.filename)
+        self.showDialog(dialog)
+        return dialog
 
     def slotFilePrint(self):
         """Print the document."""
@@ -1264,6 +1377,10 @@ class MainWindow(qt4.QMainWindow):
         # enable/disable file, save menu item
         self.vzactions['file.save'].setEnabled(ismodified)
 
+        # enable/disable reloading from saved document
+        self.vzactions['file.reload'].setEnabled(
+            bool(self.filename) and ismodified)
+
     def slotFileClose(self):
         """File close window chosen."""
         self.close()
@@ -1337,6 +1454,6 @@ class MainWindow(qt4.QMainWindow):
 
     def slotAllowedImportsDoc(self, module, names):
         """Are allowed imports?"""
-
+        from ..dialogs.safetyimport import SafetyImportDialog
         d = SafetyImportDialog(self, module, names)
         d.exec_()
