@@ -1,6 +1,6 @@
 # plotwindow.py
 # the main window for showing plots
- 
+
 #    Copyright (C) 2004 Jeremy S. Sanders
 #    Email: Jeremy Sanders <jeremy@jeremysanders.net>
 #
@@ -68,6 +68,10 @@ class PickerCrosshairItem( qt4.QGraphicsPathItem ):
 class RenderControl(qt4.QObject):
     """Object for rendering plots in a separate thread."""
 
+    # emitted when new item on plot queue
+    sigQueueChange = qt4.pyqtSignal(int)
+
+    # when a rendering job is finished
     signalRenderFinished = qt4.pyqtSignal(
         int, qt4.QImage, document.PaintHelper)
 
@@ -129,8 +133,9 @@ class RenderControl(qt4.QObject):
 
         # don't process jobs which have been superseded
         if lastadded == jobid:
-            img = qt4.QImage(helper.pagesize[0], helper.pagesize[1],
-                             qt4.QImage.Format_ARGB32_Premultiplied)
+            img = qt4.QImage(
+                int(helper.rawpagesize[0]), int(helper.rawpagesize[1]),
+                qt4.QImage.Format_ARGB32_Premultiplied)
             img.fill( setting.settingdb.color('page').rgb() )
 
             painter = qt4.QPainter(img)
@@ -148,13 +153,13 @@ class RenderControl(qt4.QObject):
             self.mutex.unlock()
 
         # tell any listeners that a job has been processed
-        self.plotwindow.sigQueueChange.emit(-1)
+        self.sigQueueChange.emit(-1)
 
     def addJob(self, helper):
         """Process drawing job in PaintHelper given."""
 
         # indicate that there is a new item to be processed to listeners
-        self.plotwindow.sigQueueChange.emit(1)
+        self.sigQueueChange.emit(1)
 
         # add the job to the queue
         self.mutex.lock()
@@ -194,6 +199,16 @@ class RenderThread( qt4.QThread ):
                 sys.stderr.write(_("Error in rendering thread\n"))
                 traceback.print_exc(file=sys.stderr)
 
+class ControlGraphRoot(qt4.QGraphicsItem):
+    """Control graph items are connected to this root item.
+    We don't use a group here as it would swallow parent events."""
+    def __init__(self):
+        qt4.QGraphicsItem.__init__(self)
+    def paint(self, painter, option, widget=None):
+        pass
+    def boundingRect(self):
+        return qt4.QRectF()
+
 class PlotWindow( qt4.QGraphicsView ):
     """Class to show the plot(s) in a scrollable window."""
 
@@ -208,7 +223,7 @@ class PlotWindow( qt4.QGraphicsView ):
     # axis values update from moving mouse
     sigAxisValuesFromMouse = qt4.pyqtSignal(dict)
     # gives widget clicked
-    sigWidgetClicked = qt4.pyqtSignal(object)
+    sigWidgetClicked = qt4.pyqtSignal(object, str)
 
     # how often the document can update
     updateintervals = (
@@ -246,13 +261,12 @@ class PlotWindow( qt4.QGraphicsView ):
         self.vzactions = None
 
         # for controlling plot elements
-        g = self.controlgraphgroup = qt4.QGraphicsItemGroup()
-        g.setHandlesChildEvents(False)
-        self.scene.addItem(g)
+        self.controlgraphroot = ControlGraphRoot()
+        self.scene.addItem(self.controlgraphroot)
 
         # zoom rectangle for zooming into graph (not shown normally)
-        self.zoomrect = self.scene.addRect( 0, 0, 100, 100,
-                                            qt4.QPen(qt4.Qt.DotLine) )
+        self.zoomrect = self.scene.addRect(
+            0, 0, 100, 100, qt4.QPen(qt4.Qt.DotLine))
         self.zoomrect.setZValue(2.)
         self.zoomrect.hide()
 
@@ -287,6 +301,8 @@ class PlotWindow( qt4.QGraphicsView ):
         self.rendercontrol = RenderControl(self)
         self.rendercontrol.signalRenderFinished.connect(
             self.slotRenderFinished)
+        self.rendercontrol.sigQueueChange.connect(
+            self.sigQueueChange)
 
         # mode for clicking
         self.clickmode = 'select'
@@ -392,7 +408,7 @@ class PlotWindow( qt4.QGraphicsView ):
                       icon='kde-zoom-page-veuszedit'),
                 'view.zoommenu':
                     a(self, _('Zoom functions menu'), _('Zoom'),
-                      self.doZoomMenuButton,
+                      None,
                       icon='kde-zoom-veuszedit'),
                 'view.prevpage':
                     a(self, _('Move to the previous page'), _('&Previous page'),
@@ -439,27 +455,21 @@ class PlotWindow( qt4.QGraphicsView ):
             utils.constructMenus(menu, {'view': menu}, menuitems,
                                  actions)
 
-        # populate menu on zoom menu toolbar icon
-        zoommenu = qt4.QMenu(self)
-        zoomag = qt4.QActionGroup(self)
-        for act in ('view.zoomin', 'view.zoomout', 'view.zoom11',
-                    'view.zoomwidth', 'view.zoomheight', 'view.zoompage'):
-            a = actions[act]
-            zoommenu.addAction(a)
-            zoomag.addAction(a)
-            a.vzname = act
-        actions['view.zoommenu'].setMenu(zoommenu)
-        zoomag.triggered.connect(self.zoomActionTriggered)
-
-        lastzoom = setting.settingdb.get('view_defaultzoom', 'view.zoompage')
-        self.updateZoomMenuButton(actions[lastzoom])
+        # populate menu on zoom toolbar icon
+        utils.makeMenuGroupSaved(
+            'view.zoommenu', self, actions, (
+                'view.zoomin', 'view.zoomout', 'view.zoom11',
+                'view.zoomwidth', 'view.zoomheight', 'view.zoompage',
+            ))
 
         # add items to toolbar
-        utils.addToolbarActions(self.viewtoolbar, actions,
-                                ('view.prevpage', 'view.nextpage',
-                                 'view.fullscreen',
-                                 'view.select', 'view.pick',
-                                 'view.zoomgraph', 'view.zoommenu'))
+        utils.addToolbarActions(
+            self.viewtoolbar, actions, (
+                'view.prevpage', 'view.nextpage',
+                'view.fullscreen',
+                'view.select', 'view.pick',
+                'view.zoomgraph', 'view.zoommenu',
+            ))
 
         # define action group for various different selection models
         grp = self.selectactiongrp = qt4.QActionGroup(self)
@@ -472,22 +482,6 @@ class PlotWindow( qt4.QGraphicsView ):
 
         return self.viewtoolbar
 
-    def zoomActionTriggered(self, action):
-        """Keep track of the last zoom action selected."""
-        setting.settingdb['view_defaultzoom'] = action.vzname
-        self.updateZoomMenuButton(action)
-
-    def updateZoomMenuButton(self, action):
-        """Make zoom button call default zoom action and change icon."""
-        menuact = self.vzactions['view.zoommenu']
-        setting.settingdb['view_defaultzoom'] = action.vzname
-        menuact.setIcon( action.icon() )
-
-    def doZoomMenuButton(self):
-        """Select previous zoom option when clicking on zoom menu."""
-        act = self.vzactions[setting.settingdb['view_defaultzoom']]
-        act.trigger()
-
     def doZoomRect(self, endpos):
         """Take the zoom rectangle drawn by the user and do the zooming.
         endpos is a QPoint end point
@@ -513,20 +507,24 @@ class PlotWindow( qt4.QGraphicsView ):
         if abs((pt2-pt1).x()) < 10 or abs((pt2-pt1).y()) < 10:
             return
 
+        # scale into graph coordinates
+        pt1 /= self.painthelper.cgscale
+        pt2 /= self.painthelper.cgscale
+
         # try to work out in which widget the first point is in
         widget = self.painthelper.pointInWidgetBounds(
             pt1.x(), pt1.y(), widgets.Graph)
         if widget is None:
             return
-        
+
         # convert points on plotter to points on axis for each axis
         # we also add a neighbouring pixel for the rounding calculation
         xpts = N.array( [pt1.x(), pt2.x(), pt1.x()+1, pt2.x()-1] )
-        ypts = N.array( [pt1.y(), pt2.y(), pt2.y()+1, pt2.y()-1] )
+        ypts = N.array( [pt1.y(), pt2.y(), pt1.y()+1, pt2.y()-1] )
 
         # build up operation list to do zoom
         operations = []
-        
+
         axes = {}
         # iterate over children, to look for plotters
         for c in [i for i in widget.children if
@@ -583,7 +581,8 @@ class PlotWindow( qt4.QGraphicsView ):
             return []
 
         pos = self.mapToScene(mousepos)
-        px, py = pos.x(), pos.y()
+        px = pos.x() / self.painthelper.cgscale
+        py = pos.y() / self.painthelper.cgscale
 
         axes = []
         for widget, bounds in self.painthelper.widgetBoundsIterator(
@@ -606,7 +605,9 @@ class PlotWindow( qt4.QGraphicsView ):
         """Report that a new point has been picked"""
 
         self.pickerinfo = pickinfo
-        self.pickeritem.setPos(pickinfo.screenpos[0], pickinfo.screenpos[1])
+        self.pickeritem.setPos(
+            pickinfo.graphpos[0] * self.painthelper.cgscale,
+            pickinfo.graphpos[1] * self.painthelper.cgscale)
         self.sigPointPicked.emit(pickinfo)
 
     def doPick(self, mousepos):
@@ -615,22 +616,26 @@ class PlotWindow( qt4.QGraphicsView ):
         self.pickerwidgets = []
 
         pickinfo = widgets.PickInfo()
+        # get scalable graph coordinates for mouse point
         pos = self.mapToScene(mousepos)
+        pos /= self.painthelper.cgscale
 
         for w, bounds in self.painthelper.widgetBoundsIterator():
+            # ask the widget for its (visually) closest point to the cursor
             try:
-                # ask the widget for its (visually) closest point to the cursor
                 info = w.pickPoint(pos.x(), pos.y(), bounds)
+            except AttributeError:
+                # widget isn't pickable
+                continue
 
-                # this is a pickable widget, so remember it for future key navigation
+            if info:
+                # this is a pickable widget, so remember it for future
+                # key navigation
                 self.pickerwidgets.append(w)
 
                 if info.distance < pickinfo.distance:
                     # and remember the overall closest
                     pickinfo = info
-            except AttributeError:
-                # ignore widgets that don't support axes or picking
-                continue
 
         if not pickinfo:
             self.pickeritem.hide()
@@ -650,14 +655,11 @@ class PlotWindow( qt4.QGraphicsView ):
         """Allow user to drag window around."""
 
         qt4.QGraphicsView.mousePressEvent(self, event)
+        if self.painthelper is None:
+            return
 
         # work out whether user is clicking on a control point
-        # we have to ignore the item group which seems to be above
-        # its constituents
         items = self.items(event.pos())
-        if len(items) > 0 and isinstance(items[0], qt4.QGraphicsItemGroup):
-            del items[0]
-
         self.ignoreclick = ( len(items)==0 or
                              items[0] is not self.pixmapitem or
                              self.painthelper is None )
@@ -685,8 +687,9 @@ class PlotWindow( qt4.QGraphicsView ):
                     qt4.QCursor(qt4.Qt.SizeAllCursor))
 
             elif self.clickmode == 'graphzoom':
-                self.zoomrect.setRect(self.grabpos.x(), self.grabpos.y(),
-                                      0, 0)
+                self.zoomrect.setRect(
+                    self.grabpos.x(), self.grabpos.y(),
+                    0, 0)
                 self.zoomrect.show()
 
                 #self.label.drawRect(self.grabpos, self.grabpos)
@@ -698,6 +701,8 @@ class PlotWindow( qt4.QGraphicsView ):
         """Scroll window by how much the mouse has moved since last time."""
 
         qt4.QGraphicsView.mouseMoveEvent(self, event)
+        if self.painthelper is None:
+            return
 
         if self.currentclickmode == 'scroll':
             event.accept()
@@ -716,10 +721,15 @@ class PlotWindow( qt4.QGraphicsView ):
             self.winpos = qt4.QPoint(event.pos())
 
         elif self.currentclickmode == 'graphzoom' and self.grabpos is not None:
-            pos = self.mapToScene(event.pos())
-            r = self.zoomrect.rect()
-            self.zoomrect.setRect( r.x(), r.y(), pos.x()-r.x(),
-                                   pos.y()-r.y() )
+            pos2 = self.mapToScene(event.pos())
+            self.zoomrect.setRect(qt4.QRectF(
+                qt4.QPointF(
+                    min(self.grabpos.x(), pos2.x()),
+                    min(self.grabpos.y(), pos2.y())),
+                qt4.QPointF(
+                    max(self.grabpos.x(), pos2.x()),
+                    max(self.grabpos.y(), pos2.y())),
+                ))
 
         elif self.clickmode == 'select' or self.clickmode == 'pick':
             # find axes which map to this position
@@ -734,9 +744,11 @@ class PlotWindow( qt4.QGraphicsView ):
 
     def mouseReleaseEvent(self, event):
         """If the mouse button is released, check whether the mouse
-        clicked on a widget, and emit a sigWidgetClicked(widget)."""
+        clicked on a widget, and emit a sigWidgetClicked(widget,mode)."""
 
         qt4.QGraphicsView.mouseReleaseEvent(self, event)
+        if self.painthelper is None:
+            return
 
         if event.button() == qt4.Qt.LeftButton and not self.ignoreclick:
             event.accept()
@@ -744,7 +756,7 @@ class PlotWindow( qt4.QGraphicsView ):
             if self.currentclickmode == 'select':
                 # work out where the mouse clicked and choose widget
                 pos = self.mapToScene(event.pos())
-                self.locateClickWidget(pos.x(), pos.y())
+                self.identifyAndClickWidget(pos.x(), pos.y(), event.modifiers())
             elif self.currentclickmode == 'scroll':
                 # return the cursor to normal after scrolling
                 self.clickmode = 'select'
@@ -792,13 +804,14 @@ class PlotWindow( qt4.QGraphicsView ):
 
                     # ask the widgets to pick their point which is closest horizontally
                     # to the last (screen) x value picked
-                    pi = w.pickPoint(self.pickerinfo.screenpos[0], p.y(),
-                                     self.painthelper.widgetBounds(w),
-                                     distance='horizontal')
+                    pi = w.pickPoint(
+                        self.pickerinfo.graphpos[0], p.y(),
+                        self.painthelper.widgetBounds(w),
+                        distance='horizontal')
                     if not pi:
                         continue
 
-                    dy = p.y() - pi.screenpos[1]
+                    dy = p.y() - pi.graphpos[1]
 
                     # take the new point which is closest vertically to the current
                     # one and either above or below it as appropriate
@@ -808,12 +821,12 @@ class PlotWindow( qt4.QGraphicsView ):
                         dist = abs(dy)
 
                 if pickinfo:
-                    oldx = self.pickerinfo.screenpos[0]
+                    oldx = self.pickerinfo.graphpos[0]
                     self.emitPicked(pickinfo)
 
                     # restore the previous x-position, so that vertical navigation
                     # stays repeatable
-                    pickinfo.screenpos = (oldx, pickinfo.screenpos[1])
+                    pickinfo.graphpos = (oldx, pickinfo.graphpos[1])
 
                 return
 
@@ -824,32 +837,34 @@ class PlotWindow( qt4.QGraphicsView ):
         """For zooming in or moving."""
 
         if event.modifiers() & qt4.Qt.ControlModifier:
-            self.sumwheeldelta += event.delta()
+            # zoom in/out with ctrl held down
+            d = event.angleDelta()
+            delta = d.x() if d.x() != 0 else d.y()
+            self.sumwheeldelta += delta
             while self.sumwheeldelta <= -120:
                 self.slotViewZoomOut()
                 self.sumwheeldelta += 120
             while self.sumwheeldelta >= 120:
                 self.slotViewZoomIn()
                 self.sumwheeldelta -= 120
+            event.accept()
         elif event.modifiers() & qt4.Qt.ShiftModifier:
-            self.sumwheeldelta += event.delta()
-            while self.sumwheeldelta <= -120:
-                # scroll left
-                self.sumwheeldelta += 120
-                scrollx = self.horizontalScrollBar()
-                scrollx.setValue(scrollx.value() + 120)
-            while self.sumwheeldelta >= 120:
-                # scroll right
-                scrollx = self.horizontalScrollBar()
-                scrollx.setValue(scrollx.value() - 120)
-                self.sumwheeldelta -= 120
+            # scroll horizontally if shift is held down
+            d = event.pixelDelta()
+            if d.isNull():
+                # Fallback mode to angleDelta
+                d = event.angleDelta()
+            delta = d.x() if d.x() != 0 else d.y()
+            scrollx = self.horizontalScrollBar()
+            scrollx.setValue(scrollx.value() + delta)
+            event.accept()
         else:
             qt4.QGraphicsView.wheelEvent(self, event)
 
-    def locateClickWidget(self, x, y):
+    def identifyAndClickWidget(self, x, y, modifier):
         """Work out which widget was clicked, and if necessary send
         a sigWidgetClicked(widget) signal."""
-        
+
         if self.document.getNumberPages() == 0:
             return
 
@@ -861,7 +876,14 @@ class PlotWindow( qt4.QGraphicsView ):
 
         # tell connected objects that widget was clicked
         if widget is not None:
-            self.sigWidgetClicked.emit(widget)
+            if modifier & qt4.Qt.ControlModifier:
+                mode = 'toggle'
+            elif modifier & qt4.Qt.ShiftModifier:
+                mode = 'add'
+            else:
+                mode = 'new'
+
+            self.sigWidgetClicked.emit(widget, mode)
 
     def setPageNumber(self, pageno):
         """Move the the selected page."""
@@ -891,64 +913,87 @@ class PlotWindow( qt4.QGraphicsView ):
         if ismodified and self.interval == -1:
             self.checkPlotUpdate()
 
+    def getDevicePixelRatio(self):
+        """Get device pixel ratio for window."""
+        # ugly, as we have to get it from QWindow
+        widget = self
+        while widget:
+            window = widget.windowHandle()
+            if window is not None:
+                try:
+                    return window.devicePixelRatio()
+                except AttributeError:
+                    return 1
+            widget = widget.parent()
+        return 1
+
     def checkPlotUpdate(self):
         """Check whether plot needs updating."""
 
-        # print >>sys.stderr, "checking update"
         # no threads, so can't get interrupted here
         # draw data into background pixmap if modified
-        if ( self.zoomfactor != self.oldzoom or
-             self.document.changeset != self.docchangeset or
-             self.pagenumber != self.oldpagenumber ):
 
-            # print >>sys.stderr, "updating"
-            self.pickeritem.hide()
-            
-            self.pagenumber = min( self.document.getNumberPages() - 1,
-                                   self.pagenumber )
-            self.oldpagenumber = self.pagenumber
+        # is an update required?
+        if ( self.zoomfactor == self.oldzoom and
+             self.document.changeset == self.docchangeset and
+             self.pagenumber == self.oldpagenumber ):
+            return
 
-            if self.pagenumber >= 0:
-                size = self.document.pageSize(
-                    self.pagenumber, scaling=self.zoomfactor)
+        self.pickeritem.hide()
 
-                # draw the data into the buffer
-                # errors cause an exception window to pop up
-                try:
-                    phelper = document.PaintHelper(
-                        size, scaling=self.zoomfactor, dpi=self.dpi)
-                    self.document.paintTo(phelper, self.pagenumber)
+        # do we need the following line?
+        self.pagenumber = min(
+            self.document.getNumberPages()-1, self.pagenumber)
+        self.oldpagenumber = self.pagenumber
 
-                except Exception:
-                    # stop updates this time round and show exception dialog
-                    d = exceptiondialog.ExceptionDialog(sys.exc_info(), self)
-                    self.oldzoom = self.zoomfactor
-                    self.docchangeset = self.document.changeset
-                    d.exec_()
+        if self.pagenumber >= 0:
+            devicepixelratio = self.getDevicePixelRatio()
+            scaling = self.zoomfactor*devicepixelratio
+            size = self.document.pageSize(
+                self.pagenumber, scaling=scaling, integer=False)
 
-                self.painthelper = phelper
-                self.rendercontrol.addJob(phelper)
-            else:
-                self.painthelper = None
-                self.pagenumber = 0
-                size = self.document.docSize()
-                pixmap = qt4.QPixmap(*size)
-                pixmap.fill( setting.settingdb.color('page') )
-                self.setSceneRect(0, 0, *size)
-                self.pixmapitem.setPixmap(pixmap)
+            # draw the data into the buffer
+            # errors cause an exception window to pop up
+            try:
+                phelper = document.PaintHelper(
+                    self.document, size,
+                    scaling=scaling,
+                    dpi=self.dpi,
+                    devicepixelratio=devicepixelratio)
+                self.document.paintTo(phelper, self.pagenumber)
 
-            self.sigUpdatePage.emit(self.pagenumber)
-            self.updatePageToolbar()
+            except Exception:
+                # stop updates this time round and show exception dialog
+                d = exceptiondialog.ExceptionDialog(sys.exc_info(), self)
+                self.oldzoom = self.zoomfactor
+                self.docchangeset = self.document.changeset
+                d.exec_()
 
-            self.updateControlGraphs(self.lastwidgetsselected)
-            self.oldzoom = self.zoomfactor
-            self.docchangeset = self.document.changeset
+            self.painthelper = phelper
+            self.rendercontrol.addJob(phelper)
+        else:
+            self.painthelper = None
+            self.pagenumber = 0
+            size = self.document.docSize()
+            pixmap = qt4.QPixmap(*size)
+            pixmap.fill( setting.settingdb.color('page') )
+            self.setSceneRect(0, 0, *size)
+            self.pixmapitem.setPixmap(pixmap)
+
+        self.sigUpdatePage.emit(self.pagenumber)
+        self.updatePageToolbar()
+
+        self.updateControlGraphs(self.lastwidgetsselected)
+        self.oldzoom = self.zoomfactor
+        self.docchangeset = self.document.changeset
 
     def slotRenderFinished(self, jobid, img, helper):
         """Update image on display if rendering (usually in other
         thread) finished."""
+        dpr = helper.devicepixelratio
         bufferpixmap = qt4.QPixmap.fromImage(img)
-        self.setSceneRect(0, 0, bufferpixmap.width(), bufferpixmap.height())
+        bufferpixmap.setDevicePixelRatio(dpr)
+        self.setSceneRect(0, 0, bufferpixmap.width()/dpr, bufferpixmap.height()/dpr)
         self.pixmapitem.setPixmap(bufferpixmap)
 
     def updatePlotSettings(self):
@@ -1048,6 +1093,7 @@ class PlotWindow( qt4.QGraphicsView ):
 
     def setZoomFactor(self, zoomfactor):
         """Set the zoom factor of the window."""
+        zoomfactor = max(0.05, min(20, zoomfactor))
         self.zoomfactor = float(zoomfactor)
         self.checkPlotUpdate()
 
@@ -1072,25 +1118,28 @@ class PlotWindow( qt4.QGraphicsView ):
         if aspectwin > aspectplot:
             # take account of scroll bar
             width -= self.verticalScrollBar().width()
-            
+
         mult = width / r.width()
         self.setZoomFactor(self.zoomfactor * mult)
-        
+
     def slotViewZoomHeight(self):
         """Make the zoom factor so that the plot fills the whole width."""
 
-        # need to take account of scroll bars when deciding size
         viewportsize = self.maximumViewportSize()
-        aspectwin = viewportsize.width() / viewportsize.height()
-        r = self.pixmapitem.boundingRect()
-        aspectplot = r.width() / r.height()
+        pixrect = self.pixmapitem.boundingRect()
+
+        try:
+            aspectwin = viewportsize.width() / viewportsize.height()
+            aspectplot = pixrect.width() / pixrect.height()
+        except ZeroDivisionError:
+            return
 
         height = viewportsize.height()
         if aspectwin < aspectplot:
             # take account of scroll bar
             height -= self.horizontalScrollBar().height()
-            
-        mult = height / r.height()
+
+        mult = height / pixrect.height()
         self.setZoomFactor(self.zoomfactor * mult)
 
     def slotViewZoomPage(self):
@@ -1110,7 +1159,7 @@ class PlotWindow( qt4.QGraphicsView ):
     def slotViewPreviousPage(self):
         """View the previous page."""
         self.setPageNumber( self.pagenumber - 1 )
- 
+
     def slotViewNextPage(self):
         """View the next page."""
         self.setPageNumber( self.pagenumber + 1 )
@@ -1130,7 +1179,7 @@ class PlotWindow( qt4.QGraphicsView ):
         modecnvt = { self.vzactions['view.select'] : 'select',
                      self.vzactions['view.pick'] : 'pick',
                      self.vzactions['view.zoomgraph'] : 'graphzoom' }
-        
+
         # close the current picker
         self.pickeritem.hide()
         self.sigPickerEnabled.emit(False)
@@ -1140,14 +1189,12 @@ class PlotWindow( qt4.QGraphicsView ):
 
         if self.clickmode == 'select':
             self.pixmapitem.unsetCursor()
-            #self.label.setCursor(qt4.Qt.ArrowCursor)
         elif self.clickmode == 'graphzoom':
-            self.pixmapitem.unsetCursor()
-            #self.label.setCursor(qt4.Qt.CrossCursor)
+            self.pixmapitem.setCursor(qt4.Qt.CrossCursor)
         elif self.clickmode == 'pick':
             self.pixmapitem.setCursor(qt4.Qt.CrossCursor)
             self.sigPickerEnabled.emit(True)
-        
+
     def getClick(self):
         """Return a click point from the graph."""
 
@@ -1168,7 +1215,7 @@ class PlotWindow( qt4.QGraphicsView ):
             pt.x(), pt.y(), widgets.Graph)
         if widget is None:
             return []
-        
+
         # convert points on plotter to points on axis for each axis
         xpts = N.array( [pt.x()] )
         ypts = N.array( [pt.y()] )
@@ -1209,21 +1256,17 @@ class PlotWindow( qt4.QGraphicsView ):
     def updateControlGraphs(self, widgets):
         """Add control graphs for the widgets given."""
 
-        cgg = self.controlgraphgroup
-
-        # delete old items
-        for c in cgg.childItems():
-            cgg.removeFromGroup(c)
+        # delete old items from root
+        for c in list(self.controlgraphroot.childItems()):
             self.scene.removeItem(c)
 
-        # add each item to the group
+        # add each item to the root
         if self.painthelper:
             for widget in widgets:
                 cgis = self.painthelper.getControlGraph(widget)
                 if cgis:
                     for control in cgis:
-                        graphitem = control.createGraphicsItem()
-                        cgg.addToGroup(graphitem)
+                        control.createGraphicsItem(self.controlgraphroot)
 
 class FullScreenPlotWindow(qt4.QScrollArea):
     """Window for showing plot in full-screen mode."""
