@@ -246,7 +246,7 @@ class SettingsProxyMulti(SettingsProxy):
         if self._root:
             sname = self._root + '/' + sname
         for w in self.widgets:
-            s = self.document.resolveFullSettingPath(w.path + '/' + sname)
+            s = self.document.resolveSettingPath(None, w.path+'/'+sname)
             if s.val != val:
                 ops.append(document.OperationSettingSet(s, val))
         # apply all operations
@@ -306,7 +306,7 @@ class PropertyList(qt4.QWidget):
 
         self.layout = qt4.QGridLayout(self)
         self.layout.setSpacing( self.layout.spacing()//2 )
-        self.layout.setMargin(4)
+        self.layout.setContentsMargins(4,4,4,4)
         
         self.childlist = []
         self.setncntrls = {}     # map setting name to controls
@@ -331,8 +331,9 @@ class PropertyList(qt4.QWidget):
 
             button = qt4.QPushButton(text)
             button.setToolTip(action.descr)
-            button.clicked[()].connect(
-                lambda a=action: setnsproxy.onAction(a, self.getConsole()))
+            button.clicked.connect(
+                lambda checked=True, a=action:
+                setnsproxy.onAction(a, self.getConsole()))
 
             self.layout.addWidget(button, row, 1)
             self.childlist.append(button)
@@ -495,6 +496,7 @@ class TabbedFormatting(qt4.QTabWidget):
 
     def __init__(self, document, setnsproxy, shownames=False):
         qt4.QTabWidget.__init__(self)
+        self.setUsesScrollButtons(True)
         self.document = document
 
         if setnsproxy is None:
@@ -580,7 +582,7 @@ class TabbedFormatting(qt4.QTabWidget):
 
         # layout for tab widget
         layout = qt4.QVBoxLayout()
-        layout.setMargin(2)
+        layout.setContentsMargins(2,2,2,2)
         layout.addWidget(scroll)
 
         # finally use layout containing items for tab
@@ -664,11 +666,11 @@ class TreeEditDock(qt4.QDockWidget):
         self.selwidgets = []
 
         self.document = document
-        self.document.sigWiped.connect(self.slotDocumentWiped)
 
         # construct tree
         self.treemodel = WidgetTreeModel(document)
         self.treeview = WidgetTreeView(self.treemodel)
+        self.document.sigWiped.connect(self.slotDocumentWiped)
 
         # receive change in selection
         self.treeview.selectionModel().selectionChanged.connect(
@@ -797,7 +799,8 @@ class TreeEditDock(qt4.QDockWidget):
             selw = self.selwidgets[0]
 
         # has to be visible if is to be enabled (yuck)
-        nonorth = self.vzactions['add.nonorthpoint'].setVisible(True)
+        self.vzactions['add.nonorthpoint'].setVisible(True)
+        self.vzactions['add.point3d'].setVisible(True)
 
         # check whether each button can have this widget
         # (or a parent) as parent
@@ -813,10 +816,19 @@ class TreeEditDock(qt4.QDockWidget):
 
         # exclusive widgets
         nonorth = self.vzactions['add.nonorthpoint'].isEnabled()
+        in3dgraph = self.vzactions['add.point3d'].isEnabled()
         self.vzactions['add.nonorthpoint'].setVisible(nonorth)
-        self.vzactions['add.xy'].setVisible(not nonorth)
+        self.vzactions['add.point3d'].setVisible(in3dgraph)
+        self.vzactions['add.xy'].setVisible(not nonorth and not in3dgraph)
         self.vzactions['add.nonorthfunc'].setVisible(nonorth)
-        self.vzactions['add.function'].setVisible(not nonorth)
+        self.vzactions['add.function'].setVisible(not nonorth and not in3dgraph)
+        self.vzactions['add.function3d'].setVisible(in3dgraph)
+        self.vzactions['add.axismenu'].setVisible(not in3dgraph)
+        self.vzactions['add.axis3d'].setVisible(in3dgraph)
+        self.vzactions['add.image'].setVisible(not in3dgraph)
+        self.vzactions['add.surface3d'].setVisible(in3dgraph)
+        self.vzactions['add.contour'].setVisible(not in3dgraph)
+        self.vzactions['add.volume3d'].setVisible(in3dgraph)
 
         # certain actions shouldn't work on root
         isnotroot = not any([isinstance(w, widgets.Root)
@@ -831,6 +843,9 @@ class TreeEditDock(qt4.QDockWidget):
     def _constructToolbarMenu(self):
         """Add items to edit/add graph toolbar and menu."""
 
+        def slotklass(klass):
+            return lambda: self.slotMakeWidgetButton(klass)
+
         iconsize = setting.settingdb['toolbar_size']
         self.addtoolbar.setIconSize( qt4.QSize(iconsize, iconsize) )
         self.edittoolbar.setIconSize( qt4.QSize(iconsize, iconsize) )
@@ -844,12 +859,14 @@ class TreeEditDock(qt4.QDockWidget):
                            'key', 'label', 'colorbar',
                            'rect', 'ellipse', 'imagefile',
                            'line', 'polygon', 'polar', 'ternary',
-                           'nonorthpoint', 'nonorthfunc'):
+                           'nonorthpoint', 'nonorthfunc',
+                           'covariance',
+                           'scene3d',
+                           'graph3d', 'function3d', 'point3d', 'axis3d',
+                           'surface3d', 'volume3d'):
 
             wc = document.thefactory.getWidgetClass(widgettype)
-            def slotfn(klass=wc):
-                return lambda: self.slotMakeWidgetButton(klass)
-            slot = slotfn(wc)
+            slot = slotklass(wc)
             self.addslots[wc] = slot
 
             actionname = 'add.' + widgettype
@@ -908,21 +925,29 @@ class TreeEditDock(qt4.QDockWidget):
             'add.grid',
             'add.graph',
             'add.axismenu',
+            'add.axis3d',
             'add.xy',
             'add.nonorthpoint',
+            'add.point3d',
             'add.bar',
             'add.fit',
             'add.function',
             'add.nonorthfunc',
+            'add.function3d',
             'add.boxplot',
             'add.image',
+            'add.surface3d',
             'add.contour',
+            'add.volume3d',
             'add.vectorfield',
             'add.key',
             'add.label',
             'add.colorbar',
             'add.polar',
             'add.ternary',
+            'add.scene3d',
+            'add.graph3d',
+            'add.covariance',
             'add.shapemenu',
             )
 
@@ -1008,6 +1033,9 @@ class TreeEditDock(qt4.QDockWidget):
         # select the widget
         self.selectWidget(w)
 
+        # bump the feedback statistics
+        utils.feedback.widgetcts[widgettype] += 1
+
     def slotWidgetCut(self):
         """Cut the selected widget"""
         self.slotWidgetCopy()
@@ -1097,18 +1125,30 @@ class TreeEditDock(qt4.QDockWidget):
         if len(selected) != 0:
             self.treeview.edit(selected[0])
 
-    def selectWidget(self, widget):
+    def selectWidget(self, widget, mode='new'):
         """Select the associated listviewitem for the widget w in the
-        listview."""
+        listview.
+
+        mode:
+         'new': new selection
+         'add': add to selection
+         'toggle': toggle selection
+        """
 
         index = self.treemodel.getWidgetIndex(widget)
+
         if index is not None:
             self.treeview.scrollTo(index)
-            self.treeview.selectionModel().select(
-                index, qt4.QItemSelectionModel.Clear |
-                qt4.QItemSelectionModel.Current |
-                qt4.QItemSelectionModel.Rows |
-                qt4.QItemSelectionModel.Select )
+
+            flags = qt4.QItemSelectionModel.Rows | {
+                'new':  (
+                    qt4.QItemSelectionModel.ClearAndSelect |
+                    qt4.QItemSelectionModel.Current),
+                'add': qt4.QItemSelectionModel.Select,
+                'toggle': qt4.QItemSelectionModel.Toggle,
+            }[mode]
+
+            self.treeview.selectionModel().select(index, flags)
 
     def slotWidgetMove(self, direction):
         """Move the selected widget up/down in the hierarchy.
@@ -1134,7 +1174,8 @@ class TreeEditDock(qt4.QDockWidget):
         hideshow is True for hiding, False for showing
         """
         ops = [ document.OperationSettingSet(w.settings.get('hide'), hideshow)
-                for w in widgets ]
+                for w in widgets
+                if 'hide' in w.settings ]
         descr = ('show', 'hide')[hideshow]
         self.document.applyOperation(
             document.OperationMultiple(ops, descr=descr))
@@ -1144,7 +1185,7 @@ class TreeEditDock(qt4.QDockWidget):
         if len(self.treeview.selectionModel().selectedRows()) == 0:
             self.selectWidget(self.document.basewidget)
 
-    def _selectWidgetsTypeAndOrName(self, wtype, wname):
+    def _selectWidgetsTypeAndOrName(self, wtype, wname, root=None):
         """Select widgets with type or name given.
         Give None if you don't care for either."""
         def selectwidget(path, w):
@@ -1156,7 +1197,7 @@ class TreeEditDock(qt4.QDockWidget):
                     idx, qt4.QItemSelectionModel.Select |
                     qt4.QItemSelectionModel.Rows)
 
-        self.document.walkNodes(selectwidget, nodetypes=('widget',))
+        self.document.walkNodes(selectwidget, nodetypes=('widget',), root=root)
 
     def _selectWidgetSiblings(self, w, wtype):
         """Select siblings of widget given with type."""
@@ -1179,21 +1220,32 @@ class TreeEditDock(qt4.QDockWidget):
         if len(self.selwidgets) == 0:
             return
 
-        wtype = self.selwidgets[0].typename
-        name = self.selwidgets[0].name
+        widget = self.selwidgets[0]
+        wtype = widget.typename
+        name = widget.name
+
+        # get page widget for selecting on page
+        page = widget
+        while page is not None and page.typename != 'page':
+            page = page.parent
 
         menu.addAction(
             _("All '%s' widgets") % wtype,
             lambda: self._selectWidgetsTypeAndOrName(wtype, None))
         menu.addAction(
             _("Siblings of '%s' with type '%s'") % (name, wtype),
-            lambda: self._selectWidgetSiblings(self.selwidgets[0], wtype))
+            lambda: self._selectWidgetSiblings(widget, wtype))
         menu.addAction(
             _("All '%s' widgets called '%s'") % (wtype, name),
             lambda: self._selectWidgetsTypeAndOrName(wtype, name))
         menu.addAction(
             _("All widgets called '%s'") % name,
             lambda: self._selectWidgetsTypeAndOrName(None, name))
+        if page and page is not widget:
+            menu.addAction(
+                _("All widgets called '%s' on page '%s'") % (name, page.name),
+                lambda: self._selectWidgetsTypeAndOrName(
+                    None, name, root=page))
 
 class SettingLabel(qt4.QWidget):
     """A label to describe a setting.
@@ -1218,7 +1270,7 @@ class SettingLabel(qt4.QWidget):
         self.setnsproxy = setnsproxy
 
         self.layout = qt4.QHBoxLayout(self)
-        self.layout.setMargin(2)
+        self.layout.setContentsMargins(2,2,2,2)
 
         if setting.usertext:
             text = setting.usertext
@@ -1374,26 +1426,36 @@ class SettingLabel(qt4.QWidget):
         name = widget.name
 
         popup = qt4.QMenu(self)
-        popup.addAction(_('Reset to default'),
-                        self.actionResetDefault)
+        popup.addAction(
+            _('Reset to default'),
+            self.actionResetDefault)
 
-        copyto = popup.addMenu(_('Copy to'))
-        copyto.addAction(_("all '%s' widgets") % wtype,
-                         self.actionCopyTypedWidgets)
-        copyto.addAction(_("'%s' siblings") % wtype,
-                         self.actionCopyTypedSiblings)
-        copyto.addAction(_("'%s' widgets called '%s'") % (wtype, name),
-                         self.actionCopyTypedNamedWidgets)
-        copyto.addSeparator()
-        self.addCopyToWidgets(copyto)
+        if self.setting.path[:12] != '/StyleSheet/':
+            # settings not relevant for style sheet items
 
-        popup.addAction(_('Use as default style'),
-                        self.actionSetStyleSheet)
+            copyto = popup.addMenu(_('Copy to'))
+            copyto.addAction(
+                _("all '%s' widgets") % wtype,
+                self.actionCopyTypedWidgets)
+            copyto.addAction(
+                _("'%s' siblings") % wtype,
+                self.actionCopyTypedSiblings)
+            copyto.addAction(
+                _("'%s' widgets called '%s'") % (wtype, name),
+                self.actionCopyTypedNamedWidgets)
+            copyto.addSeparator()
+            self.addCopyToWidgets(copyto)
+
+            popup.addAction(
+                _('Use as default style'),
+                self.actionSetStyleSheet)
 
         # special actions for references
         if self.setting.isReference():
             popup.addSeparator()
-            popup.addAction(_('Unlink setting'), self.actionUnlinkSetting)
+            popup.addAction(
+                _('Unlink setting'),
+                self.actionUnlinkSetting)
 
         self.inmenu = True
         self.updateHighlight()
